1,786c1,762
< //
< // CIMClient.java
< //
< // The contents of this file are subject to the SNIA Public License Version 1.0
< // (the "License"); you may not use this file except in compliance with the
< // License. You may obtain a copy of the License at
< //
< //      http://www.snia.org/resources/openSource.html
< //
< // Software distributed under the License is distributed on an "AS IS" basis,
< // WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
< // License for the specific language governing rights and limitations
< // under the License.
< //
< // The Original Code is CIMClient.java
< //
< // The Initial Developer of the Original Code is David Simons.
< //
< // Contributor(s): Viktor Mihajlovski,Frank Piorko, Adrian Schuur
< //
< // This source uses the Solaris WBEM APIs version 2.2.
< // Synchronization will be maintained with the Solaris WBEM APIS,
< // which are being standardized through the Java Community Process.
< //
< //
< // $Id$
< //
< // $Log$
< // Revision 1.17  2001/07/22 19:53:05  davidsi
< // - From Adrian Schuur - Added partial indication support.
< //
< // Revision 1.16  2001/07/22 13:31:38  davidsi
< // - BUG FIX. The preCheck() modification which always set the object path
< //   to that used by the CIMClient caused compatibility problems whereby
< //   the same host/cimom was still being used but a new namespace within
< //   that cimom was required. Now only assign the namespace if it would
< //   otherwise not be set.
< //
< // Revision 1.15  2001/07/07 15:45:49  davidsi
< // - From Angela Lee. General clean up plus ensure that the preCheck()
< //   method sets the object path correctly.
< //
< // Revision 1.12  2001/01/07 16:45:26  davidsi
< // - Added query support.
< //
< // Revision 1.11  2001/01/07 13:28:11  davidsi
< // - Fixes from Victor to support a common method for building the
< //   connection for the client.
< //
< // Revision 1.2  2000/12/20 12:48:41  mihajlov
< // Fixed protocol comparison.
< // Added initFromNameSpace to setup the correct CIMOMHandle depending on
< // the namespace, if no explicit protocol was supplied.
< //
< // Revision 1.1  2000/12/20 11:12:47  mihajlov
< // Initial revision
< //
< // Revision 1.9  2000/11/06 18:15:17  davidsi
< // - Reconciled error IDs.
< // - Port number is now extracted from the original URL rather than
< //   being assigned behind the scenes.
< //
< // Revision 1.8  2000/09/15 17:10:48  davidsi
< // - createInstance() now returns CIMObjectPath.
< //
< // Revision 1.7  2000/09/15 16:31:25  davidsi
< // - BUG FIX. From Heung-for Cheng. Fixed setting for default namespace.
< //   Modifications to preserve the URL.
< //
< // Revision 1.6  2000/08/16 05:59:53  shawn
< // merge P_TOMCAT31 branch
< //
< // Revision 1.5.8.1  2000/08/10 00:03:28  shawn
< // head merge
< //
< // Revision 1.5  2000/05/11 10:12:22  davidsi
< // - Fixed tabs.
< //
< // Revision 1.4  2000/05/09 20:15:36  davidsi
< // - Added support for extracting additional information from the name
< //   space when a URL is specified.
< //
< // Revision 1.3  2000/05/06 17:04:51  davidsi
< // - Brought inline with the SUN WBEM 2.2 APIs.
< //
< // Revision 1.2  2000/02/13 15:09:26  davidsi
< // - Added text to cover re-use of Solaris WBEM APIs.
< //
< // Revision 1.1.1.1  2000/02/06 16:41:04  davidsi
< // Initial version.
< //
< //
< //
< package org.snia.wbem.client;
< 
< import java.io.*;
< import java.util.Enumeration;
< import java.util.Vector;
< import java.util.Hashtable;
< import java.net.*;
< import java.text.MessageFormat;
< import org.snia.wbemcmd.xml.*;
< import org.snia.wbemcmd.cimom.CIMOMHandleXML;
< import org.snia.wbemcmd.cimom.CIMOMHandleRMI;
< import org.snia.wbemcmd.cimom.CIMOMHandleLocal;
< import org.snia.wbemcmd.cimom.CIMOMHandleSNIA;
< import org.snia.wbemcmd.cimom.CIMFeatures;
< import org.snia.wbem.cim.CIMException;
< import org.snia.wbem.cim.*;
< import org.snia.wbem.cim.CIMNameSpace;
< 
< public class CIMClient implements CIMOMHandle {
<     //
<     // This class brings together all the classes that can implement
<     // a CIM client and allows them to be selected via a flag (e.g.
<     // RMI, XML and LOCAL).
<     //
<     // A local connection means that the client has its own private
<     // object manager and is primarily intended for debugging
<     //
<     CIMOMHandleSNIA ch=null;    // The handle for the CIM object mgr
<     CIMNameSpace ns=null;   // requested name space
< 
<     public static final int WQL=0;
<     public static final int WQL1=1;
<     public static final int WQL2=2;
< 
<     public static final int RMI=1;          // selects RMI
<     public static final int XML=2;          // selects XML (deprecated?)
<     public static final int HTTP=2;         // selects XML/HTTP
<     public static final int LOCAL=3;        // selects LOCAL    - SUNDIFF
<     public static final boolean DEEP=true;  // As per the sun definition:
<                 // DEEP instance enumeration = the class instances and all
<                 //                          instances of its subclasses.
<                 // DEEP class enumeration = all subclasses but not
<                 //                          the class requested
<     public static final boolean SHALLOW=false; //
<                 // shallow instances = the instances of that class
<                 // shallow class returns the subclasses of the class
<     public static String DEFAULTNAMESPACE="\\root\\cimv2";
<     private static String GUEST_NAME="guest";
<     private static String GUEST_PASSWORD="guest";
< 
<     /** Cim Server URL Format message  **/
<     private final static String CIM_SERVER_URL_FMT = "http://{0}/cimom";
< 
<     /** Local Host name - used when user does not supply one. **/
<     private final static String s_localHostName;
< 
<     /** Local Host URL **/
<     private final static String s_localHostUrl;
< 
< 
<     //
<     // preCheck() ensures there is a handle in place. In the case
<     // of a object path being specified it also ensures that a
<     // namespace is assigned into the path. Note that the namespace
<     // isn't necessarily known by the user of the handle because it
<     // was specified when the CIMClient was created and the caller
<     // may not wish to track this.
<     //
<     private void preCheck() throws CIMException {
<         if (ch==null)
<             throw new CIMException(CIMException.CIM_ERR_FAILED,
<                                    "Not bound to a cimom");
<     }
< 
<     private void preCheck(CIMObjectPath opath) throws CIMException {
<         if (ch==null)
<             throw new CIMException(CIMException.CIM_ERR_FAILED,
<                                    "Not bound to a cimom");
<         // always use this CIMClient's host name
<         // use the namespace if one hasn't been set
<         String curNameSpace=opath.getNameSpace();
<         if (curNameSpace==null || curNameSpace.equals(CIMObjectPath.NOT_SET))
<             opath.setNameSpace(ns.getNameSpace());
<         opath.setHost(ns.getHost());
<     }
< 
< 
<     ///////////////////////////////////////////////////////////////////////////
<     // Static Initialization Block
<     ///////////////////////////////////////////////////////////////////////////
< 
<     /**
<         Initializes local host data (to be used by the default constructor).
<     **/
<     static {
<         // compiler complains if we assign the 'final' variable
<         // (i.e., s_localHostName) in both blocks.
<         String localHostName = null;
<         try {
<             // form server url string for the local host
<             localHostName = (InetAddress.getLocalHost()).getHostName();
<         } catch (Exception e) {
<             // not expecting to get an exception
<             e.printStackTrace();
<             //????? Does it work?  Should we use "128.0.0.1"?
<             localHostName = "localhost";
<         }
< 
<         s_localHostName = localHostName;
< 
<         Object[] args = new Object[1];
<         args[0] = s_localHostName;
<         s_localHostUrl = MessageFormat.format(CIM_SERVER_URL_FMT, args);
<     }
< 
< 
<     ///////////////////////////////////////////////////////////////////////////
<     // Constructors
<     //
<     //  [NOTE]: Per Jim Davis
<     //  (mail on 12/19/2000, "Re: SNIA-CIMOM: CIMClient Bug/Consistency"),
<     //  1. JSR-000048 will change the default CIMOM protocol type to HTTP.
<     //     (Sun WBEM SDK 2.2 javadoc specifies RMI).
<     //  2. CIMClient(CIMNameSpace, string, string, string, string) will honor
<     //     both RMI & HTTP protocols as specified by CIMNameSpace data.
<     //     (Sun WBEM SDK 2.2 javadoc states "This connection uses the RMI
<     //    protocol ...").
<     ///////////////////////////////////////////////////////////////////////////
< 
<     //
<     // CIMClient
<     //
<     // Builds a connection to a CIMOM on the local system (note that
<     // this isn't the same as LOCAL, it does require a CIMOM to be
<     // running on the local system). All defaults apply and RMI
<     // is used for transport.
<     //
<     public CIMClient() throws CIMException {
<         // use local host, default namespace, GUEST account,
<         // and HTTP CIMOM protocol
<         this(new CIMNameSpace(s_localHostUrl, DEFAULTNAMESPACE));
<         }
< 
< 
<     //
<     // CIMClient
<     //
<     // Builds a connection to the specified host and namespace using
<     // XML with anon access
<     //
<     public CIMClient(CIMNameSpace name)
<                 throws CIMException {
<         // use GUEST account, and HTTP CIMOM protocol
<         this(name, GUEST_NAME, GUEST_PASSWORD);
<     }
< 
<     //
<     // CIMClient
<     //
<     // Builds a connection to the specified host and namespace using
<     // XML with the specified username and password
<     //
<     public CIMClient(CIMNameSpace name, String userName,String pword)
<                 throws CIMException {
<         // use HTTP CIMOM protocol
<         this(name, userName, pword, HTTP);
<     }
< 
< 
< 
<     //
<     // CIMClient
<     //
<     // Builds a connection to the specified host and namespace using
<     // the specified protocol, username and password
<     //
<     public CIMClient(CIMNameSpace name,String userName,
<                      String pword,int type)
<             throws CIMException{
<         if (name.getNameSpace()!=null)
<             ns=new CIMNameSpace(name.getHostURL().toString(),
<                                 name.getNameSpace());
< 
<         if ((type == HTTP) || (type==XML))
<             ch=new CIMOMHandleXML(name);
<         else if (type==RMI) {
<             ch=new CIMOMHandleRMI(name);
<             ((CIMOMHandleRMI)ch).client=this;
<         }
<         else if (type==LOCAL)
<             ch=new CIMOMHandleLocal();
<         else
<             throw new CIMException("CIMOM protocol type not supported");
< 
<         if (userName==null || pword==null) {
<                 userName = GUEST_NAME;
<                 pword = GUEST_PASSWORD;
<         }
<         ch.authenticate(userName,pword);
<     }
< 
<     //
<     // CIMClient
<     //
<     // SUNDIFF: Note that roles are not supported yet
<     //
<     public CIMClient(CIMNameSpace name,String userName,
<                      String pword,
<                      String roleName,
<                      String rolePwd) throws CIMException {
<         // use HTTP CIMOM protocol
<         this(name, userName, pword, roleName, rolePwd, HTTP);
<     }
< 
<     //
<     // CIMClient
<     //
<     // SUNDIFF: Note that roles are not supported yet
<     //
<     public CIMClient(CIMNameSpace name,String userName,
<                      String pword,
<                      String roleName,
<                      String rolePwd,
<                      int type) throws CIMException {
<         // ignore roles for now
<         this(name, userName, pword, type);
<     }
< 
<     //
<     // createNameSpace
<     //
<     // Creates a CIM namespace (note that this is a pass-through
<     // function onto the handle).
<     //
<     public void createNameSpace(CIMNameSpace ins) throws CIMException{
<         preCheck();
<         ch.createNameSpace(ins);
<     }
< 
< 
<     //
<     // close
<     //
<     // Closes the CIMOM connection
<     //
<     public void close() throws CIMException {
<         ch.close();
<         ch=null;
<         ns=null;
<     }
< 
<     //
<     // deleteNameSpace
<     //
<     // Deletes a namespace
<     //
<     public void deleteNameSpace(CIMNameSpace cns) throws CIMException{
<         preCheck();
<         ch.deleteNameSpace(cns);
<     }
< 
< 
<     //
<     // deleteClass
<     //
<     // Deletes a class
<     //
<     public void deleteClass(CIMObjectPath path) throws CIMException{
<         preCheck(path);
<         ch.deleteClass(path);
<     }
< 
< 
<     //
<     // deleteInstance
<     //
<     // Deletes an instance
<     //
<     public void deleteInstance(CIMObjectPath path) throws CIMException{
<         preCheck(path);
<         ch.deleteInstance(path);
<     }
< 
< 
< 
<     //
<     // deleteQualifierType
<     //
<     // Deletes a qualfier
<     //
<     public void deleteQualifierType(CIMObjectPath path) throws CIMException{
<         preCheck(path);
<         ch.deleteQualifierType(path);
<     }
< 
< 
< 
<     //enumNameSpace
<     //
<     // Gets the namespace list for namespaces within the specified
<     // path.
<     //
<     public Enumeration enumNameSpace(CIMObjectPath path,boolean deep)
<                 throws CIMException {
<         Enumeration en;
< 
<         preCheck(path);
<         en=ch.enumNameSpace(path,deep);
<         return(en);
<     }
< 
< 
<     // enumClass
<     //
<     // Enumerates a class (with all properties).
<     //
<     // If local is true only those properties in the class definition
<     // (not inherited ones) will be returned.
<     // If deep is requested all derived classes are returned, else
<     // just the children of the specified class are returned.
<     public Enumeration enumClass(CIMObjectPath path,
<                    boolean deep,
<                    boolean local) throws CIMException {
<         Enumeration en;
< 
<         preCheck(path);
<         en=ch.enumClass(path,deep,local);
<         return(en);
<     }
< 
< 
<     //
<     // enumClass
<     //
<     // Enumerates a class (only names)
<     //
<     // If deep is requested all derived classes are returned, else
<     // just the children of the specified class are returned.
<     //
<     public Enumeration enumClass(CIMObjectPath path,
<                    boolean deep) throws CIMException {
<         Enumeration en;
< 
<         preCheck(path);
<         en=ch.enumClass(path,deep);
<         return(en);
<     }
< 
< 
< 
<     //
<     // enumInstances
<     //
<     // Returns the paths of all instances of a particular class
<     //
<     // If deep then the enumeration contains instances of all derived
<     // classes in-addition to those of the class.
<     public Enumeration enumInstances(CIMObjectPath path,
<                        boolean deep) throws CIMException{
<         preCheck(path);
<         return(ch.enumInstances(path,deep));
<     }
< 
< 
< 
<     // enumInstances
<     //
<     // Returns the instances of a class including properties
<     //
<     // If deep then the enumeration contains instances of all derived
<     // classes in-addition to those of the class.
<     // If local then only non-inherited properies are returned
<     //
<     public Enumeration enumInstances(CIMObjectPath path,
<                        boolean deep,
<                        boolean localOnly) throws CIMException{
<         preCheck(path);
<         return(ch.enumInstances(path,deep,localOnly));
<     }
< 
< 
<     //
<     // enumQualifierTypes
<     //
<     // Returns the qualifiers from a namespace
<     //
<     public Enumeration enumQualifierTypes(CIMObjectPath path)
<                 throws CIMException{
<         Enumeration en;
< 
<         preCheck(path);
<         en=ch.enumQualifierTypes(path);
<         return(en);
<     }
< 
< 
< 
< 
<     //getClass
<     //
<     // Gets a class given by an object path
<     //
<     // If local is true then non-inherited properties and methods
<     // are returned
<     //
<     public CIMClass getClass(CIMObjectPath path,
<                boolean localOnly) throws CIMException{
<         preCheck(path);
<         return(ch.getClass(path,localOnly));
<     }
< 
< 
< 
<     // getInstance
<     //
<     // Gets an instance for the specified object path.
<     //
<     // If local, only non-inherited properties are returned
<     //
<     public CIMInstance getInstance(CIMObjectPath path,
<                      boolean localOnly) throws CIMException {
<         CIMInstance instance;
< 
<         preCheck(path);
<         instance=ch.getInstance(path,localOnly);
<         return(instance);
<     }
< 
< 
<     //
<     // invokeMethod
<     //
<     // Executes a method in an object.
<     //
<     // @param path The path of the object on which the method is being invoked
<     //             this can be a class or instance
<     // @param inParams A vector of CIMProperty for inbound values
<     // @param outParams A vector of CIMProperty for outbound values
<     // @returns
<     //  A CIMValue or a null return is issued for all methods that
<     //  return nothing.
<     // @throws CIMException
<     //
<     public CIMValue invokeMethod(CIMObjectPath path,
<                    String methodName,
<                    Vector inParams,
<                    Vector outParams) throws CIMException{
<         preCheck(path);
<         return(ch.invokeMethod(path,methodName,inParams,outParams));
<     }
< 
< 
< 
<     // getQualifierType
<     //
<     // Returns a qualifiertype
<     //
<     public CIMQualifierType getQualifierType(CIMObjectPath path)
<             throws CIMException{
<         preCheck(path);
<         return(ch.getQualifierType(path));
<     }
< 
< 
< 
<     // createQualifierType
<     //
<     // Creates a qualifier type
<     //
<     public void createQualifierType(CIMObjectPath path,
<                       CIMQualifierType qt) throws CIMException{
<         preCheck(path);
<         ch.createQualifierType(path,qt);
<     }
< 
< 
< 
< 
<     // setQualifierType
<     //
<     // Modifies a qualifier type.
<     //
<     public void setQualifierType(CIMObjectPath path,
<                    CIMQualifierType qt) throws CIMException{
<         preCheck(path);
<         ch.setQualifierType(path,qt);
<     }
< 
< 
<     // createClass
<     //
<     // Adds a CIM class to a namespace
<     //
<     public void createClass(CIMObjectPath path,
<               CIMClass cc) throws CIMException {
<         preCheck(path);
<         ch.createClass(path,cc);
<     }
< 
< 
< 
<     // setClass
<     //
<     // Modifies a class in a namespace.
<     //
<     public void setClass(CIMObjectPath path,
<            CIMClass cc) throws CIMException {
<         preCheck(path);
<         ch.setClass(path,cc);
<     }
< 
< 
<     //
<     // createInstance
<     //
<     // creates an instance
<     //
<     public CIMObjectPath createInstance(CIMObjectPath path,
<                  CIMInstance ci) throws CIMException{
<         preCheck(path);
<         return(ch.createInstance(path,ci));
<     }
< 
< 
<     //
<     // setInstance
<     //
<     // Modifies an instance
<     //
<     public void setInstance(CIMObjectPath path,
<               CIMInstance ci) throws CIMException{
<         preCheck(path);
<         ch.setInstance(path,ci);
<     }
< 
< 
< 
<     // getProperty
<     //
<     // Gets a property value from an instance
<     //
<     public CIMValue getProperty(CIMObjectPath path,
<                   String propertyName) throws CIMException{
<         preCheck(path);
<         return(ch.getProperty(path,propertyName));
<     }
< 
< 
<     // setProperty
<     //
<     // Sets a property value
<     //
<     public void setProperty(CIMObjectPath path,
<               String propertyName,
<               CIMValue newValue) throws CIMException{
<         preCheck(path);
<         ch.setProperty(path,propertyName,newValue);
<     }
< 
< 
< 
<     // encryptData
<     //
<     // Encrypts data
<     //
<     public String encryptData(String data) throws CIMException{
<         return(data);   // SUNDIFF - no encryption
<     }
< 
< 
< 
<     // execQuery
<     //
<     public Enumeration execQuery(CIMObjectPath cop,
<                                  String query,int ql) throws CIMException{
<         preCheck(cop);
<         return(ch.execQuery(cop,query,ql));
<     }
< 
< 
< 
<     public Enumeration associatorNames(CIMObjectPath path,String assocClass,
<                                         String resultClass,
<                                         String role,String resultRole)
<             throws CIMException{
<         preCheck(path);
<         return(ch.associatorNames(path,assocClass,resultClass,role,resultRole));
<     }
< 
<     public Enumeration associators(CIMObjectPath path,
<                                         String assocClass,
<                                         String resultClass,
<                                         String role,
<                                         String resultRole,
<                                         boolean includeQualifiers,
<                                         boolean includeClassOrigin,
<                                         String propertyList[])throws CIMException {
<             preCheck(path);
<             return(ch.associators(path,assocClass,resultClass,role,
<                                  resultRole,includeQualifiers,
<                                  includeClassOrigin,propertyList));
<     }
< 
<     public Enumeration referenceNames(CIMObjectPath path,
<                                         String resultClass,
<                                         String role)
<             throws CIMException{
<         preCheck(path);
<         return(ch.referenceNames(path,resultClass,role));
<     }
< 
<     public Enumeration references(CIMObjectPath path,
<                                         String resultClass,
<                                         String role,
<                                         boolean includeQualifiers,
<                                         boolean includeClassOrigin,
<                                         String propertyList[])
<                 throws CIMException {
<         preCheck(path);
<         return(ch.references(path,resultClass,role,includeQualifiers,
<                 includeClassOrigin,propertyList));
<     }
< 
< 
<    // vvvvv (start) experimental indication support from Adrian Schuur  vvvvvv
< 
<     Vector listeners=new Vector();
<     Vector ports=new Vector();
< 
<     Vector getListeners() {  // used by IndReceiver
<         return listeners;
<     }
< 
<     public void addCIMListener(CIMListener l) throws CIMException
<     {
<         addCIMListener(0,l);
<     }
< 
<     public void addCIMListener(int port, CIMListener l) throws CIMException
<     {
<         for (int i=0; i<listeners.size(); i++) {
<             if (((CIMListener)(listeners.elementAt(i)))==l)
<                 throw new CIMException("Duplicate Listener defined");
<         }
< 
<         if (l instanceof CIMHTTPListener) {
<            Integer ip=new Integer(port);
<            for (int i=0; i<ports.size(); i++) {
<               if (((Integer)(ports.elementAt(i)))==ip)
<                  throw new CIMException("Duplicate HTPP Listener port defined");
<            }
< 
<            CIMHTTPListener chl=(CIMHTTPListener)l;
<            IndicationReceiverXMLHTTP rcv=
<               chl.prepareHTTPReceiver(this,port);
<            ports.addElement(new Integer(chl.getLocalPort()));
<            rcv.start();
<         }
<         else if (l instanceof CIMRMIListener) {
<            CIMRMIListener chl=(CIMRMIListener)l;
<            IndicationReceiverRMI rcv=
<               chl.prepareRMIReceiver(this);
<         }
<         listeners.addElement(l);
<     }
< 
<     public void remCIMListener(CIMListener l) throws CIMException
<     {
<         for (int i=0; i<=listeners.size(); i++) {
<             if (((CIMListener)(listeners.elementAt(i)))==l)
<                 listeners.removeElementAt(i);
<         }
<         throw new CIMException("Duplicate Listener defined");
<     }
< 
<     // ^^^^^^ (end) experimental indication support   ^^^^^^
< 
< 
<     //
<     // NOT SUN
<     //
<     public CIMFeatures getServerFeatures() throws CIMException
<     {
<         return(ch.getServerFeatures());
<     }
<     public void authenticate(String userName,String password) throws CIMException {
<         // dummy function
<     }
<     public CIMNameSpace getNameSpace() {
<        return ns;
<     }
< 
< }
---
> /*
> static char sccsid[] = "@(#)68  1.2  src/com/ibm/cim/CIMClient.java, cimom, ICAT1, ICAT1_1.1.0.0 11/25/02 07:54:40";
> */
> 
> //
> // Filename: CIMClient.java
> //
> // Description: CIM Client API
> //
> // Change History:
> //
> //  Modifier            Date                Change ID
> // ------------------------------------------------------------------------------
> // Martine Wedlake              11-22-2002
> //  Fixed defect 88 -- CIMClient should throw finer-grained exceptions
> //-------------------------------------------------------------------------------
> //
> // Original header below...
> //
> // CIMClient.java
> //
> // The contents of this file are subject to the SNIA Public License Version 1.0
> // (the "License"); you may not use this file except in compliance with the
> // License. You may obtain a copy of the License at
> //
> //      http://www.snia.org/resources/openSource.html
> //
> // Software distributed under the License is distributed on an "AS IS" basis,
> // WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
> // License for the specific language governing rights and limitations
> // under the License.
> //
> // The Original Code is CIMClient.java
> //
> // The Initial Developer of the Original Code is David Simons.
> //
> // Contributor(s): Viktor Mihajlovski,Frank Piorko, Adrian Schuur
> //
> // This source uses the Solaris WBEM APIs version 2.2.
> // Synchronization will be maintained with the Solaris WBEM APIS,
> // which are being standardized through the Java Community Process.
> //
> //
> // $Id$
> //
> // $Log$
> // Revision 1.17  2001/07/22 19:53:05  davidsi
> // - From Adrian Schuur - Added partial indication support.
> //
> // Revision 1.16  2001/07/22 13:31:38  davidsi
> // - BUG FIX. The preCheck() modification which always set the object path
> //   to that used by the CIMClient caused compatibility problems whereby
> //   the same host/cimom was still being used but a new namespace within
> //   that cimom was required. Now only assign the namespace if it would
> //   otherwise not be set.
> //
> // Revision 1.15  2001/07/07 15:45:49  davidsi
> // - From Angela Lee. General clean up plus ensure that the preCheck()
> //   method sets the object path correctly.
> //
> // Revision 1.12  2001/01/07 16:45:26  davidsi
> // - Added query support.
> //
> // Revision 1.11  2001/01/07 13:28:11  davidsi
> // - Fixes from Victor to support a common method for building the
> //   connection for the client.
> //
> // Revision 1.2  2000/12/20 12:48:41  mihajlov
> // Fixed protocol comparison.
> // Added initFromNameSpace to setup the correct CIMOMHandle depending on
> // the namespace, if no explicit protocol was supplied.
> //
> // Revision 1.1  2000/12/20 11:12:47  mihajlov
> // Initial revision
> //
> // Revision 1.9  2000/11/06 18:15:17  davidsi
> // - Reconciled error IDs.
> // - Port number is now extracted from the original URL rather than
> //   being assigned behind the scenes.
> //
> // Revision 1.8  2000/09/15 17:10:48  davidsi
> // - createInstance() now returns CIMObjectPath.
> //
> // Revision 1.7  2000/09/15 16:31:25  davidsi
> // - BUG FIX. From Heung-for Cheng. Fixed setting for default namespace.
> //   Modifications to preserve the URL.
> //
> // Revision 1.6  2000/08/16 05:59:53  shawn
> // merge P_TOMCAT31 branch
> //
> // Revision 1.5.8.1  2000/08/10 00:03:28  shawn
> // head merge
> //
> // Revision 1.5  2000/05/11 10:12:22  davidsi
> // - Fixed tabs.
> //
> // Revision 1.4  2000/05/09 20:15:36  davidsi
> // - Added support for extracting additional information from the name
> //   space when a URL is specified.
> //
> // Revision 1.3  2000/05/06 17:04:51  davidsi
> // - Brought inline with the SUN WBEM 2.2 APIs.
> //
> // Revision 1.2  2000/02/13 15:09:26  davidsi
> // - Added text to cover re-use of Solaris WBEM APIs.
> //
> // Revision 1.1.1.1  2000/02/06 16:41:04  davidsi
> // Initial version.
> //
> //
> //
> package com.ibm.cim;
> 
> import java.io.*;
> import java.util.Enumeration;
> import java.util.Vector;
> import java.util.Hashtable;
> import java.net.*;
> import java.text.MessageFormat;
> import com.ibm.xml.*;
> import com.ibm.cimom.client.*;
> 
> public class CIMClient {
> 
>     CIMOMHandleXML ch=null;    // The handle for the CIM object mgr
>     CIMNameSpace ns=null;      // requested name space
> 
>     public static final boolean DEEP=true;  // As per the sun definition:
>                 // DEEP instance enumeration = the class instances and all
>                 //                          instances of its subclasses.
>                 // DEEP class enumeration = all subclasses but not
>                 //                          the class requested
> 
>     public static final boolean SHALLOW=false; //
>                 // shallow instances = the instances of that class
>                 // shallow class returns the subclasses of the class
>     public boolean closeConnection = false;
> 
>     public static final int WQL=0;
>     public static final int WQL1=1;
>     public static final int WQL2=2;
> 
> 
>     //
>     // preCheck() ensures there is a handle in place. In the case
>     // of a object path being specified it also ensures that a
>     // namespace is assigned into the path. Note that the namespace
>     // isn't necessarily known by the user of the handle because it
>     // was specified when the CIMClient was created and the caller
>     // may not wish to track this.
>     //
>     private void preCheck() throws CIMException {
>         if (ch==null) {
>             throw new CIMException(CIMException.CIM_ERR_FAILED,
>                                    "Not bound to a cimom");
> 	   }
>     }
> 
>     private void preCheck(CIMObjectPath opath) throws CIMException {
>         if (ch==null) {
>             throw new CIMException(CIMException.CIM_ERR_FAILED,
>                                    "Not bound to a cimom");
> 	  }
> 
>         // always use this CIMClient's host name
>         // use the namespace if one hasn't been set
>         String curNameSpace = opath.getNameSpace();
>         if (curNameSpace==null || curNameSpace.equals(CIMObjectPath.NOT_SET))
>             opath.setNameSpace(ns.getNameSpace());
>         opath.setHost(ns.getHost());
>     }
>   
>     //
>     // CIMClient
>     //
>     // Builds a connection to the specified host and namespace using
>     // XML with the specified username and password
>     //
>     public CIMClient(CIMNameSpace name, String userName, String pword)
>                 throws CIMException {
>         if (name != null && name.getNameSpace() != null) {
> 		ns = name;
> 		ch=new CIMOMHandleXML(ns, userName, pword);
>         } else {
>             throw new CIMException(CIMException.CIM_ERR_INVALID_NAMESPACE);
>         }
>     }
> 
>     public CIMClient() {
> 	ch = new CIMOMHandleXML();
> 	ns = null;
>     }
>  
>     //
>     // createNameSpace
>     //
>     // Creates a CIM namespace (note that this is a pass-through
>     // function onto the handle).
>     //
>     public void createNameSpace(CIMNameSpace ins) throws CIMException{
>         preCheck();
>         ch.createNameSpace(ins);
> 	  if (closeConnection == true) {
> 		close();
> 	  }
>     }
> 
> 
>     //
>     // close
>     //
>     // Closes the CIMOM connection
>     //
>     public void close() {
> 
> 	  try {
> 	        ch.close();
> 	  } catch (Exception e) {}
> 
>         ch=null;
>         ns=null;
>     }
> 
>     //
>     // deleteNameSpace
>     //
>     // Deletes a namespace
>     //
>     public void deleteNameSpace(CIMNameSpace cns) throws CIMException{
>         preCheck();
>         ch.deleteNameSpace(cns);
> 	  if (closeConnection == true) {
> 		close();
> 	  }
>     }
> 
> 
>     //
>     // deleteClass
>     //
>     // Deletes a class
>     //
>     public void deleteClass(CIMObjectPath path) throws CIMException{
>         preCheck(path);
>         ch.deleteClass(path);
> 	  if (closeConnection == true) {
> 		close();
> 	  }
>     }
> 
> 
>     //
>     // deleteInstance
>     //
>     // Deletes an instance
>     //
>     public void deleteInstance(CIMObjectPath path) throws CIMException{
>         preCheck(path);
>         ch.deleteInstance(path);
> 	  if (closeConnection == true) {
> 		close();
> 	  }
>     }
> 
> 
> 
>     //
>     // deleteQualifierType
>     //
>     // Deletes a qualfier
>     //
>     public void deleteQualifierType(CIMObjectPath path) throws CIMException{
>         preCheck(path);
>         ch.deleteQualifierType(path);
> 	  if (closeConnection == true) {
> 		close();
> 	  }
>     }
> 
> 
> 
>     //enumNameSpace
>     //
>     // Gets the namespace list for namespaces within the specified
>     // path.
>     //
>     public Enumeration enumNameSpace(CIMObjectPath path,boolean deep)
>                 throws CIMException {
>         Enumeration en;
> 
>         preCheck(path);
>         en=ch.enumNameSpace(path,deep);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
>         return(en);
>     }
> 
> 
>     // enumClass
>     //
>     // Enumerates a class (with all properties).
>     //
>     // If local is true only those properties in the class definition
>     // (not inherited ones) will be returned.
>     // If deep is requested all derived classes are returned, else
>     // just the children of the specified class are returned.
>     public Enumeration enumClass(CIMObjectPath path,
>                    boolean deep,
>                    boolean local) throws CIMException {
>         Enumeration en;
> 
>         preCheck(path);
>         en=ch.enumClass(path,deep,local);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
>         return(en);
>     }
> 
>     //
>     // enumClass
>     //
>     // Enumerates a class (only names)
>     //
>     // If deep is requested all derived classes are returned, else
>     // just the children of the specified class are returned.
>     //
>     public Enumeration enumClass(CIMObjectPath path,
>                    boolean deep) throws CIMException {
>         Enumeration en;
> 
>         preCheck(path);
>         en=ch.enumClass(path, deep);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
>         return(en);
>     }
> 
>     //
>     // enumInstances
>     //
>     // Returns the paths of all instances of a particular class
>     //
>     // If deep then the enumeration contains instances of all derived
>     // classes in-addition to those of the class.
>     public Enumeration enumInstances(CIMObjectPath path,
>                        boolean deep) throws CIMException{
>         preCheck(path);
> 
> 	  Enumeration en = ch.enumInstances(path, deep);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
>         return en;
>     }
> 
> 
> 
>     // enumInstances
>     //
>     // Returns the instances of a class including properties
>     //
>     // If deep then the enumeration contains instances of all derived
>     // classes in-addition to those of the class.
>     // If local then only non-inherited properies are returned
>     //
>     public Enumeration enumInstances(CIMObjectPath path,
>                        boolean deep,
>                        boolean localOnly) throws CIMException{
>         preCheck(path);
> 
> 	  Enumeration en = ch.enumInstances(path, deep, localOnly);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
> 	  return en;
>     }
> 
> 
>     //
>     // enumQualifierTypes
>     //
>     // Returns the qualifiers from a namespace
>     //
>     public Enumeration enumQualifierTypes(CIMObjectPath path)
>                 throws CIMException{
>         Enumeration en;
> 
>         preCheck(path);
>         en=ch.enumQualifierTypes(path);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
>         return(en);
>     }
> 
>     //getClass
>     //
>     // Gets a class given by an object path
>     //
>     // If local is true then non-inherited properties and methods
>     // are returned
>     //
>     public CIMClass getClass(CIMObjectPath path,
>                boolean localOnly) throws CIMException{
>         preCheck(path);
> 
> 	  CIMClass cc = ch.getClass(path, localOnly);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
> 	  return cc;
>     }
> 
> 
> 
>     // getInstance
>     //
>     // Gets an instance for the specified object path.
>     //
>     // If local, only non-inherited properties are returned
>     //
>     public CIMInstance getInstance(CIMObjectPath path,
>                      boolean localOnly) throws CIMException {
>         CIMInstance instance;
> 
>         preCheck(path);
>         instance=ch.getInstance(path,localOnly);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
>         return(instance);
>     }
> 
> 
>     //
>     // invokeMethod
>     //
>     // Executes a method in an object.
>     //
>     // @param path The path of the object on which the method is being invoked
>     //             this can be a class or instance
>     // @param inParams A vector of CIMProperty for inbound values
>     // @param outParams A vector of CIMProperty for outbound values
>     // @returns
>     //  A CIMValue or a null return is issued for all methods that
>     //  return nothing.
>     // @throws CIMException
>     //
>     public CIMValue invokeMethod(CIMObjectPath path,
>                    String methodName,
>                    Vector inParams,
>                    Vector outParams) throws CIMException{
>         preCheck(path);
> 
> 	  CIMValue cv = ch.invokeMethod(path, methodName, inParams, outParams);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
> 	  return cv;
>     }
> 
> 
> 
>     // getQualifierType
>     //
>     // Returns a qualifiertype
>     //
>     public CIMQualifierType getQualifierType(CIMObjectPath path)
>             throws CIMException{
>         preCheck(path);
> 
> 	  CIMQualifierType cqt = ch.getQualifierType(path);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
> 	  return cqt;
>     }
> 
> 
> 
>     // createQualifierType
>     //
>     // Creates a qualifier type
>     //
>     public void createQualifierType(CIMObjectPath path,
>                       CIMQualifierType qt) throws CIMException{
>         preCheck(path);
>         ch.createQualifierType(path,qt);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
>     }
> 
> 
> 
> 
>     // setQualifierType
>     //
>     // Modifies a qualifier type.
>     //
>     public void setQualifierType(CIMObjectPath path,
>                    CIMQualifierType qt) throws CIMException{
>         preCheck(path);
>         ch.setQualifierType(path,qt);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
>     }
> 
> 
>     // createClass
>     //
>     // Adds a CIM class to a namespace
>     //
>     public void createClass(CIMObjectPath path,
>               CIMClass cc) throws CIMException {
>         preCheck(path);
>         ch.createClass(path,cc);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
>     }
> 
> 
> 
>     // setClass
>     //
>     // Modifies a class in a namespace.
>     //
>     public void setClass(CIMObjectPath path,
>            CIMClass cc) throws CIMException {
>         preCheck(path);
>         ch.setClass(path,cc);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
>     }
> 
> 
>     //
>     // createInstance
>     //
>     // creates an instance
>     //
>     public CIMObjectPath createInstance(CIMObjectPath path,
>                  CIMInstance ci) throws CIMException{
>         preCheck(path);
> 
> 	  CIMObjectPath cop = ch.createInstance(path, ci);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
> 	  return cop;
>     }
> 
> 
>     //
>     // setInstance
>     //
>     // Modifies an instance
>     //
>     public void setInstance(CIMObjectPath path,
>               CIMInstance ci) throws CIMException{
>         preCheck(path);
>         ch.setInstance(path,ci);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
>     }
> 
> 
> 
>     // getProperty
>     //
>     // Gets a property value from an instance
>     //
>     public CIMValue getProperty(CIMObjectPath path,
>                   String propertyName) throws CIMException{
>         preCheck(path);
> 
> 	  CIMValue cv = ch.getProperty(path, propertyName);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
> 	  return cv;
>     }
> 
> 
>     // setProperty
>     //
>     // Sets a property value
>     //
>     public void setProperty(CIMObjectPath path,
>               String propertyName,
>               CIMValue newValue) throws CIMException{
>         preCheck(path);
>         ch.setProperty(path,propertyName,newValue);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
>     }
> 
>     // execQuery
>     //
>     public Enumeration execQuery(CIMObjectPath cop,
>                                  String query,int ql) throws CIMException{
>         preCheck(cop);
> 	
>   	  Enumeration en = ch.execQuery(cop,query,ql);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
> 	  return en;
>     }
> 
> 
> 
>     public Enumeration associatorNames(CIMObjectPath path,String assocClass,
>                                         String resultClass,
>                                         String role,String resultRole)
>             throws CIMException{
>         preCheck(path);
> 
> 	  Enumeration en = ch.associatorNames(path,assocClass,resultClass,role,resultRole);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
> 	  return en;
>     }
> 
>     public Enumeration associators(CIMObjectPath path,
>                                         String assocClass,
>                                         String resultClass,
>                                         String role,
>                                         String resultRole,
>                                         boolean includeQualifiers,
>                                         boolean includeClassOrigin,
>                                         String propertyList[])throws CIMException {
>             preCheck(path);
> 
> 		Enumeration en = ch.associators(path,assocClass,resultClass,role,
>                                  resultRole,includeQualifiers,
>                                  includeClassOrigin,propertyList);
> 
> 	
> 	      if (closeConnection == true) {
> 			close();
> 	      }
> 
> 		return en;
>     }
> 
>     public Enumeration referenceNames(CIMObjectPath path,
>                                         String resultClass,
>                                         String role)
>             throws CIMException{
>         preCheck(path);
> 
> 	  Enumeration en = ch.referenceNames(path,resultClass,role);
> 
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
> 	  return en;
>     }
> 
>     public Enumeration references(CIMObjectPath path,
>                                         String resultClass,
>                                         String role,
>                                         boolean includeQualifiers,
>                                         boolean includeClassOrigin,
>                                         String propertyList[])
>                 throws CIMException {
>         preCheck(path);
> 
> 	  Enumeration en = ch.references(path,resultClass,role,includeQualifiers,
>                 includeClassOrigin,propertyList);
> 	   
> 	  if (closeConnection == true) {
> 		close();
> 	  }
> 
> 	  return en;
>     }
> 
>     public CIMNameSpace getNameSpace() {
>        return ns;
>     }
> 
>     public Vector getCIMOMServices() throws CIMException {
> 		preCheck();
> 		if (ch.output != null) {
> 			return ch.output.slp.findCIMOMSrvAndAttrs();
> 		}
> 		return null;
>     }
> 
>     public String getCIMOMOptions() throws CIMException {
> 		preCheck();
> 		if (ch.output != null) {
> 			try {
> 				return ch.output.getOptions();
> 			} catch (Exception e) {}
> 		}
> 		return null;
>     }
> 
>     public void addListener(CIMListener cl) throws CIMException {
> 		preCheck();
> 		if (ch.indications == false) {
> 			throw new CIMException(CIMException.CIM_ERR_INDICATION_NOT_COLLECTED);
> 		}
> 		ch.lmgr.addCIMListener(cl);
>     }
> 
>     public void removeListener(CIMListener cl) throws CIMException {
> 		preCheck();
> 		if (ch.indications == false) {
> 			throw new CIMException(CIMException.CIM_ERR_INDICATION_NOT_COLLECTED);
> 		}
> 		ch.lmgr.removeCIMListener(cl);
>     }
> 
> }
