1,825c1,769
< //
< // CIMObjectPath.java
< //
< // The contents of this file are subject to the SNIA Public License Version 1.0
< // (the "License"); you may not use this file except in compliance with the
< // License. You may obtain a copy of the License at
< //
< //      http://www.snia.org/resources/openSource.html
< //
< // Software distributed under the License is distributed on an "AS IS" basis,
< // WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
< // License for the specific language governing rights and limitations
< // under the License.
< //
< // The Original Code is CIMObjectPath.java
< //
< // The Initial Developer of the Original Code is David Simons.
< //
< // Contributor(s):
< //
< // This source uses the Solaris WBEM APIs version 2.2.
< // Synchronization will be maintained with the Solaris WBEM APIS,
< // which are being standardized through the Java Community Process.
< //
< // $Id$
< //
< // $Log$
< // Revision 1.16  2001/07/22 14:34:28  davidsi
< // - From Jean Lamontagne - using toString().hashCode() is potentially
< //   problematic because it assumes the keys will be correctly ordered.
< //   For the moment it is safer to rely on the objectName.hashCode().
< //
< // Revision 1.15  2001/07/22 13:29:57  davidsi
< // - Further updates from Angela.
< //
< // Revision 1.14  2001/07/07 16:29:23  davidsi
< // - Make the not set value less like to clash with a real value by adding
< //   non standard characters to it.
< //
< // Revision 1.13  2001/07/07 15:47:00  davidsi
< // - From Angela Lee. Significant reworking to ensure all values are
< //   correctly set - now rather than defaulting to null/"" use
< //   an explicit not set value.
< //
< // Revision 1.12  2001/07/07 13:41:45  davidsi
< // - From Daniel Abramovich, fixed equals() method for handling non
< //   instance cases.
< //
< // Revision 1.11  2001/06/23 13:57:20  davidsi
< // - Fixed hashCode for CIMObjectPath (Tatsuya Kainuma)
< //
< // Revision 1.10  2001/03/19 17:34:49  davidsi
< // - Key values are now compared as strings without reference to their
< //   types.
< //
< // Revision 1.9  2000/11/06 18:07:39  davidsi
< // - Fixed error IDs.
< // - Changes to suport the movement of the escaping functionality to
< //   CIMOMEscape.
< //
< // Revision 1.8  2000/08/16 05:59:52  shawn
< // merge P_TOMCAT31 branch
< //
< // Revision 1.7.6.1  2000/08/10 00:03:27  shawn
< // head merge
< //
< // Revision 1.7  2000/07/19 16:48:57  davidsi
< // - Revised escaping mechanisms for object paths.
< //
< // Revision 1.6  2000/05/19 14:43:12  davidsi
< // - Fixed the assumption that key components will always be in the same
< //   order when performing a comparison - they may not be.
< // - Convert key values to strings prior to comparing. In some (rare)
< //   cases the values may be stored as different types but be
< //   representing the same values (strictly one shouldn't need to
< //   do this but some providers can be inconsistent about how they
< //   type properties)
< //
< // Revision 1.5  2000/05/11 10:12:20  davidsi
< // - Fixed tabs.
< //
< // Revision 1.4  2000/05/06 17:04:22  davidsi
< // - Modifications to bring classes inline with the SUN WBEM APIs 2.2.
< //
< // Revision 1.3  2000/04/09 13:17:58  davidsi
< // - Switched to StringBuffer (from String) for XML generation.
< // - Switched to forcing a <KEYBINDING> tag - no longer parse
< //   singleton property values without a property name.
< //
< // Revision 1.2  2000/02/13 15:09:07  davidsi
< // - Added text to cover re-use of Solaris WBEM APIs.
< //
< // Revision 1.1.1.1  2000/02/06 16:41:03  davidsi
< // Initial version.
< //
< //
< //
< package org.snia.wbem.cim;
< 
< import java.util.Vector;
< import java.net.MalformedURLException;
< import java.io.Serializable;
< import java.util.StringTokenizer;
< import org.snia.wbemcmd.xml.*;
< import org.snia.wbemcmd.cimom.CIMOMEscape;
< //
< public class CIMObjectPath extends Object implements Serializable {
< 
<     ///////////////////////////////////////////////////////////////////////////
<     // Public Static Data
<     ///////////////////////////////////////////////////////////////////////////
< 
<     /** invalid value indicating the data has not been initialized by
<         the user who instantiated the object.  See comment block for
<         constructors below for details.                                 **/
<     public final static String NOT_SET = "$$Not$$Set$$";
< 
< 
<     ///////////////////////////////////////////////////////////////////////////
<     // Private Member Data
<     ///////////////////////////////////////////////////////////////////////////
< 
<     // The default case (where we lack the host/namespace/objectName
<     // information):
<     //
<     // 1. can not use the default CIMNameSpace instance since that is
<     //    specifically
<     //    for the local host environment.
<     // 2. Sun SDK (and therefore JSR-000048) behavior is to return an
<     //    empty string for getNameSpace() & getHost().
<     // 3. For now, we will return an invalid string for uninitialized values so
<     //    that all the callers can be identified and corrected.
<     //
<     CIMNameSpace nameSpace = new CIMNameSpace(CIMObjectPath.NOT_SET,
<                                               CIMObjectPath.NOT_SET);
<     String hostNameString = CIMObjectPath.NOT_SET;
<     String namespaceString = CIMObjectPath.NOT_SET;
<     String objectName = CIMObjectPath.NOT_SET;
<     Vector keys= new Vector();
< 
< 
<     ///////////////////////////////////////////////////////////////////////////
<     // Constructors
<     //
<     //  [NOTE]: JSR-000048 does not have a good interface to this object and
<     //  it allows its construction without valid host name & namespace data.
<     //  1. The caller is responsible for setting the correct host name and
<     //     namespace before using the constructed instance object.
<     //  2. Until that is done, the namespace data will contain NOT_SET strings
<     //     as the host name and namespace.  This of course is an invalid
<     //     situation and some operations will not work properly.
<     ///////////////////////////////////////////////////////////////////////////
< 
< 
<     // CIMObjectPath
<     public CIMObjectPath() {
<     }
< 
<     // CIMObjectPath
<     //
<     // Constructs an object path to access the specified object
<     public CIMObjectPath(String on) {
<         // If there is a namespace but it will be set via CIMClient on
<         // the next call
<         this(on, (String)null);
<     }
< 
<     // CIMObjectPath
<     //
<     // Constructs an object path to access the specified object (a
<     // qualifier or class) in a particular namespace
<     //
<     public CIMObjectPath(String on,String ns) {
<         if (on != null)
<             setObjectName(on);
<         if (ns != null)
<             setNameSpace(ns);
<     }
< 
< 
<     // CIMObjectPath
<     //
<     // Constructs a path for an instance. The vector should be
<     // the keys and the string a class name.
<     //
<     public CIMObjectPath(String iClassName,Vector v) {
<         // If there is a namespace but it will be set via CIMClient on
<         // the next call
<         setObjectName(iClassName);
<         keys=v;
<     }
< 
< 
<     //
<     // CIMObjectPath
<     //
<     // Create an instance using a XML definition
<     //
<     // TODO-SUN, its not clear which types of NODE the SUN API is intended
<     // to accept. This is a best guess!
<     //
<     // Note does not accept XML_ELEMENT_OBJECTPATH - caller must jump
<     // over such an element
<     //
<     // @param node The node containing an XML definition of CIMObjectPath
<     //
<     public CIMObjectPath(org.w3c.dom.Node inNode) {
<         XMLNode node=XMLNode.getXMLNode(inNode);
<         int token=node.getToken();
< 
<         try {
<             if (token==XMLNode.XML_ELEMENT_LOCALCLASSPATH
<                 || token==XMLNode.XML_ELEMENT_CLASSPATH) {
<                 boolean dolocal=false;
<                 if (node.getToken()!=XMLNode.XML_ELEMENT_LOCALCLASSPATH)
<                     dolocal=true;
< 
<                 XMLNode localNameNode=node.mustChildElementChild(XMLNode.XML_ELEMENT_NAMESPACEPATH);
<                 //
<                 // Get NameSpace path
<                 //
<                 if (!dolocal) {
<                     localNameNode.mustElement(XMLNode.XML_ELEMENT_HOST);
<                     setHost(localNameNode.getUnescapeText());
<                 }
<                 localNameNode=localNameNode.mustNextElement(XMLNode.XML_ELEMENT_LOCALNAMESPACEPATH);
<                 String readNS=XMLClass.getNameSpace(localNameNode.getChild());
<                 setNameSpace(readNS);
<                 node=node.mustNextElement(XMLNode.XML_ELEMENT_CLASSNAME);
<                 String className=node.mustGetAttribute("Name");
<                 setObjectName(className);
<                 return;
<             }
<             //
<             // Must be an instance path or an error...
<             //
<             boolean dolocal=false;
<             XMLNode pathNode;
< 
<             if (token==XMLNode.XML_ELEMENT_LOCALINSTANCEPATH) {
<                 dolocal=true;
<                 node=node.getChild();       // check for null below!
<                 pathNode=node;
<             } else if (token==XMLNode.XML_ELEMENT_INSTANCEPATH) {
<                 node=node.mustChildElement(XMLNode.XML_ELEMENT_NAMESPACEPATH);
<                 pathNode=node.getChild();
<                 pathNode.mustElement(XMLNode.XML_ELEMENT_HOST);
<                 setHost(pathNode.getUnescapeText());
<                 pathNode=pathNode.getNext();
<             } else
<                 throw new java.lang.InstantiationError(
<                                     "Required <INSTANCEPATH> or  <CLASSPATH>");
< 
<             //
<             // Make sure nodes are valid before continuing..
<             //
<             if (pathNode==null)
<                 throw new CIMException(CIMException.CIM_ERR_FAILED,
<                                     "Required <LOCALNAMESPACEPATH>");
<             if (node==null)
<                 throw new CIMException(CIMException.CIM_ERR_FAILED,
<                                     "Required <INSTANCENAME>");
<             //
<             // Get NameSpace path
<             //
<             pathNode.mustElement(XMLNode.XML_ELEMENT_LOCALNAMESPACEPATH);
<             setNameSpace(XMLClass.getNameSpace(pathNode.getChild()));
<             node=node.mustNextElement(XMLNode.XML_ELEMENT_INSTANCENAME);
< 
<             XMLClass.getInstanceName(node,this);
<         } catch (CIMException ce) {
<             throw new java.lang.InstantiationError(ce.toString());
<         }
<     }
< 
< 
<     // addKey
<     //
<     // Adds a further key to a path.
<     //
<     public void addKey(String keyname,CIMValue value) {
<         CIMProperty cp=new CIMProperty(keyname,value);
<         cp.setType(value.getType());
<         keys.addElement(cp);
<     }
< 
< 
< 
< 
<     // getKeys
<     //
<     // Returns the keys
<     //
<     public Vector getKeys() {
<         return(keys);
<     }
< 
<     // setKeys
<     //
<     // Sets the keys for this path (vector contains CIMProperty instances)
<     public void setKeys(Vector v) {
<         if (v!=null)
<             keys=v;
<         else
<             keys.clear();
<     }
< 
< 
< 
<     // getNameSpace
<     //
<     // Returns the current namespace
<     //
<     public String getNameSpace() {
<         return(namespaceString);
<     }
< 
<     //
<     // getHost
<     //
<     //
<     public String getHost() {
<         return(hostNameString);
<     }
< 
< 
<     // getObjectName
<     //
<     // Returns the object name for this path
<     //
<     public String getObjectName() {
<         return(objectName);
<     }
< 
< 
< 
<     /***
<      * parse
<      *
<      * Converts a string representation of an object path
<      *
<      * @param instanceName
<      *    The object path to convert - assumed to be to an instance
<      * @throws
<      *    CIMException an error occurred parsing the path
<      */
<     public void parse(String instanceName) throws CIMException {
<         char []values=instanceName.toCharArray();
<         int i;
<         int keystart=0;
<         String text;
<         int equalspos;
<         Vector tmpkeys=new Vector();
<         boolean singleKey=true;
<         boolean inquote=false;
<         String keyvalue;
<         boolean nstype=true;
<         boolean nshdl=false;
<         int classNamePos=0;         // starting position of classname
<         String nameSpaceType=null;  // namespace type
<         String nameSpaceHdl=null;   // handle for namespace
< 
<         singleKey=false;        // set this to TRUE if it is intended that
<                                 // a property value can be specified without
<                                 // its name when there is only one key.
< 
<         for(i=0;i<values.length;i++) {
<             //
<             // One can come across '.' in the namespace handle because
<             // it may be an IP address
<             //
<             if (values[i]=='.' && !nshdl)
<                 break;
<             if (values[i]==':') {
<                 if (nstype) {
<                     // found namespace type
<                     nameSpaceType=new String(values,0,i);
<                     classNamePos=i+1;
<                     nstype=false;
<                     nshdl=true;
<                 } else {
<                     // found namespace handle
<                     nameSpaceHdl=new String(values,classNamePos,i-classNamePos);
<                     classNamePos=i+1;
<                     nshdl=false;
<                 }
<             }
<         }
< 
<         //
<         // There must be a '.' in the instance name if it is valid
<         // as this delimits the class name
<         //
<         if (i==values.length)
<             throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
<                         "Instance name \""+instanceName+"\" did not have a '.' as a class delimeter");
< 
<         //
<         // If a namespace type was found then there must have also been
<         // a handle
<         //
<         if (!nstype && nshdl)
<             throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
<                         "Instance name \""+instanceName+"\" did not have a namespace handle");
< 
<         //
<         // Only http namespace types are supported
<         if (nameSpaceType!=null && !nameSpaceType.equalsIgnoreCase("HTTP"))
<             throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
<                         "Instance name \""+instanceName+"\" does not represent a HTTP namespace");
< 
< 
<         String className=new String(values,classNamePos,i-classNamePos);
< 
<         //
<         // Now extract keys
<         //
<         i++;
<         equalspos=0;
<         for(keystart=i;i<values.length;i++) {
<             char ch=values[i];
<                 //
<                 // Skip escaped characters
<                 //
<             if (i<values.length-1 && ch=='\\') {
<                 i++;
<                 continue;
<             }
<             //
<             // Check for quotes
<             //
<             if (ch=='\"')
<                 inquote = !inquote;
<             if (inquote)
<                 continue;
< 
<             if (ch=='=')
<                 equalspos=i+1;
<             if (ch==',' || (i+1==values.length) ) {
<                 if ((i+1==values.length) && equalspos==0 && singleKey) {
<                     //
<                     // This is the special case of when its classname.value
<                     //
<                     CIMProperty cp=new CIMProperty();
<                     cp.setName("test it");
<                     //
<                     // Generally there will be quotes but for integer values
<                     // they are not strictly necessary so check for them
<                     //
<                     if (values[keystart]!='\"')
<                         keyvalue=new String(values,keystart,i-keystart+1);
<                     else
<                         keyvalue=new String(values,keystart+1,i-keystart+1-2);
<                     cp.setValue(new CIMValue(keyvalue,
<                                             CIMDataType.getPredefinedType(
<                                             CIMDataType.STRING)
<                                             ));
<                     cp.setType(CIMDataType.getPredefinedType(
<                                         CIMDataType.STRING));
<                     tmpkeys.addElement(cp);
<                     break;
<                 }
< 
<                 if (i+1==values.length)
<                     i++;
< 
<                 if (equalspos==0) {
<                     throw new CIMException(CIMException.CIM_ERR_NOT_FOUND,
<                         "Bad key in string ("+instanceName+")");
<                 }
<                 String keyprop=new String(values,keystart,equalspos-keystart-1);
<                 //
<                 // Generally there will be quotes but for integer values
<                 // they are not strictly necessary so check for them
<                 //
<                 if (values[equalspos]!='\"')
<                     keyvalue=new String(values,equalspos,i-equalspos);
<                 else
<                     keyvalue=CIMOMEscape.unEscape(new String(values,equalspos+1,i-equalspos-2));
<                 i++;
<                 equalspos=0;
<                 keystart=i;
<                 CIMProperty cp=new CIMProperty();
<                 cp.setName(keyprop);
<                 cp.setValue(new CIMValue(keyvalue,
<                                 CIMDataType.getPredefinedType(
<                                 CIMDataType.STRING) ));
<                 cp.setType(CIMDataType.getPredefinedType(
<                                     CIMDataType.STRING));
<                 tmpkeys.addElement(cp);
<                 singleKey=false;
<             }
<         }
< 
< 
<         if (nameSpaceHdl!=null) {
<             String hostName=null;
<             String newNameSpace=null;
< 
<             if (!nameSpaceHdl.substring(0,2).equals("//"))
<                 throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
<                         "Instance name \""+instanceName+"\" does not specify a host");
<             StringTokenizer stok=new StringTokenizer(nameSpaceHdl,"/",false);
<             for (int tokenNumber=0;stok.hasMoreTokens();tokenNumber++) {
<                 String token=stok.nextToken();
<                 if (tokenNumber==0) {
<                     hostName=token;
<                 } else if (tokenNumber==1) {
<                     newNameSpace=token;
<                 } else {
<                     newNameSpace=newNameSpace.concat("/"+token);
<                 }
<             }
<             if (hostName==null)
<                 throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
<                         "Instance name \""+instanceName+"\" does not specify a host");
<             if (newNameSpace==null)
<                 throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
<                         "Instance name \""+instanceName+"\" does not specify a namespace");
<             setNameSpace(newNameSpace);
<             setHost(hostName);
<         } else
<             nameSpace= new CIMNameSpace(CIMObjectPath.NOT_SET,
<                                               CIMObjectPath.NOT_SET);
< 
<         //
<         // Now assign the values to this instance
<         //
<         objectName=className;
<         keys=tmpkeys;
<     }
< 
<     //
<     // setHost
<     //
<     //
<     public void setHost(String host) {
<         if (host!=null) {
<             hostNameString = host;
<             nameSpace.setHost(host);
<         }
<     }
< 
<     // setNameSpace
<     //
<     // Sets the namespace for the object
<     //
<     public void setNameSpace(String ns) {
<         if (ns!=null) {
<             namespaceString = ns;
<             nameSpace.setNameSpace(ns);
<         }
<     }
< 
< 
< 
<     /**
<      * Assigns the object name as a string
<      *
<      * @param op
<      *    The object name to assign
<      */
<     public void setObjectName(String objectName) {
<         this.objectName=objectName;
<     }
< 
<     // toString
<     public String toString() {
<         if ( !(keys.isEmpty()) ) {
<             String ret=objectName;
< 
<             for(int i=0;i<keys.size();i++) {
<                 CIMProperty cp=(CIMProperty)(keys.elementAt(i));
<                 if (i>0)
<                     ret=ret.concat(","+cp.getName()+"=\""+
<                                    CIMOMEscape.escape(cp.getValue().getValue().toString())+
<                                    "\"");
<                 else
<                     ret=ret.concat("."+cp.getName()+"=\""+
<                                    CIMOMEscape.escape(cp.getValue().getValue().toString())+
<                                    "\"");
<             }
<             return(ret);
<         }
<         return(objectName);
<     }
< 
< 
<     // hashCode
<     //
<     // Computes the hash code
<     //
<     public int hashCode() {
<         return objectName.hashCode();
<     }
< 
<     // equals
<     //
<     // Compares this object path with another object path
<     //
<     public boolean equals(Object o) {
<         if (!(o instanceof CIMObjectPath))
<             return(false);
<         CIMObjectPath cop=(CIMObjectPath)o;
< 
<         // not the same object name...
<         if (! objectName.equalsIgnoreCase(cop.objectName))
<             return(false);
<         // different namespace...
<         else if (! getNameSpace().equalsIgnoreCase(cop.getNameSpace()))
<             return(false);
<         // different host name...
<         else if (! getHost().equalsIgnoreCase(cop.getHost()))
<             return(false);
< 
<         // both have one or more keys
<         if (!keys.isEmpty() && !cop.keys.isEmpty()) {
<             //
<             // An instance path
<             //
< 
<             if (keys.size()!=cop.keys.size()) {
<                 return(false);
<             }
<             if (keys.size()==1) {
<                 //
<                 // Special case for a single, just use class name and
<                 // value
<                 //
<                 CIMProperty cp1=(CIMProperty)(keys.elementAt(0));
<                 CIMProperty cp2=(CIMProperty)(cop.keys.elementAt(0));
<                 CIMValue cv1 = cp1.getValue();
<                 CIMValue cv2 = cp2.getValue();
<                 if (cv1==null && cv2==null)
<                     return(true);
<                 if (cv1==null || cv2==null)
<                     return(false);
<                 return (cv1.toString().equals(cv2.toString()));
<             }
< 
<             int maxNoKeys=keys.size();
<             for(int i=0;i<maxNoKeys;i++) {
<                 CIMProperty cp1=(CIMProperty)(keys.elementAt(i));
< 
<                 //
<                 // In general keys will be in the same order, so
<                 // try the same index first...
<                 //
<                 CIMProperty cp2=(CIMProperty)(cop.keys.elementAt(i));
<                 if (!cp1.getName().equalsIgnoreCase(cp2.getName())) {
<                         //
<                         // No match so start searching for the
<                         // equivalent key
<                         //
<                         int j;
<                         for(j=0;j<maxNoKeys;j++) {
<                                 cp2=(CIMProperty)(cop.keys.elementAt(j));
<                                 if (cp1.getName().equalsIgnoreCase(
<                                                         cp2.getName()))
<                                         break;
<                         }
<                         //
<                         // Not found, failed to match
<                         //
<                         if (j==maxNoKeys)
<                                 return(false);
<                 }
<                 CIMValue cv1 = cp1.getValue();
<                 CIMValue cv2 = cp2.getValue();
<                 if (cv1==null && cv2==null)
<                     continue;
<                 if (cv1==null || cv2==null)
<                     return(false);
<                 if ( !(cv1.toString().equals(cv2.toString())))
<                     return(false);
<             }
< 
<             return(true);
<         } else if ( (keys.isEmpty() && !cop.keys.isEmpty()) ||
<                         (!keys.isEmpty() && cop.keys.isEmpty()) ) {
<             // only one has one or more keys
<             return(false);
<         } else {
<             // both are empty
<             return(true);
<         }
<     }
< 
<     public boolean equals(Object o,boolean ignoreClassOrigins) {
<         //
<         // SUNDIFF - unclear why classorigins would ever be checked!
<         //
<         return(equals(o));
<     }
< 
<     // toXml
<     public String toXml() throws MalformedURLException {
<         StringBuffer ret;
< 
<         //
<         // Instance path
<         //
<         ret=new StringBuffer("<INSTANCENAME CLASSNAME=\""+
<                          objectName+"\">");
< 
<         if (keys==null || keys.size()==0) {
<             //
<             // TODO-SUN
<             // This only supports writing instance paths, but does the
<             // sun implementation write class paths as well?
<             //
<             // support for query: empty objects, nothing to project (AS)
<             ret=ret.append("</INSTANCENAME>");
<             return(ret.toString());
<         //    throw new MalformedURLException("Bad path to object");
<         }
< 
< 
<         int numkeys=keys.size();
<             //
<             // If keys >1 then must use KEYBINDING - we also use it for
<             // the key==1 case - most implementations can't cope with
<             // a single KEYVALUE without a KEYBINDING element
<             //
<             if (numkeys>0) {
<                 for(int i=0;i<numkeys;i++) {
<                     CIMProperty cp=(CIMProperty)(keys.elementAt(i));
<                     String name=cp.getName();
<                     ret=ret.append("<KEYBINDING NAME=\""+name+"\">"
<                                         +outputKEYVALUE(cp)
<                                         +"</KEYBINDING>");
<                 }
<             }
<             else if (numkeys==1) {
<                 //
<                 // Keys =1, so go with KEYVALUE
<                 //
<                 CIMProperty cp=(CIMProperty)(keys.elementAt(0));
<                 ret=ret.append(outputKEYVALUE(cp));
<         } else {
<                 //
<                 // No keys, so no instances
<                 //
<                 throw new MalformedURLException("No instance path because no keys");
<             }
<             ret=ret.append("</INSTANCENAME>");
< 
<         return(ret.toString());
<     }
< 
<     private String outputKEYVALUE(CIMProperty cp) throws MalformedURLException  {
<         CIMDataType dtype=cp.getType();
<         String type;
< 
<         if (dtype.isArrayType())
<             throw new MalformedURLException("An array cannot be a KEY");
< 
<         if (dtype.isReferenceType()) {
<             //
<             // Is a reference
<             //
<             return(cp.getValue().toXml());
<         } else {
<             //
<             // Regular key value
<             int cd=dtype.getType();
< 
<             if (cd==CIMDataType.CHAR16 || cd==CIMDataType.DATETIME || cd==CIMDataType.STRING)
<                 type="string";
<             else if (cd==CIMDataType.BOOLEAN)
<                 type="boolean";
<             else
<                 type="numeric";
<             CIMValue keyCIMValue=cp.getValue();
<             if (keyCIMValue==null)
<                 throw new MalformedURLException("No key value");
<             String keyvalue=keyCIMValue.getValue().toString();
<             return("<KEYVALUE VALUETYPE=\""+type+"\">"+
<                    XMLCIM.xmlCIMEscape.XMLEscape(keyvalue)+
<                    "</KEYVALUE>");
<         }
<     }
< 
<     //
<     // NOT SUN
<     //
< 
< 
<     ///////////////////////////////////////////////////////////////////////////
<     // [NOTE]: This is not aligned with JSR-000048, and the instance
<     // returned may not be a valid one if the user has not set the
<     // namespace and hostname correctly!!!
<     ///////////////////////////////////////////////////////////////////////////
< 
< 
<     /***
<      * @deprecated
<      */
<     public CIMNameSpace getFullNameSpace() {
<         return(nameSpace);
<     }
< 
< 
<     //
<     // For testing purposes
<     //
<     public static void main(String[] args) {
<         int i;
<         for(i=0;i<args.length;i++) {
<             System.err.println("Processing arg#"+i+" : "+args[i]);
<             try {
<                 CIMObjectPath cop=new CIMObjectPath();
<                 cop.setNameSpace("\\root\\cimv2");
<                 cop.setHost("myHostName");
<                 cop.parse(args[i]);
<                 System.err.println("PATH="+cop.toString());
<                 CIMNameSpace ns=cop.getFullNameSpace();
<                 System.err.println("NAMESPACE="+ns.toString());
<             } catch (CIMException e) {
<                 e.printStackTrace(System.err);
<                 System.err.println("XML parse failed:"+e);
<             }
<         }
<     }
< }
---
> /*
> static char sccsid[] = "@(#)87  1.1  src/com/ibm/cim/CIMObjectPath.java, cimom, ICAT2 10/10/02 14:42:21";
> */
> 
> //
> // CIMObjectPath.java
> //
> // The contents of this file are subject to the SNIA Public License Version 1.0
> // (the "License"); you may not use this file except in compliance with the
> // License. You may obtain a copy of the License at
> //
> //	http://www.snia.org/resources/openSource.html
> //
> // Software distributed under the License is distributed on an "AS IS" basis,
> // WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
> // License for the specific language governing rights and limitations
> // under the License.
> //
> // The Original Code is CIMObjectPath.java
> //
> // The Initial Developer of the Original Code is David Simons.
> //
> // Contributor(s):
> //
> // This source uses the Solaris WBEM APIs version 2.2.
> // Synchronization will be maintained with the Solaris WBEM APIS,
> // which are being standardized through the Java Community Process.
> //
> // $Id$
> //
> // $Log$
> // Revision 1.16  2001/07/22 14:34:28  davidsi
> // - From Jean Lamontagne - using toString().hashCode() is potentially
> //   problematic because it assumes the keys will be correctly ordered.
> //   For the moment it is safer to rely on the objectName.hashCode().
> //
> // Revision 1.15  2001/07/22 13:29:57  davidsi
> // - Further updates from Angela.
> //
> // Revision 1.14  2001/07/07 16:29:23  davidsi
> // - Make the not set value less like to clash with a real value by adding
> //   non standard characters to it.
> //
> // Revision 1.13  2001/07/07 15:47:00  davidsi
> // - From Angela Lee. Significant reworking to ensure all values are
> //   correctly set - now rather than defaulting to null/"" use
> //   an explicit not set value.
> //
> // Revision 1.12  2001/07/07 13:41:45  davidsi
> // - From Daniel Abramovich, fixed equals() method for handling non
> //   instance cases.
> //
> // Revision 1.11  2001/06/23 13:57:20  davidsi
> // - Fixed hashCode for CIMObjectPath (Tatsuya Kainuma)
> //
> // Revision 1.10  2001/03/19 17:34:49  davidsi
> // - Key values are now compared as strings without reference to their
> //   types.
> //
> // Revision 1.9  2000/11/06 18:07:39  davidsi
> // - Fixed error IDs.
> // - Changes to suport the movement of the escaping functionality to
> //   CIMOMEscape.
> //
> // Revision 1.8  2000/08/16 05:59:52  shawn
> // merge P_TOMCAT31 branch
> //
> // Revision 1.7.6.1  2000/08/10 00:03:27  shawn
> // head merge
> //
> // Revision 1.7  2000/07/19 16:48:57  davidsi
> // - Revised escaping mechanisms for object paths.
> //
> // Revision 1.6  2000/05/19 14:43:12  davidsi
> // - Fixed the assumption that key components will always be in the same
> //   order when performing a comparison - they may not be.
> // - Convert key values to strings prior to comparing. In some (rare)
> //   cases the values may be stored as different types but be
> //   representing the same values (strictly one shouldn't need to
> //   do this but some providers can be inconsistent about how they
> //   type properties)
> //
> // Revision 1.5  2000/05/11 10:12:20  davidsi
> // - Fixed tabs.
> //
> // Revision 1.4  2000/05/06 17:04:22  davidsi
> // - Modifications to bring classes inline with the SUN WBEM APIs 2.2.
> //
> // Revision 1.3  2000/04/09 13:17:58  davidsi
> // - Switched to StringBuffer (from String) for XML generation.
> // - Switched to forcing a <KEYBINDING> tag - no longer parse
> //   singleton property values without a property name.
> //
> // Revision 1.2  2000/02/13 15:09:07  davidsi
> // - Added text to cover re-use of Solaris WBEM APIs.
> //
> // Revision 1.1.1.1  2000/02/06 16:41:03  davidsi
> // Initial version.
> //
> //
> //
> /* Change history:
>    Name                Date        Subject
>    Karsten Jancke      10/15/2002  Deep clone instead of shallow clone
> */
> 
> 
> package com.ibm.cim;
> 
> 
> import java.util.Vector;
> import java.io.Serializable;
> import java.net.MalformedURLException;
> import java.util.StringTokenizer;
> 
> import com.ibm.xml.*;
> 
> 
> 
> 
> public class CIMObjectPath implements Serializable {
> 
>  	public final static String NOT_SET = "$$Not$$Set$$";
> 
> 	CIMNameSpace ns = new CIMNameSpace(CIMObjectPath.NOT_SET, CIMObjectPath.NOT_SET);
> 
> 	/**
> 	 *  Host name of Object path
> 	 */
> 	private String hostName = "";
> 
> 	/**
> 	 *	name Space of Object Path
> 	 */
> 	private String nameSpace= "";
> 
> 	/**
> 	 *	Qualifier of Object Path
> 	 */
> 	private String objectName= "";
> 
> 	/**
> 	 *	Keys
> 	 */
> 	private Vector keys;
> 
> 
> 	//Constructor
> 	public CIMObjectPath(){
> 		keys = new Vector();
> 	}
> 
> 	//Constructor
> 	public CIMObjectPath(String name){
> 		this();
> 		objectName = name;
> 	}
> 
> 
> 	//constructor
> 	public CIMObjectPath(String name, String namespace) {
> 		this();
> 		objectName = name;
> 		setNameSpace(namespace);
> 	}
> 
> 	public CIMObjectPath(String name, CIMNameSpace ns) {
> 		this();
> 		objectName = name;
> 		this.ns = ns;
> 	}
> 
> //
>     // CIMObjectPath
>     //
>     // Create an instance using a XML definition
>     //
>     // TODO-SUN, its not clear which types of NODE the SUN API is intended
>     // to accept. This is a best guess!
>     //
>     // Note does not accept XML_ELEMENT_OBJECTPATH - caller must jump
>     // over such an element
>     //
>     // @param node The node containing an XML definition of CIMObjectPath
>     //
>     public CIMObjectPath(org.w3c.dom.Node inNode) {
> 	XMLNode node=XMLNode.getXMLNode(inNode);
> 	int token=node.getToken();
> 
> 	try {
> 	    if (token==XMLNode.XML_ELEMENT_LOCALCLASSPATH
> 		|| token==XMLNode.XML_ELEMENT_CLASSPATH) {
> 		boolean dolocal=false;
> 		if (node.getToken()!=XMLNode.XML_ELEMENT_LOCALCLASSPATH)
> 		    dolocal=true;
> 
> 		XMLNode localNameNode=node.mustChildElementChild(XMLNode.XML_ELEMENT_NAMESPACEPATH);
> 		//
> 		// Get NameSpace path
> 		//
> 		if (!dolocal) {
> 		    localNameNode.mustElement(XMLNode.XML_ELEMENT_HOST);
> 		    setHost(localNameNode.getUnescapeText());
> 		}
> 		localNameNode=localNameNode.mustNextElement(XMLNode.XML_ELEMENT_LOCALNAMESPACEPATH);
> 		String readNS=XMLClass.getNameSpace(localNameNode.getChild());
> 		setNameSpace(readNS);
> 		node=node.mustNextElement(XMLNode.XML_ELEMENT_CLASSNAME);
> 		String className=node.mustGetAttribute("Name");
> 		setObjectName(className);
> 		return;
> 	    }
> 	    //
> 	    // Must be an instance path or an error...
> 	    //
> 	    boolean dolocal=false;
> 	    XMLNode pathNode;
> 
> 	    if (token==XMLNode.XML_ELEMENT_LOCALINSTANCEPATH) {
> 		dolocal=true;
> 		node=node.getChild();	    // check for null below!
> 		pathNode=node;
> 	    } else if (token==XMLNode.XML_ELEMENT_INSTANCEPATH) {
> 		node=node.mustChildElement(XMLNode.XML_ELEMENT_NAMESPACEPATH);
> 		pathNode=node.getChild();
> 		pathNode.mustElement(XMLNode.XML_ELEMENT_HOST);
> 		setHost(pathNode.getUnescapeText());
> 		pathNode=pathNode.getNext();
> 	    } else
> 		throw new java.lang.InstantiationError(
> 				    "Required <INSTANCEPATH> or  <CLASSPATH>");
> 
> 	    //
> 	    // Make sure nodes are valid before continuing..
> 	    //
> 	    if (pathNode==null)
> 		throw new CIMException(CIMException.CIM_ERR_FAILED,
> 				    "Required <LOCALNAMESPACEPATH>");
> 	    if (node==null)
> 		throw new CIMException(CIMException.CIM_ERR_FAILED,
> 				    "Required <INSTANCENAME>");
> 	    //
> 	    // Get NameSpace path
> 	    //
> 	    pathNode.mustElement(XMLNode.XML_ELEMENT_LOCALNAMESPACEPATH);
> 	    setNameSpace(XMLClass.getNameSpace(pathNode.getChild()));
> 	    node=node.mustNextElement(XMLNode.XML_ELEMENT_INSTANCENAME);
> 
> 	    XMLClass.getInstanceName(node,this);
> 	} catch (CIMException ce) {
> 	    throw new java.lang.InstantiationError(ce.toString());
> 	}
>     }
> 
> 
> 	//Constructor
> 	public CIMObjectPath(String objName, Vector kVect){
> 		this();
> 		objectName = objName;
> 		keys = kVect;
> 	}
> 
> 	/**
> 	 *	Returns object name
> 	 */
> 	public String getObjectName(){
> 		return objectName;
> 	}
> 
> 
> 
> 	/**
> 	 *	Sets object name
> 	 */
> 	public void setObjectName(String objName) {
> 		objectName = objName;
> 	}
> 
> 
> 	/**
> 	 *	Returns Namespace string
> 	 */
> 	public String getNameSpace(){
> 		return nameSpace;
> 	}
> 
> 	/**
> 	 *	Sets Namespace string
> 	 */
> 	public void setNameSpace(String ns){
> 		nameSpace =ns;
> 		this.ns.setNameSpace(ns);
> 	}
> 
> 
> 	/**
> 	 *	returns CIMNameSpace
> 	 */
> 	public CIMNameSpace getFullNameSpace(){
> 		return ns;
> 	}
> 
> 
> 
> 	/**
> 	 *	Returns keys
> 	 */
> 	public Vector getKeys(){
> 		return keys;
> 	}
> 
> 	/**
> 	 *	Sets keys
> 	 */
> 	public void setKeys(Vector objKey){
> 		keys = objKey;
> 	}
> 
> 	/**
> 	 *	Adds new Key
> 	 */
> 	public void addKey(String name, CIMValue value) {
> 		CIMProperty cimProp = new CIMProperty(name,value);
> 		if( value != null){
> 			cimProp.setType(value.getType());
> 		}
> 		cimProp.addQualifier(new CIMQualifier("key"));
> 		keys.addElement(cimProp);
> 	}
> 
> 
> 	/**
> 	 *	Returns Host for the ObjectPath
> 	 */
> 	public String getHost() {
> 		return hostName;
> 	}
> 
> 	/**
> 	 *	Sets Host for the ObjectPath
> 	 */
> 	public void setHost(String host) {
> 		hostName = host;
> 	}
> 
> 	/**
> 	 *	Returns  string representation of objectpath
> 	 */
>     public String toString() {
> 	if ( !(keys.isEmpty()) ) {
> 	    String ret=objectName;
> 
> 	    for(int i=0;i<keys.size();i++) {
> 		CIMProperty cp=(CIMProperty)(keys.elementAt(i));
> 		if (i>0) {
> 			ret=ret.concat("," + cp.getName()+ "=\"" +
> 				   CIMOMEscape.escape(cp.getValue().getValue().toString())+
> 				   "\"");
> 		}else {
> 		      ret=ret.concat("." + cp.getName() + "=\"" +
> 				   CIMOMEscape.escape(cp.getValue().getValue().toString())+
> 				   "\"");
> 		}
> 	    }
> 	    return(ret);
> 	}
> 	return(objectName);
>     }
> 
> 	    /***
>      * parse
>      *
>      * Converts a string representation of an object path
>      *
>      * @param instanceName
>      *    The object path to convert - assumed to be to an instance
>      * @throws
>      *    CIMException an error occurred parsing the path
>      */
>     public void parse(String instanceName) throws CIMException {
> 	char []values=instanceName.toCharArray();
> 	int i;
> 	int keystart=0;
> 	String text;
> 	int equalspos;
> 	Vector tmpkeys=new Vector();
> 	boolean singleKey=true;
> 	boolean inquote=false;
> 	String keyvalue;
> 	boolean nstype=true;
> 	boolean nshdl=false;
> 	int classNamePos=0;	    // starting position of classname
> 	String nameSpaceType=null;  // namespace type
> 	String nameSpaceHdl=null;   // handle for namespace
> 
> 	singleKey=false;	// set this to TRUE if it is intended that
> 				// a property value can be specified without
> 				// its name when there is only one key.
> 
> 	for(i=0;i<values.length;i++) {
> 	    //
> 	    // One can come across '.' in the namespace handle because
> 	    // it may be an IP address
> 	    //
> 	    if (values[i]=='.' && !nshdl)
> 		break;
> 	    if (values[i]==':') {
> 		if (nstype) {
> 		    // found namespace type
> 		    nameSpaceType=new String(values,0,i);
> 		    classNamePos=i+1;
> 		    nstype=false;
> 		    nshdl=true;
> 		} else {
> 		    // found namespace handle
> 		    nameSpaceHdl=new String(values,classNamePos,i-classNamePos);
> 		    classNamePos=i+1;
> 		    nshdl=false;
> 		}
> 	    }
> 	}
> 
> 	//
> 	// There must be a '.' in the instance name if it is valid
> 	// as this delimits the class name
> 	//
> 	if (i==values.length)
> 	    throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
> 			"Instance name \""+instanceName+"\" did not have a '.' as a class delimeter");
> 
> 	//
> 	// If a namespace type was found then there must have also been
> 	// a handle
> 	//
> 	if (!nstype && nshdl)
> 	    throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
> 			"Instance name \""+instanceName+"\" did not have a namespace handle");
> 
> 	//
> 	// Only http namespace types are supported
> 	if (nameSpaceType!=null && !nameSpaceType.equalsIgnoreCase("HTTP"))
> 	    throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
> 			"Instance name \""+instanceName+"\" does not represent a HTTP namespace");
> 
> 
> 	String className=new String(values,classNamePos,i-classNamePos);
> 
> 	//
> 	// Now extract keys
> 	//
> 	i++;
> 	equalspos=0;
> 	for(keystart=i;i<values.length;i++) {
> 	    char ch=values[i];
> 		//
> 		// Skip escaped characters
> 		//
> 	    if (i<values.length-1 && ch=='\\') {
> 		i++;
> 		continue;
> 	    }
> 	    //
> 	    // Check for quotes
> 	    //
> 	    if (ch=='\"')
> 		inquote = !inquote;
> 	    if (inquote)
> 		continue;
> 
> 	    if (ch=='=')
> 		equalspos=i+1;
> 	    if (ch==',' || (i+1==values.length) ) {
> 		if ((i+1==values.length) && equalspos==0 && singleKey) {
> 		    //
> 		    // This is the special case of when its classname.value
> 		    //
> 		    CIMProperty cp=new CIMProperty();
> 		    cp.setName("test it");
> 		    //
> 		    // Generally there will be quotes but for integer values
> 		    // they are not strictly necessary so check for them
> 		    //
> 		    if (values[keystart]!='\"')
> 			keyvalue=new String(values,keystart,i-keystart+1);
> 		    else
> 			keyvalue=new String(values,keystart+1,i-keystart+1-2);
> 		    cp.setValue(new CIMValue(keyvalue,
> 					    CIMDataType.getPredefinedType(
> 					    CIMDataType.STRING)
> 					    ));
> 		    cp.setType(CIMDataType.getPredefinedType(
> 					CIMDataType.STRING));
> 		    tmpkeys.addElement(cp);
> 		    break;
> 		}
> 
> 		if (i+1==values.length)
> 		    i++;
> 
> 		if (equalspos==0) {
> 		    throw new CIMException(CIMException.CIM_ERR_NOT_FOUND,
> 			"Bad key in string ("+instanceName+")");
> 		}
> 		String keyprop=new String(values,keystart,equalspos-keystart-1);
> 		//
> 		// Generally there will be quotes but for integer values
> 		// they are not strictly necessary so check for them
> 		//
> 		if (values[equalspos]!='\"')
> 		    keyvalue=new String(values,equalspos,i-equalspos);
> 		else
> 		    keyvalue=CIMOMEscape.unEscape(new String(values,equalspos+1,i-equalspos-2));
> 		i++;
> 		equalspos=0;
> 		keystart=i;
> 		CIMProperty cp=new CIMProperty();
> 		cp.setName(keyprop);
> 		cp.setValue(new CIMValue(keyvalue,
> 				CIMDataType.getPredefinedType(
> 				CIMDataType.STRING) ));
> 		cp.setType(CIMDataType.getPredefinedType(
> 				    CIMDataType.STRING));
> 		tmpkeys.addElement(cp);
> 		singleKey=false;
> 	    }
> 	}
> 
> 
> 	if (nameSpaceHdl!=null) {
> 	    String hostName=null;
> 	    String newNameSpace=null;
> 
> 	    if (!nameSpaceHdl.substring(0,2).equals("//"))
> 		throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
> 			"Instance name \""+instanceName+"\" does not specify a host");
> 	    StringTokenizer stok=new StringTokenizer(nameSpaceHdl,"/",false);
> 	    for (int tokenNumber=0;stok.hasMoreTokens();tokenNumber++) {
> 		String token=stok.nextToken();
> 		if (tokenNumber==0) {
> 		    hostName=token;
> 		} else if (tokenNumber==1) {
> 		    newNameSpace=token;
> 		} else {
> 		    newNameSpace=newNameSpace.concat("/"+token);
> 		}
> 	    }
> 	    if (hostName==null)
> 		throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
> 			"Instance name \""+instanceName+"\" does not specify a host");
> 	    if (newNameSpace==null)
> 		throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
> 			"Instance name \""+instanceName+"\" does not specify a namespace");
> 	    setNameSpace(newNameSpace);
> 	    setHost(hostName);
> 	}
> 	/*else
> 	    nameSpace= new CIMNameSpace();
>     */
> 	//
> 	// Now assign the values to this instance
> 	//
> 	objectName=className;
> 	keys=tmpkeys;
>     }
> 
>  // equals
>     //
>     // Compares this object path with another object path
>     //
>     public boolean equals(Object o) {
> 	if (!(o instanceof CIMObjectPath))
> 	    return(false);
> 	CIMObjectPath cop=(CIMObjectPath)o;
> 
> 	// not the same object name...
> 	if (! objectName.equalsIgnoreCase(cop.objectName))
> 	    return(false);
> 	// different namespace...
> 	else if (! getNameSpace().equalsIgnoreCase(cop.getNameSpace()))
> 	    return(false);
> 	// different host name...
> 	else if (! getHost().equalsIgnoreCase(cop.getHost()))
> 	    return(false);
> 
> 	// both have one or more keys
> 	if (!keys.isEmpty() && !cop.keys.isEmpty()) {
> 	    //
> 	    // An instance path
> 	    //
> 
> 	    if (keys.size()!=cop.keys.size()) {
> 		return(false);
> 	    }
> 	    if (keys.size()==1) {
> 		//
> 		// Special case for a single, just use class name and
> 		// value
> 		//
> 		CIMProperty cp1=(CIMProperty)(keys.elementAt(0));
> 		CIMProperty cp2=(CIMProperty)(cop.keys.elementAt(0));
> 		CIMValue cv1 = cp1.getValue();
> 		CIMValue cv2 = cp2.getValue();
> 		if (cv1==null && cv2==null)
> 		    return(true);
> 		if (cv1==null || cv2==null)
> 		    return(false);
> 		return (cv1.toString().equals(cv2.toString()));
> 	    }
> 
> 	    int maxNoKeys=keys.size();
> 	    for(int i=0;i<maxNoKeys;i++) {
> 		CIMProperty cp1=(CIMProperty)(keys.elementAt(i));
> 
> 		//
> 		// In general keys will be in the same order, so
> 		// try the same index first...
> 		//
> 		CIMProperty cp2=(CIMProperty)(cop.keys.elementAt(i));
> 		if (!cp1.getName().equalsIgnoreCase(cp2.getName())) {
> 			//
> 			// No match so start searching for the
> 			// equivalent key
> 			//
> 			int j;
> 	    		for(j=0;j<maxNoKeys;j++) {
> 				cp2=(CIMProperty)(cop.keys.elementAt(j));
> 				if (cp1.getName().equalsIgnoreCase(
> 							cp2.getName()))
> 					break;
> 			}
> 			//
> 			// Not found, failed to match
> 			//
> 			if (j==maxNoKeys)
> 				return(false);
> 		}
> 		CIMValue cv1 = cp1.getValue();
> 		CIMValue cv2 = cp2.getValue();
> 		if (cv1==null && cv2==null)
> 		    continue;
> 		if (cv1==null || cv2==null)
> 		    return(false);
> 		if ( !(cv1.toString().equals(cv2.toString())))
> 		    return(false);
> 	    }
> 
> 	    return(true);
> 	} else if ( (keys.isEmpty() && !cop.keys.isEmpty()) ||
> 			(!keys.isEmpty() && cop.keys.isEmpty()) ) {
> 	    // only one has one or more keys
> 	    return(false);
> 	} else {
> 	    // both are empty
> 	    return(true);
> 	}
>     }
> 
>     public boolean equals(Object o,boolean ignoreClassOrigins) {
> 	//
> 	// SUNDIFF - unclear why classorigins would ever be checked!
> 	//
> 	return(equals(o));
>     }
> 
>     // toXml
>     public String toXml() throws MalformedURLException {
> 	StringBuffer ret;
> 
> 	//
> 	// Instance path
> 	//
> 	ret=new StringBuffer("<INSTANCENAME CLASSNAME=\""+
> 			 objectName+"\">");
> 
> 	if (keys==null || keys.size()==0) {
> 	    //
> 	    // TODO-SUN
> 	    // This only supports writing instance paths, but does the
> 	    // sun implementation write class paths as well?
> 	    //
> 	    throw new MalformedURLException("Bad path to object");
> 	}
> 
> 
> 	int numkeys=keys.size();
> 	    //
> 	    // If keys >1 then must use KEYBINDING - we also use it for
> 	    // the key==1 case - most implementations can't cope with
> 	    // a single KEYVALUE without a KEYBINDING element
> 	    //
> 	    if (numkeys>0) {
> 		for(int i=0;i<numkeys;i++) {
> 		    CIMProperty cp=(CIMProperty)(keys.elementAt(i));
> 		    String name=cp.getName();
> 		    ret=ret.append("<KEYBINDING NAME=\""+name+"\">"
> 					+outputKEYVALUE(cp)
> 					+"</KEYBINDING>");
> 		}
> 	    }
> 	    else if (numkeys==1) {
> 		//
> 		// Keys =1, so go with KEYVALUE
> 		//
> 		CIMProperty cp=(CIMProperty)(keys.elementAt(0));
> 		ret=ret.append(outputKEYVALUE(cp));
> 	} else {
> 		//
> 		// No keys, so no instances
> 		//
> 		throw new MalformedURLException("No instance path because no keys");
> 	    }
> 	    ret=ret.append("</INSTANCENAME>");
> 
> 	return(ret.toString());
>     }
> 
>     private String outputKEYVALUE(CIMProperty cp) throws MalformedURLException	{
> 	CIMDataType dtype=cp.getType();
> 	String type;
> 
> 
> 	if (dtype.isArrayType())
> 	    throw new MalformedURLException("An array cannot be a KEY");
> 
> 	if (dtype.isReferenceType()) {
> 	    //
> 	    // Is a reference
> 	    //
> 	    return(cp.getValue().toXml());
> 	} else {
> 	    //
> 	    // Regular key value
> 	    int cd=dtype.getType();
> 
> 	    if (cd==CIMDataType.CHAR16 || cd==CIMDataType.DATETIME || cd==CIMDataType.STRING)
> 		type="string";
> 	    else if (cd==CIMDataType.BOOLEAN)
> 		type="boolean";
> 	    else
> 		type="numeric";
> 	    CIMValue keyCIMValue=cp.getValue();
> 	    if (keyCIMValue==null)
> 		throw new MalformedURLException("No key value");
> 	    String keyvalue=keyCIMValue.getValue().toString();
> 	    return("<KEYVALUE VALUETYPE=\""+type+"\">"+
> 		   XMLCIM.xmlCIMEscape.XMLEscape(keyvalue)+
> 		   "</KEYVALUE>");
> 	}
>     }
> 
> 
> 	public Object clone() {
> 
> 		Vector newKeys = new Vector(keys != null ? keys.size() : 0);
> 		if( keys != null )
> 			for( int i=0; i<keys.size(); i++ )
> 				newKeys.add(((CIMProperty)keys.elementAt(i)).clone());
> 				
> 		CIMObjectPath theClone = new CIMObjectPath( objectName, newKeys );
> 		theClone.setHost( hostName );
> 		theClone.setNameSpace( nameSpace );
> 		return theClone;
> 	}
> 
> 	/*1800A*/
> 	public int hashCode(){
> 		return toString().hashCode();
> 	}
> }
