1,484c1,491
< //
< // CIMValue.java
< //
< // The contents of this file are subject to the SNIA Public License Version 1.0
< // (the "License"); you may not use this file except in compliance with the
< // License. You may obtain a copy of the License at
< //
< //      http://www.snia.org/resources/openSource.html
< //
< // Software distributed under the License is distributed on an "AS IS" basis,
< // WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
< // License for the specific language governing rights and limitations
< // under the License.
< //
< // The Original Code is CIMValue.java
< //
< // The Initial Developer of the Original Code is David Simons.
< //
< // Contributor(s): Michael Leizin, Angela Lee
< //
< //
< // This source uses the Solaris WBEM APIs version 2.2.
< // Synchronization will be maintained with the Solaris WBEM APIS,
< // which are being standardized through the Java Community Process.
< //
< //
< // $Id$
< //
< // $Log$4
< // Revision 1.16  2001/07/22 13:26:02  davidsi
< // - The object pathes in the key-value pairs for association class instances
< //   are mistyped. They are of type String but should be of type
< //   CIMObjectPath. (From Viktor)
< //
< // Revision 1.15  2001/04/15 23:59:33  davidsi
< // - Fixed <VALUE.ARRAY> generation for xml output.
< //
< // Revision 1.14  2001/03/31 12:13:50  davidsi
< // - Fixed toString() to display arrays with ',' separation
< // - Modified ctr to handle vectors.
< //
< // Revision 1.12  2001/01/07 16:44:53  davidsi
< // - Added fixes to handle nulls in arrays.
< //
< // Revision 1.11  2000/11/12 14:29:45  davidsi
< // - Fixed error in last checkin which meant XML parsing would fail
< //   for CIMValue nodes.
< // - Added check for null datatypes in toString().
< //
< // Revision 1.10  2000/11/09 17:33:38  davidsi
< // - Added the ability to pass a null xml node into the constructor
< //   because people may use the constructor to build a null CIMValue
< //   instances and the Object based ctr may not be selected.
< // - Handle null in the toString() method.
< //
< // Revision 1.9  2000/11/06 18:13:21  davidsi
< // - Reconciled error ids.
< //
< // Revision 1.8  2000/08/16 05:59:52  shawn
< // merge P_TOMCAT31 branch
< //
< // Revision 1.7.6.1  2000/08/10 00:03:27  shawn
< // head merge
< //
< // Revision 1.7  2000/07/06 17:42:56  davidsi
< // - Added missing java.lang.reflect.Array.
< //
< // Revision 1.6  2000/07/05 13:33:40  davidsi
< // - Allow null values to be passed during creation.
< // - Handle arrays being passed as the Java objects to be handled as
< //   CIMValues.
< //
< // Revision 1.5  2000/05/11 10:12:20  davidsi
< // - Fixed tabs.
< //
< // Revision 1.4  2000/05/06 17:04:23  davidsi
< // - Modifications to bring classes inline with the SUN WBEM APIs 2.2.
< //
< // Revision 1.3  2000/04/09 13:15:30  davidsi
< // - Switched to StringBuffer (from String) for XML generation.
< //
< // Revision 1.2  2000/02/13 15:09:08  davidsi
< // - Added text to cover re-use of Solaris WBEM APIs.
< //
< // Revision 1.1.1.1  2000/02/06 16:41:03  davidsi
< // Initial version.
< //
< //
< //
< package org.snia.wbem.cim;
< 
< import java.util.Vector;
< import java.lang.reflect.Array;
< import java.io.Serializable;
< import java.net.MalformedURLException;
< import org.snia.wbemcmd.xml.*;
< import org.snia.wbemcmd.xml.XMLCIM;
< 
< public class CIMValue extends Object implements Serializable {
< 
<     private CIMDataType dataType=null;
<     private Object objValue=null;
< 
<     public static final CIMValue TRUE=new CIMValue(Boolean.TRUE,new CIMDataType(CIMDataType.BOOLEAN));
<     public static final CIMValue FALSE=new CIMValue(Boolean.FALSE,new CIMDataType(CIMDataType.BOOLEAN));
< 
<     // CIMValue
<     //
<     // Constructs any value type
<     //
<     public CIMValue(Object o,CIMDataType dt) {
<         objValue=o;
<         dataType=dt;
<         if (dt.isReferenceType() && !(o instanceof CIMObjectPath))
<             throw new ExceptionInInitializerError(
<                 "A ref type must be represented with an object path");
<     }
< 
<     //
<     // CIMValue
<     //
<     // Constructs array values
<     //
<     public CIMValue(Vector v,CIMDataType dt) {
<         if (!dt.isArrayType())
<             throw new ExceptionInInitializerError(
<                 "CIMValue created with non array type on wrong constructor ("+v+")");
<         objValue=v;
<         dataType=dt;
<     }
< 
<     //
<     // CIMValue
<     //
<     // Constructs a value on the basis of the passed object
<     //
<     public CIMValue(Object o) {
< 
<         if (o==null) {
<             objValue=null;
<             dataType=null;
<             return;
<         }
< 
<         // if it is a Vector, assume (for now) that all elements are
<         // of the same type & no array type elements are present
<         if (o instanceof Vector) {
<             // objValue & dataType are set to null by default
<             Vector v = (Vector)o;
<             if (! v.isEmpty()) {
<                 objValue = v;
<                 Object elementObj = v.elementAt(0);
<                 // if null, assume String type
<                 if (elementObj == null)
<                     dataType=new CIMDataType(CIMDataType.STRING_ARRAY, v.size());
<                 else {
<                     int simpleType = CIMDataType.findType(elementObj);
<                     // if the element object is an invalid type,
<                     // throw an Exception
<                     if (simpleType == CIMDataType.INVALID) {
<                         objValue = null;
<                         throw new ExceptionInInitializerError(
<                                 "Vector element is an invalid type object of "
<                                 + elementObj.getClass().getName());
<                     } else
<                         dataType = new CIMDataType(CIMDataType.findArrayType(simpleType), v.size());
<                 }
<             } else {
<                 throw new ExceptionInInitializerError(
<                                         "An empty Vector was passed in");
<             }
<         } else {
<             Class objClass=o.getClass();
<             if ( objClass.isArray() ) {
<                 String arrayClass = objClass.getComponentType().getName();
<                 try {
<                     CIMDataType cdt=CIMDataType.getDataType(arrayClass);
<                     dataType=new CIMDataType(
<                                    CIMDataType.findArrayType(cdt.getType()) );
<                     objValue=o;
<                     return;
<                 } catch (CIMException ce) {
<                     throw new ExceptionInInitializerError(ce.toString());
<                 }
<             }
<             int type=CIMDataType.findType(o);
<             if (type==CIMDataType.INVALID) {
<                 objValue=null;
<                 dataType=null;
<                 throw new ExceptionInInitializerError(
<                                         "Failed to determine CIMDataType of "
<                                         +objClass.getName());
<             }
<             dataType=CIMDataType.getPredefinedType(type);
<             objValue=o;
<         }
<     }
< 
<     //
<     // CIMValue
<     //
<     // Create a value using a XML definition
<     //
<     // @param node The node containing an XML definition of CIMValue
<     //
<     public CIMValue(org.w3c.dom.Node inNode) {
<         if (inNode==null) {
<             objValue=null;
<             dataType=null;
<             return;
<         }
< 
<         XMLNode node=XMLNode.getXMLNode(inNode);
<         int token=node.getToken();
<         try {
<             //
<             // Look for <VALUE> elements
<             //
<             if (token==XMLNode.XML_ELEMENT_VALUE ||
<                 token==XMLNode.XML_ELEMENT_KEYVALUE) {
<                 objValue=node.getUnescapeText();
<                 dataType=CIMDataType.getPredefinedType(CIMDataType.STRING);
<                 return;
<             }
<             //
<             // Look for <VALUE.ARRAY> elements
<             //
<             if (token==XMLNode.XML_ELEMENT_VALUE_ARRAY) {
<                 Vector v=new Vector();
<                 XMLNode valueNode=node.getChild();
<                 while(valueNode!=null) {
<                     if (valueNode.getToken()==XMLNode.XML_ELEMENT_VALUE) {
<                         v.addElement(valueNode.getUnescapeText());
<                     }
<                     valueNode=valueNode.getNext();
<                 }
<                 objValue=v;
<                 dataType=CIMDataType.getPredefinedType(CIMDataType.STRING_ARRAY);
<                 return;
<             }
<             if (token==XMLNode.XML_ELEMENT_VALUE_REFERENCE) {
<                 XMLNode valueNode=node.getChild();
<                 token=valueNode.getToken();
<                 CIMObjectPath cop;
<                 String result;
<                 if (token==XMLNode.XML_ELEMENT_CLASSPATH
<                     || token==XMLNode.XML_ELEMENT_LOCALCLASSPATH
<                     || token==XMLNode.XML_ELEMENT_INSTANCEPATH
<                     || token==XMLNode.XML_ELEMENT_LOCALINSTANCEPATH )
<                     cop=new CIMObjectPath(valueNode);
<                 else if (token==XMLNode.XML_ELEMENT_CLASSNAME) {
<                     cop=new CIMObjectPath();
< //                    cop.setObjectName(node.mustGetAttribute("Name"));
<                     cop.setObjectName(valueNode.mustGetAttribute("Name"));
<                 } else if (token==XMLNode.XML_ELEMENT_INSTANCENAME) {
<                     cop=new CIMObjectPath();
<                     XMLClass.getInstanceName(valueNode, cop);
<                 } else
<                     throw new CIMException(CIMException.CIM_ERR_FAILED,
<                         "Attempting to extract object path");
<                 objValue=cop;
<                 dataType=CIMDataType.getPredefinedType(CIMDataType.REFERENCE);
<                 return;
<             }
<             throw new java.lang.InstantiationError("Not a value node:" + token);
<         } catch (CIMException ce) {
<             throw new java.lang.InstantiationError(ce.toString());
<         }
<     }
< 
<     // isArrayValue
<     //
<     // Returns true if the value is an array type
<     //
<     public boolean isArrayValue() {
<         return(dataType.isArrayType());
<     }
< 
<     // isNullValue
<     //
<     // Returns true if the datatype is null
<     //
<     public boolean isNullValue() {
<         return(dataType==null);
<     }
< 
<     // toMOF
<     public String toMOF() {
<         String ret;
<         int type;
< 
<         if (objValue==null)
<             return("");
< 
<         type=dataType.getType();
<         if (dataType.isArrayType() && objValue instanceof Vector) {
<             int i;
<             Vector vect=(Vector)objValue;
<             type=CIMDataType.findSimpleType(type);
< 
<             ret=new String("{");
<             for(i=0;i<vect.size();i++) {
<                 if (i>0)
<                      ret=ret.concat(",");
<                 if (type==CIMDataType.STRING)
<                     ret=ret.concat("\"");
<                 ret=ret.concat(vect.elementAt(i).toString());
<                 if (type==CIMDataType.STRING)
<                     ret=ret.concat("\"");
<             }
<             ret=ret.concat("}");
<             return(ret);
<         }
<         ret=new String();
< 
<         if (type==CIMDataType.STRING)
<             ret=ret.concat("\"");
<         ret=ret.concat(objValue.toString());
< 
<         if (type==CIMDataType.STRING)
<             ret=ret.concat("\"");
<         return(ret);
<     }
< 
<     // toString
<     public String toString() {
<         String retVal = "null";
<         // return "null" string (not null reference) if valid
<         // data is not available
< 
<         if ((objValue != null) && (dataType != null)
<                  && (dataType.getType() != CIMDataType.INVALID)) {
<             int inType=dataType.getType();
< 
<             if (inType==CIMDataType.REFERENCE) {
<                 if (objValue instanceof CIMObjectPath)
<                     retVal = objValue.toString();
<                     //else, return "null"
<             } else if (dataType.isArrayType()) {
<                 Vector v=(Vector)objValue;
<                 StringBuffer buf = new StringBuffer();
<                 buf.append("[");
<                 for(int i=0;i<v.size();i++) {
<                     if (v.elementAt(i) != null)
<                         buf.append(v.elementAt(i).toString());
<                     else
<                         buf.append("null");
<                     if (i < v.size() - 1)
<                         buf.append(", ");
<                 }
<                 buf.append("]");
<                 retVal = buf.toString();
<             } else
<                 retVal = objValue.toString();
<         }
<         return retVal;
<     }
< 
<     // contains
<     public boolean contains(Object obj) {
< 
< 		/*Changed by prabhakar, to fix the problem faced with associators*/
< 		if(obj instanceof CIMObjectPath)
< 		{
< 			return(objValue.toString().equals(obj.toString()));
< 		}
< 		else
< 		{
< 			return(objValue.equals(obj));
< 		}
< 
< 
<         //return(objValue.equals(obj));
<     }
< 
<     // isEmpty
<     //
<     // TRUE if no datatype of value
<     //
<     public boolean isEmpty() {
<         return( dataType==null || objValue==null);
<     }
< 
<     // size
<     //
<     // Returns size of type
<     //
<     public int size() {
<         if (dataType==null)
<             return(0);
<         return(dataType.getSize());
<     }
< 
<     // getValue
<     //
<     // Gets current value
<     //
<     public Object getValue() {
<        return(objValue);
<     }
< 
<     // getType
<     //
<     // Gets current datatype
<     //
<     public CIMDataType getType() {
<         return(dataType);
<     }
< 
<     // equals
<     public boolean equals(Object obj) {
<         CIMValue cv;
< 
<         if (!(obj instanceof CIMValue) )
<             return(false);
< 
<         cv=(CIMValue)obj;
< 
<         return(cv.dataType.equals(dataType) &&
<                cv.objValue.equals(objValue));
<     }
< 
<     // toXml
<     //
<    public String toXml() throws MalformedURLException {
<         //
<         // If no value, don't even return a <VALUE> element
<         //
<         if (objValue==null)
<             return("");
< 
<         if (dataType.isArrayType()) {
<             if (dataType.isReferenceType()) {
<                 throw new MalformedURLException(
<                         "Arrays of references are not valid");
<             } else {
<                 StringBuffer arrayValues=new StringBuffer("<VALUE.ARRAY>");
<                 if (objValue instanceof Vector) {
<                     Vector vect=(Vector)objValue;
<                     for(int i=0;i<vect.size();i++) {
<                         Object obj = vect.elementAt(i);
<                         if( obj == null )
<                             arrayValues=arrayValues.append("<VALUE></VALUE>");
<                         else
<                             arrayValues=arrayValues.append("<VALUE>"
<                                 +XMLCIM.xmlCIMEscape.XMLEscape(obj.toString())
<                                 +"</VALUE>");
<                     }
<                 } else {
<                     // assume an array
<                     int length = Array.getLength(objValue);
<                     for(int i=0;i<length;i++) {
<                         Object ele=Array.get(objValue,i);
<                         if ( ele == null)
<                             arrayValues=arrayValues.append("<VALUE></VALUE>");
<                         else
<                             arrayValues=arrayValues.append("<VALUE>"
<                                 +XMLCIM.xmlCIMEscape.XMLEscape(ele.toString())
<                                 +"</VALUE>");
<                     }
<                 }
<                 arrayValues=arrayValues.append("</VALUE.ARRAY>");
<                 return(arrayValues.toString());
<             }
<         } else if (dataType.isReferenceType()) {
<             CIMObjectPath refObj=(CIMObjectPath)objValue;
<             CIMNameSpace ns=refObj.getFullNameSpace();
< 
<             if (ns.getHost()==null)
<                 return ("<VALUE.REFERENCE><LOCALINSTANCEPATH>"+
<                     ns.toXml(true)+
<                     refObj.toXml()+
<                     "</LOCALINSTANCEPATH></VALUE.REFERENCE>");
<             else
<                 return ("<VALUE.REFERENCE><INSTANCEPATH>"+
<                     ns.toXml(false)+
<                     refObj.toXml()+
<                     "</INSTANCEPATH></VALUE.REFERENCE>");
<         }
<         return("<VALUE>"
<                     +XMLCIM.xmlCIMEscape.XMLEscape(objValue.toString())
<                     +"</VALUE>");
<     }
< }
---
> /*
> static char sccsid[] = "@(#)94  1.1  src/com/ibm/cim/CIMValue.java, cimom, ICAT1, ICAT1_1.0.0.7 10/10/02 14:42:26";
> */
> 
> //
> // CIMValue.java
> //
> // The contents of this file are subject to the SNIA Public License Version 1.0
> // (the "License"); you may not use this file except in compliance with the
> // License. You may obtain a copy of the License at
> //
> //	http://www.snia.org/resources/openSource.html
> //
> // Software distributed under the License is distributed on an "AS IS" basis,
> // WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
> // License for the specific language governing rights and limitations
> // under the License.
> //
> // The Original Code is CIMValue.java
> //
> // The Initial Developer of the Original Code is David Simons.
> //
> // Contributor(s): Michael Leizin, Angela Lee
> //
> //
> // This source uses the Solaris WBEM APIs version 2.2.
> // Synchronization will be maintained with the Solaris WBEM APIS,
> // which are being standardized through the Java Community Process.
> //
> //
> // $Id$
> //
> // $Log$
> // Revision 1.16  2001/07/22 13:26:02  davidsi
> // - The object pathes in the key-value pairs for association class instances
> //   are mistyped. They are of type String but should be of type
> //   CIMObjectPath. (From Viktor)
> //
> // Revision 1.15  2001/04/15 23:59:33  davidsi
> // - Fixed <VALUE.ARRAY> generation for xml output.
> //
> // Revision 1.14  2001/03/31 12:13:50  davidsi
> // - Fixed toString() to display arrays with ',' separation
> // - Modified ctr to handle vectors.
> //
> // Revision 1.12  2001/01/07 16:44:53  davidsi
> // - Added fixes to handle nulls in arrays.
> //
> // Revision 1.11  2000/11/12 14:29:45  davidsi
> // - Fixed error in last checkin which meant XML parsing would fail
> //   for CIMValue nodes.
> // - Added check for null datatypes in toString().
> //
> // Revision 1.10  2000/11/09 17:33:38  davidsi
> // - Added the ability to pass a null xml node into the constructor
> //   because people may use the constructor to build a null CIMValue
> //   instances and the Object based ctr may not be selected.
> // - Handle null in the toString() method.
> //
> // Revision 1.9  2000/11/06 18:13:21  davidsi
> // - Reconciled error ids.
> //
> // Revision 1.8  2000/08/16 05:59:52  shawn
> // merge P_TOMCAT31 branch
> //
> // Revision 1.7.6.1  2000/08/10 00:03:27  shawn
> // head merge
> //
> // Revision 1.7  2000/07/06 17:42:56  davidsi
> // - Added missing java.lang.reflect.Array.
> //
> // Revision 1.6  2000/07/05 13:33:40  davidsi
> // - Allow null values to be passed during creation.
> // - Handle arrays being passed as the Java objects to be handled as
> //   CIMValues.
> //
> // Revision 1.5  2000/05/11 10:12:20  davidsi
> // - Fixed tabs.
> //
> // Revision 1.4  2000/05/06 17:04:23  davidsi
> // - Modifications to bring classes inline with the SUN WBEM APIs 2.2.
> //
> // Revision 1.3  2000/04/09 13:15:30  davidsi
> // - Switched to StringBuffer (from String) for XML generation.
> //
> // Revision 1.2  2000/02/13 15:09:08  davidsi
> // - Added text to cover re-use of Solaris WBEM APIs.
> //
> // Revision 1.1.1.1  2000/02/06 16:41:03  davidsi
> // Initial version.
> //
> //
> //
> 
> 
> package com.ibm.cim;
> 
> import java.util.Vector;
> import java.io.Serializable;
> import java.net.MalformedURLException;
> import java.lang.reflect.Array;
> 
> import com.ibm.xml.*;
> 
> 
> /**
>  *  represents the value of properties
>  */
> public class CIMValue implements Serializable {
> 
> 	// CIMDataType of the Value
> 	private CIMDataType dataType = null;
> 
> 	// object value
> 	private Object value = null;
> 
> 
> 	public static final CIMValue TRUE = new CIMValue(Boolean.TRUE,
> 											new CIMDataType(CIMDataType.BOOLEAN));
> 	public static final CIMValue FALSE = new CIMValue(Boolean.FALSE,
> 											new CIMDataType(CIMDataType.BOOLEAN));
> 
> 
> 
> 	// constructor
> 	public CIMValue( Object obj, CIMDataType type) {
> 		value = obj;
> 		dataType = type;
> 
> 		if((type.isReferenceType()) && obj != null &&(!(obj instanceof CIMObjectPath))){
> 			throw new ExceptionInInitializerError(
> 				"A ref type must be represented with an object path");
> 
> 		}
> 	}
> 
> //
>     // CIMValue
>     //
>     // Constructs array values
>     //
>     public CIMValue(Vector v,CIMDataType dt) {
> 	if (!dt.isArrayType())
> 	    throw new ExceptionInInitializerError(
> 		"CIMValue created with non array type on wrong constructor ("+v+")");
> 	value=v;
> 
> 	dataType=dt;
>     }
> 	
> 	
> 
> 	 //
>     // CIMValue
>     //
>     // Constructs a value on the basis of the passed object
>     //
>     public CIMValue(Object o) {
> 
> 	if (o==null) {
> 	    value=null;
> 	    dataType=null;
> 	    return;
> 	}
> 
> 	// if it is a Vector, assume (for now) that all elements are
>  	// of the same type & no array type elements are present
> 	if (o instanceof Vector) {
> 	    // objValue & dataType are set to null by default
> 	    Vector v = (Vector)o;
> 	    if (! v.isEmpty()) {
> 		value = v;
> 		Object elementObj = v.elementAt(0);
> 		// if null, assume String type
> 		if (elementObj == null)
> 		    dataType=new CIMDataType(CIMDataType.STRING_ARRAY, v.size());
> 		else {
> 		    int simpleType = CIMDataType.findType(elementObj);
> 		    // if the element object is an invalid type,
> 		    // throw an Exception
> 		    if (simpleType == CIMDataType.INVALID) {
> 			value = null;
> 			throw new ExceptionInInitializerError(
> 				"Vector element is an invalid type object of "
> 				+ elementObj.getClass().getName());
> 		    } else
> 			dataType = new CIMDataType(CIMDataType.findArrayType(simpleType), v.size());
> 		}
> 	    } else {
> 		throw new ExceptionInInitializerError(
> 					"An empty Vector was passed in");
> 	    }
> 	} else {
> 	    Class objClass=o.getClass();
> 	    if ( objClass.isArray() ) {
> 		String arrayClass = objClass.getComponentType().getName();
> 		try {
> 		    CIMDataType cdt=CIMDataType.getDataType(arrayClass);
> 		    dataType=new CIMDataType(
> 				   CIMDataType.findArrayType(cdt.getType()) );
> 		    value=o;
> 		    return;
> 		} catch (CIMException ce) {
> 		    throw new ExceptionInInitializerError(ce.toString());
> 		}
> 	    }
> 	    int type=CIMDataType.findType(o);
> 	    if (type==CIMDataType.INVALID) {
> 		value=null;
> 		dataType=null;
> 		throw new ExceptionInInitializerError(
> 					"Failed to determine CIMDataType of "
> 					+objClass.getName());
> 	    }
> 	    dataType=CIMDataType.getPredefinedType(type);
> 	    value=o;
> 	}
>     }
> 
>     //
>     // CIMValue
>     //
>     // Create a value using a XML definition
>     //
>     // @param node The node containing an XML definition of CIMValue
>     //
>     public CIMValue(org.w3c.dom.Node inNode) {
> 	if (inNode==null) {
> 	    value=null;
> 	    dataType=null;
> 	    return;
> 	}
> 
> 	XMLNode node=XMLNode.getXMLNode(inNode);
> 	int token=node.getToken();
> 	try {
> 	    //
> 	    // Look for <VALUE> elements
> 	    //
> 	    if (token==XMLNode.XML_ELEMENT_VALUE ||
> 		token==XMLNode.XML_ELEMENT_KEYVALUE) {
> 		value=node.getUnescapeText();
> 		if (value == null) {
> 			value = "";
> 		}
> 		dataType=CIMDataType.getPredefinedType(CIMDataType.STRING);
> 		return;
> 	    }
> 	    //
> 	    // Look for <VALUE.ARRAY> elements
> 	    //
> 	    if (token==XMLNode.XML_ELEMENT_VALUE_ARRAY) {
> 		Vector v=new Vector();
> 		XMLNode valueNode=node.getChild();
> 		while(valueNode!=null) {
> 		    if (valueNode.getToken()==XMLNode.XML_ELEMENT_VALUE) {
> 			v.addElement(valueNode.getUnescapeText());
> 		    }
> 		    valueNode=valueNode.getNext();
> 		}
> 		value=v;
> 		dataType=CIMDataType.getPredefinedType(CIMDataType.STRING_ARRAY);
> 		return;
> 	    }
> 	    if (token==XMLNode.XML_ELEMENT_VALUE_REFERENCE) {
> 		XMLNode valueNode=node.getChild();
> 		token=valueNode.getToken();
> 		CIMObjectPath cop;
> 		String result;
> 		if (token==XMLNode.XML_ELEMENT_CLASSPATH
> 		    || token==XMLNode.XML_ELEMENT_LOCALCLASSPATH
> 		    || token==XMLNode.XML_ELEMENT_INSTANCEPATH
> 		    || token==XMLNode.XML_ELEMENT_LOCALINSTANCEPATH )
> 		    cop=new CIMObjectPath(valueNode);
> 		else if (token==XMLNode.XML_ELEMENT_CLASSNAME) {
> 		    cop=new CIMObjectPath();
> 		    cop.setObjectName(node.mustGetAttribute("Name"));
> 		} else if (token==XMLNode.XML_ELEMENT_INSTANCENAME) {
> 		    cop=new CIMObjectPath();
> 		    XMLClass.getInstanceName(valueNode, cop);
> 		} else
> 		    throw new CIMException(CIMException.CIM_ERR_FAILED,
> 			"Attempting to extract object path");
> 		value=cop;
> 		dataType=CIMDataType.getPredefinedType(CIMDataType.REFERENCE);
> 		return;
> 	    }
> 	    throw new java.lang.InstantiationError("Not a value node:" + token);
> 	} catch (CIMException ce) {
> 	    throw new java.lang.InstantiationError(ce.toString());
> 	}
>     }
> 
> 	/**
> 	 *	Determines if CIMValue object is empty or not
> 	 */
> 	public boolean isEmpty(){
> 		return ( dataType == null || value == null);
> 	}
> 
> 	/**
> 	 *	Returns  value 
> 	 */
> 	public Object getValue(){
> 		return value;
> 	}
> 
> 	/**
> 	 *	Returns  CIMDataType of CIMValue 
> 	 */
> 	public CIMDataType getType(){
> 		return dataType;
> 	}
> 
> 
> 	/**
> 	 *	Determines if two CIMValue objects are equal
> 	 */
> 	public boolean equals(Object obj) {
> 		if (! (obj instanceof CIMValue)) {
> 			return false;
> 		}	
> 
> 		CIMValue cvObj = (CIMValue)obj; 
> 
> 		return ((dataType.equals(cvObj.getType())) &&
> 					(value.equals(cvObj.getValue()))); 
> 	}
> 	
> 
> 	/**
> 	 * Returns true if value is an array type
> 	 */
> 	public boolean isArrayValue() {
> 		return (dataType.isArrayType());
> 	}	
> 
> 	// isNullValue
> 	//
> 	// Returns true if the datatype is null
>       //
> 	public boolean isNullValue() {
>         return(dataType==null);
> 	}
> 
> 	// contains
> 	public boolean contains(Object obj) { 
> 		/*Changed by prabhakar, to fix the problem faced with
> 					   * associators*/
> 		if(obj instanceof CIMObjectPath) {   
> 			return(value.toString().equals(obj.toString()));
> 		} else {
> 			return(value.equals(obj));
> 		}
> 	}
> 
>   // toMOF
>     public String toMOF() {
> 	String ret;
> 	int type;
> 
> 	if (value==null)
> 	    return("");
> 
> 	type=dataType.getType();
> 	if (dataType.isArrayType() && value instanceof Vector) {
> 	    int i;
> 	    Vector vect=(Vector)value;
> 	    type=CIMDataType.findSimpleType(type);
> 
> 	    ret=new String("{");
> 	    for(i=0;i<vect.size();i++) {
> 		if (i>0)
> 		     ret=ret.concat(",");
> 		if (type==CIMDataType.STRING)
> 		    ret=ret.concat("\"");
> 		ret=ret.concat(vect.elementAt(i).toString());
> 		if (type==CIMDataType.STRING)
> 		    ret=ret.concat("\"");
> 	    }
> 	    ret=ret.concat("}");
> 	    return(ret);
> 	}
> 	ret=new String();
> 
> 	if (type==CIMDataType.STRING)
> 	    ret=ret.concat("\"");
> 	ret=ret.concat(value.toString());
> 
> 	if (type==CIMDataType.STRING)
> 	    ret=ret.concat("\"");
> 	return(ret);
>     }
> 
>     // toString
>     public String toString() {
> 	String retVal = "null";
> 	// return "null" string (not null reference) if valid
> 	// data is not available
> 
> 	if ((value != null) && (dataType != null)
> 		 && (dataType.getType() != CIMDataType.INVALID)) {
> 	    int inType=dataType.getType();
> 
> 	    if (inType==CIMDataType.REFERENCE) {
> 		if (value instanceof CIMObjectPath)
> 		    retVal = ((CIMObjectPath)value).toString();
> 		    //else, return "null"
> 	    } else if (dataType.isArrayType()) {
> 		Vector v=(Vector)value;
> 		StringBuffer buf = new StringBuffer();
> 		buf.append("[");
> 		for(int i=0;i<v.size();i++) {
> 		    if (v.elementAt(i) != null)
> 			buf.append(v.elementAt(i).toString());
> 		    else
> 			buf.append("null");
> 		    if (i < v.size() - 1)
> 			buf.append(", ");
> 		}
> 		buf.append("]");
> 		retVal = buf.toString();
> 	    } else
> 		retVal = value.toString();
> 	}
> 	return retVal;
>     }
> 
> 	   // toXml
>     //
>    public String toXml() throws MalformedURLException {
> 	//
> 	// If no value, don't even return a <VALUE> element
> 	//
> 	if (value==null)
> 	    return("");
> 
> 	if (dataType.isArrayType()) {
> 	    if (dataType.isReferenceType()) {
> 		throw new MalformedURLException(
> 			"Arrays of references are not valid");
> 	    } else {
> 		StringBuffer arrayValues=new StringBuffer("<VALUE.ARRAY>");
> 		if (value instanceof Vector) {
> 		    Vector vect=(Vector)value;
> 		    for(int i=0;i<vect.size();i++) {
> 			Object obj = vect.elementAt(i);
>                         if( obj == null )
> 			    arrayValues=arrayValues.append("<VALUE></VALUE>");
>                         else
> 			    arrayValues=arrayValues.append("<VALUE>"
> 				+XMLCIM.xmlCIMEscape.XMLEscape(obj.toString())
> 				+"</VALUE>");
> 		    }
> 		} else {
> 		    // assume an array
> 		    int length = Array.getLength(value);
> 		    for(int i=0;i<length;i++) {
> 			Object ele=Array.get(value,i);
> 			if ( ele == null)
> 			    arrayValues=arrayValues.append("<VALUE></VALUE>");
> 			else
> 			    arrayValues=arrayValues.append("<VALUE>"
> 				+XMLCIM.xmlCIMEscape.XMLEscape(ele.toString())
> 				+"</VALUE>");
> 		    }
> 		}
> 		arrayValues=arrayValues.append("</VALUE.ARRAY>");
> 		return(arrayValues.toString());
> 	    }
> 	} else if (dataType.isReferenceType()) {
> 	    CIMObjectPath refObj=(CIMObjectPath)value;
> 	    CIMNameSpace ns=refObj.getFullNameSpace();
> 
> 	    if (ns.getHost()==null)
> 		return ("<VALUE.REFERENCE><LOCALINSTANCEPATH>"+
> 		    ns.toXml(true)+
> 		    refObj.toXml()+
> 		    "</LOCALINSTANCEPATH></VALUE.REFERENCE>");
> 	    else
> 		return ("<VALUE.REFERENCE><INSTANCEPATH>"+
> 		    ns.toXml(false)+
> 		    refObj.toXml()+
> 		    "</INSTANCEPATH></VALUE.REFERENCE>");
> 	}
> 	return("<VALUE>"
> 		    +XMLCIM.xmlCIMEscape.XMLEscape(value.toString())
> 		    +"</VALUE>");
>     }
> 
> }
