1,537c1,527
< //
< // InternalProvider.java
< //
< // The contents of this file are subject to the SNIA Public License Version 1.0
< // (the "License"); you may not use this file except in compliance with the
< // License. You may obtain a copy of the License at
< //
< //      http://www.snia.org/resources/openSource.html
< //
< // Software distributed under the License is distributed on an "AS IS" basis,
< // WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
< // License for the specific language governing rights and limitations
< // under the License.
< //
< // The Original Code is InternalProvider.java
< //
< // The Initial Developer of the Original Code is Viktor Mihajlovski.
< //
< // Contributor(s): David Simons
< //
< //
< // This source uses the Solaris WBEM APIs version 2.2.
< // Synchronization will be maintained with the Solaris WBEM APIS,
< // which are being standardized through the Java Community Process.
< //
< //
< // $Id: InternalProvider.java,v 1.3 2001/08/05 16:04:42 davidsi Exp $
< //
< // $Log: InternalProvider.java,v $
< // Revision 1.3  2001/08/05 16:04:42  davidsi
< // - Fixed to support object paths when passing classnames.
< //
< // Revision 1.2  2001/06/23 13:49:02  davidsi
< // - Refactored ServerNameSpace and separated out repository handling
< //   code (from Viktor Mihajlovski). Please see separate design.txt
< //   for a complete description.
< //
< // Revision 1.7  2001/04/19 07:55:13  mihajlov
< // Added support for localOnly, deep, propertyList, includeQualifier and
< // includeClassOrigin filtering.
< //
< // Revision 1.6  2001/04/11 15:41:19  mihajlov
< // Modifications for usage as the instance repository for static objeczs.
< //
< // Revision 1.5  2001/04/04 09:24:01  mihajlov
< // Intermediate version (based on SNIA CIMOM 0.6) for repository rewrite.
< //
< // Revision 1.4  2001/03/14 13:18:08  mihajlov
< // Added subclass instance enumeration.
< // Fixed assocName=null case.
< //
< // Revision 1.3  2001/02/12 14:05:29  mihajlov
< // Add caller's namespace to returned object paths.
< //
< // Revision 1.2  2001/02/06 14:56:26  mihajlov
< // Added Association Provider Functionality.
< //
< // Revision 1.1  2001/01/07 22:21:53  mihajlov
< // Initial revision
< //
< package org.snia.wbemcmd.cimom;
< 
< import java.util.Hashtable;
< import java.util.Vector;
< import java.util.Enumeration;
< import org.snia.wbem.cim.*;
< import org.snia.wbem.client.CIMOMHandle;
< import org.snia.wbem.provider.InstanceProvider;
< import org.snia.wbem.provider20.AssociatorProvider;
< import org.snia.wbem.provider20.EventProvider;
< import org.snia.wbemcfg.GlobalConfig;
< 
< import org.snia.wbemcmd.indication.*;
< import org.snia.wbemcmd.query.*;
< import org.snia.wbemcmd.utils.*;
< 
< //
< // This is an internal provider based on the CIMOMHandleInstanceStore
< // classes. It is the new interface for ServerNameSpace to access "static"
< // instances (i.e. without "provider" qualifier).
< //
< 
< public class InternalProvider implements InstanceProvider, AssociatorProvider,
<              EventProvider {
< 
<     public static IndicationProcessor irh=null;
<     public static InternalProvider internalProvider=null;
<     public static boolean indicationProcessorAvailable=false;
< 
<     public InternalProvider(CIMOMHandle ch) throws CIMException {
<         initialize(ch);
<     }
< 
<     public void initialize(CIMOMHandle ch) throws CIMException {
<         instanceStores = new Hashtable();
<         cimomHdl=ch;
<         System.err.println("--- InternalProvider() started");
<         irh=IndicationProcessor.irh;
<         IndicationProcessor.cimomHdl=cimomHdl;
<         internalProvider=this;
<     }
< 
<     public void cleanup() throws CIMException {
<         instanceStores=null;
<         cimomHdl=null;
<     }
< 
<     // this returns instance names=object pathes
<     // therefore we have to extract the object paths from the instances
<     // NOTE: would be nice if CIMOMInstanceStore could have a method to
<     //       enumerate the instance names
<     public Vector enumInstances(CIMObjectPath cop,
<                                 boolean deep,
<                                 CIMClass cimClass) throws CIMException  {
<         Vector v=enumInstances(cop, true, cimClass, false);
<         Vector w=new Vector();
<         for (int i=0; i<v.size();i++) {
<             CIMInstance ci=(CIMInstance)v.elementAt(i);
<             CIMObjectPath copInst = new CIMObjectPath(ci.getClassName(),
<                                                       ci.getKeyValuePairs());
<             copInst.setNameSpace(cop.getNameSpace());
<             w.addElement(copInst);
<         }
<         return w;
<     }
< 
<     // this returns the instances
<     public Vector enumInstances(CIMObjectPath cop,
<                                 boolean deep,
<                                 CIMClass cimClass,
<                                 boolean localOnly) throws CIMException {
<         // check whether class name is valid in the parent CIMOM namespace
<         CIMClass cc=cimomHdl.getClass(cop,false);
<         if (cc==null)
<             throw new CIMException(CIMException.CIM_ERR_INVALID_CLASS,
<                                    cop.getObjectName());
<         Vector v=new Vector();
<         String[] propertyList=null;
<         if (!deep) {
<             // must not return properties of child class instances
<             Vector vProps=cc.getAllProperties();
<             propertyList=new String[vProps.size()];
<             for (int i=0; i<vProps.size(); i++) {
<                 CIMProperty cp=(CIMProperty)vProps.elementAt(i);
<                 propertyList[i]=cp.getName();
<             }
<         }
<         getInstancesDeep(v, cop, localOnly, true, true, propertyList);
<         return v;
<     }
< 
<     public CIMInstance getInstance(CIMObjectPath cop,
<                                    CIMClass cimClass,
<                                    boolean localOnly) throws CIMException {
<         CIMOMInstanceStore iStore=getStore(cop.getNameSpace());
<         synchronized(iStore) {
<             if (localOnly)
<                 return iStore.getCIMInstance(cop).localElements();
<             else
<                 return iStore.getCIMInstance(cop);
<         }
<     }
< 
<     public CIMObjectPath createInstance(CIMObjectPath cop,
<                                         CIMInstance cimInstance)
<         throws CIMException {
<         CIMOMInstanceStore iStore=getStore(cop.getNameSpace());
<         synchronized(iStore) {
<             iStore.createInstance(cop,cimInstance);
<         }
< 
<         if (indicationProcessorAvailable)
<            irh.processCreation(cimInstance);
<         return (cop);
<     }
< 
<     public void setInstance(CIMObjectPath cop,CIMInstance cimInstance)
<         throws CIMException {
<         boolean subscriber=false;
<         CIMInstance origInstance=null;
< 
<         if (indicationProcessorAvailable &&
<            irh.eventSubscriberExists(cimInstance,
<               IndicationProcessor.modificationCand)) {
<            subscriber=true;
<            origInstance=getInstance(cop,cimomHdl.getClass(cop,false),false);
<         }
< 
<         CIMOMInstanceStore iStore=getStore(cop.getNameSpace());
<         synchronized(iStore) {
<             iStore.setInstance(cop,cimInstance);
<         }
<         if (subscriber) {
<            irh.processModification(cimInstance,origInstance);
<         }
<     }
< 
<     public void deleteInstance(CIMObjectPath cop) throws CIMException {
< 
<         if (indicationProcessorAvailable) {
<            irh.processDeletion(cop);
<         }
< 
<         CIMOMInstanceStore iStore=getStore(cop.getNameSpace());
<         synchronized(iStore) {
<             iStore.deleteInstance(cop);
<         }
<     }
< 
<     public Vector execQuery(CIMObjectPath op,
<                             String query,
<                             int ql,
<                             CIMClass cc) throws CIMException {
< 
<        WQLParser wp=null;
<        Vector result=new Vector();
< 
<        try {
<           wp=WQLParser.make(query);
< 
<           QueryExp qe=wp.getWhereClause();
<           SelectList sl= wp.getSelectList();
< 
<           String opn=op.getObjectName();
<           Enumeration e=enumInstances(op,true,cc,false).elements();
<           while (e.hasMoreElements()) {
<              CIMInstance ci=(CIMInstance)(e.nextElement());
<              String cln=ci.getClassName();
< 
<              boolean test=qe.apply(ci);
<              if (test) {
<                 if (sl!=null) ci=(CIMInstance)sl.apply(ci);
<                 result.addElement(ci);
<              }
<           }
<        }
<        catch (CIMException e) {
<           if (wp!=null) wp.dispose();
<           throw e;
<        }
< 
<        wp.dispose();
<        return result;
<     }
< 
< 
< 
<     public void authorizeFilter(SelectExp filter,
<                                 String eventType,
<                                 CIMObjectPath classPath,
<                                 String owner) throws CIMException {
<        //System.err.println("--- authorizeFilter()");
< 
<        return;
<     }
< 
<     public boolean mustPoll(SelectExp filter,
<                             String eventType,
<                             CIMObjectPath classPath) throws CIMException {
<        //System.err.println("--- mustPoll()");
<        return false;
<     }
< 
<     public void activateFilter(SelectExp filter,
<                                String eventType,
<                                CIMObjectPath classPath,
<                                boolean firstActivation) throws CIMException {
<        //if (firstActivation)
<        //   System.err.println("--- InternalProvider: activateFilter() first: "
<        //                    +firstActivation+" "+eventType);
<        return;
<     }
< 
<     public void deActivateFilter(SelectExp filter,
<                                  String eventType,
<                                  CIMObjectPath classPath,
<                                  boolean lastActivation) throws CIMException {
<        //if (lastActivation)
<        //   System.err.println("--- InternalProvider: deActivateFilter() last: "
<        //                     +lastActivation+" "+eventType);
<        return;
<     }
< 
< 
<     public Vector associators(CIMObjectPath assocName,
<                               CIMObjectPath pathName,
<                               String resultClass,
<                               String role,
<                               String resultRole,
<                               boolean includeQualifiers,
<                               boolean includeClassOrigin,
<                               String[] propertyList) throws CIMException {
<         return enumRefs(assocName, pathName, resultClass,
<                         role, resultRole, includeQualifiers,
<                         includeClassOrigin, propertyList, ASSOC);
<     }
< 
<     public Vector associatorNames(CIMObjectPath assocName,
<                                   CIMObjectPath pathName,
<                                   String resultClass,
<                                   String role,
<                                   String resultRole) throws CIMException {
<         return enumRefs(assocName, pathName, resultClass,
<                         role, resultRole, true, true, null, ASSOC_NAME);
<     }
< 
<     public Vector references(CIMObjectPath assocName,
<                              CIMObjectPath pathName,
<                              String role,
<                              boolean includeQualifiers,
<                              boolean includeClassOrigin,
<                              String[] propertyList) throws CIMException {
<         return enumRefs(assocName, pathName, null, role, null,
<                         includeQualifiers, includeClassOrigin, propertyList,
<                         REF);
<     }
< 
<     public Vector referenceNames(CIMObjectPath assocName,
<                                  CIMObjectPath pathName,
<                                  String role) throws CIMException {
<         return enumRefs(assocName, pathName, null, role, null,
<                         true, true, null, REF_NAME);
<     }
< 
<     private Vector enumRefs(CIMObjectPath assocName,
<                             CIMObjectPath pathName,
<                             String resultClass,
<                             String role,
<                             String resultRole,
<                             boolean includeQualifiers,
<                             boolean includeClassOrigin,
<                             String[] propertyList,
<                             int associatorFunction) throws CIMException {
<         // first get the association class instances
<         Vector references = new Vector();
<         if (assocName != null) {
<             // the assocName is given - good
<             if (assocForName(assocName,role,resultRole) == null)
<                 throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
<                                        assocName);
<             getInstancesDeep(references, assocName, false,
<                              includeQualifiers, includeClassOrigin,
<                              propertyList);
<         } else {
<             // no assocName - this will hurt (performance)
<             Enumeration classNames =
<                 cimomHdl.enumClass(new CIMObjectPath(null,
<                                                      pathName.getNameSpace()),
<                                    true);
<             while (classNames.hasMoreElements()) {
<                 CIMObjectPath cop = (CIMObjectPath)(classNames.nextElement());
<                 if (assocForName(cop,role,resultRole) != null)
<                     getInstances(references, cop, cop.getObjectName(), false,
<                                  includeQualifiers, includeClassOrigin,
<                                  propertyList);
<             }
<         }
<         if (role != null) {
<             // filter out the associations not matching the role property
<             for (int i=references.size(); i > 0; i--) {
<                 CIMInstance ci = (CIMInstance) references.elementAt(i-1);
<                 if (!ci.getProperty(role).getValue().contains(pathName)) {
<                     references.remove(i-1);
<                 }
<             }
<         } else {
<             // filter out associations not referencing pathName
<             for (int i=references.size(); i > 0; i--) {
<                 CIMInstance ci = (CIMInstance) references.elementAt(i-1);
<                 Vector properties = ci.getProperties();
<                 boolean matched = false;
<                 for (int j=0; j < properties.size(); j++) {
<                     CIMProperty cp = (CIMProperty) properties.elementAt(j);
<                     if (cp.getType().isReferenceType() &&
<                         cp.getValue().contains(pathName)) {
<                         matched = true;
<                         break;
<                     }
<                 }
<                 if (!matched)
<                     references.remove(i-1);
<             }
<         }
<         if (associatorFunction==REF) {
<             return references;
<         } else if (associatorFunction==REF_NAME) {
<             Vector ref_names = new Vector();
<             for (int i=0; i < references.size(); i++) {
<                 CIMInstance ci=(CIMInstance)references.elementAt(i);
<                 CIMObjectPath copRef =
<                     new CIMObjectPath(ci.getClassName(),
<                                       ci.getKeyValuePairs());
<                 copRef.setNameSpace(pathName.getNameSpace());
<                 ref_names.addElement(copRef);
<             }
<             return ref_names;
<         } else {
<             // Use hashtable to avoid dup'd associators
<             Hashtable associators = new Hashtable();
<             for (int i=0; i < references.size(); i++) {
<                 CIMInstance ci=(CIMInstance)references.elementAt(i);
<                 // Q: for ASSOC_NAME we should not require the
<                 // object exist if we go by the book, should we?
<                 // The current approach retrieves the instances
<                 // via the CIMOM handle
<                 if (resultRole != null) {
<                     CIMObjectPath cop = (CIMObjectPath)
<                         ci.getProperty(resultRole).getValue().getValue();
<                     if (!cop.equals(pathName) // don't want self-ref
<                         && isInstanceOf(cop,resultClass))
<                         associators.put(cop, cimomHdl.getInstance(cop,false));
<                 } else {
<                     // must loop over the properties to find ref instances
<                     Vector properties = ci.getProperties();
<                     for (int j=0; j < properties.size(); j++) {
<                         CIMProperty cp = (CIMProperty) properties.elementAt(j);
<                         if (cp.getType().isReferenceType()) {
<                             CIMObjectPath cop =
<                                 (CIMObjectPath) cp.getValue().getValue();
<                             if (!cop.equals(pathName) // don't want self-ref
<                                 && isInstanceOf(cop,resultClass))
<                                 associators.put(cop,
<                                                 cimomHdl.
<                                                 getInstance(cop,false));
<                         }
<                     }
<                 }
<             }
<             Enumeration ae;
<             if (associatorFunction == ASSOC) {
<                 // get instances
<                 ae = associators.elements();
<             } else {
<                 // get object pathes
<                 ae = associators.keys();
<             }
<             Vector assocs = new Vector();
<             while (ae.hasMoreElements()) {
<                 assocs.addElement(ae.nextElement());
<             }
<             return assocs;
<         }
<     }
< 
<     private CIMClass assocForName(CIMObjectPath assocName, String role,
<                                   String resultRole) throws CIMException {
<         // check whether class is valid association in parent CIMOM namespace
<         CIMClass cc = cimomHdl.getClass(assocName,false);
<         if (cc!=null && cc.getQualifier("association") != null &&
<             (role==null || cc.getProperty(role) != null) &&
<             (resultRole==null || cc.getProperty(resultRole) != null))
<             return cc;
<         else
<             return null;
<     }
< 
<     private void getInstancesDeep(Vector v,
<                                   CIMObjectPath cop,
<                                   boolean localOnly,
<                                   boolean includeQualifiers,
<                                   boolean includeClassOrigin,
<                                   String[] propertyList) throws CIMException {
<         Enumeration classNames = cimomHdl.enumClass(cop,true);
<         while (classNames.hasMoreElements())
<             getInstances(v, cop,
<                         ((CIMObjectPath)(classNames.nextElement())).getObjectName(),
<                          localOnly, includeQualifiers, includeClassOrigin,
<                          propertyList);
<     }
< 
<     private void getInstances(Vector v,
<                               CIMObjectPath cop,
<                               String cls,
<                               boolean localOnly,
<                               boolean includeQualifiers,
<                               boolean includeClassOrigin,
<                               String[] propertyList) throws CIMException {
<         // Note: cop is needed to select the correct instance store
<         //       and is usually a parent class of cls
<         CIMOMInstanceStore iStore=getStore(cop.getNameSpace());
<         Enumeration e;
<         synchronized (iStore) {
<             e=getStore(cop.getNameSpace()).getCIMInstances(cls);
<         }
<         while (e.hasMoreElements()) {
<             CIMInstance ci=(CIMInstance)e.nextElement();
<             if (localOnly)
<                 ci=ci.localElements();
<             v.addElement(ci.filterProperties(propertyList,includeQualifiers,
<                                              includeClassOrigin));
<         }
<     }
< 
<     private boolean isInstanceOf(CIMObjectPath instancePath,String className)
<         throws CIMException {
<         if (className == null)
<             return true;
<         if (instancePath.getObjectName().equals(className))
<             return true;
<         CIMObjectPath classPath
<             = new CIMObjectPath(className, instancePath.getNameSpace());
<         Enumeration classes = cimomHdl.enumClass(classPath,true);
<         while (classes.hasMoreElements()) {
<             className = ((CIMObjectPath)(classes.nextElement())).getObjectName();
<             if (instancePath.getObjectName().equals(className))
<                 return true;
<         }
<         return false;
<     }
< 
<     public void addNameSpace(CIMNameSpace cns) {
<         CIMOMInstanceStore instanceStore = getStore(cns.getNameSpace());
<         if (instanceStore == null) {
<             if (GlobalConfig.usePersistence) {
<                 instanceStore=new CIMOMInstanceStoreDisk(cns.getNameSpace());
<             } else {
<                 instanceStore=new CIMOMInstanceStoreMemory();
<             }
<             instanceStores.put(cns.getNameSpace(), instanceStore);
< /* added by prabhakar since put/get are case sensitive */
<             instanceStores.put(cns.getNameSpace().toLowerCase(), instanceStore);
<         }
<     }
< 
<     private CIMOMInstanceStore getStore(String ns) {
<         return (CIMOMInstanceStore) instanceStores.get(ns);
<     }
< 
<     // Different associator function types
<     private static int ASSOC      = 0;
<     private static int ASSOC_NAME = 1;
<     private static int REF        = 2;
<     private static int REF_NAME   = 3;
<     // Use hash table for the name space specific instance stores
<     private Hashtable   instanceStores;
<     private CIMOMHandle cimomHdl;
< }
---
> /*
> static char sccsid[] = "@(#)13  1.1  src/com/ibm/cimom/InternalProvider.java, cimom, ICAT1, ICAT1_1.0.0.7 10/10/02 14:42:41";
> */
> 
> //
> // InternalProvider.java
> //
> // The contents of this file are subject to the SNIA Public License Version 1.0
> // (the "License"); you may not use this file except in compliance with the
> // License. You may obtain a copy of the License at
> //
> //      http://www.snia.org/resources/openSource.html
> //
> // Software distributed under the License is distributed on an "AS IS" basis,
> // WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
> // License for the specific language governing rights and limitations
> // under the License.
> //
> // The Original Code is InternalProvider.java
> //
> // The Initial Developer of the Original Code is Viktor Mihajlovski.
> //
> // Contributor(s): David Simons
> //
> //
> // This source uses the Solaris WBEM APIs version 2.2.
> // Synchronization will be maintained with the Solaris WBEM APIS,
> // which are being standardized through the Java Community Process.
> //
> //
> // $Id: InternalProvider.java,v 1.3 2001/08/05 16:04:42 davidsi Exp $
> //
> // $Log: InternalProvider.java,v $
> // Revision 1.3  2001/08/05 16:04:42  davidsi
> // - Fixed to support object paths when passing classnames.
> //
> // Revision 1.2  2001/06/23 13:49:02  davidsi
> // - Refactored ServerNameSpace and separated out repository handling
> //   code (from Viktor Mihajlovski). Please see separate design.txt
> //   for a complete description.
> //
> // Revision 1.7  2001/04/19 07:55:13  mihajlov
> // Added support for localOnly, deep, propertyList, includeQualifier and
> // includeClassOrigin filtering.
> //
> // Revision 1.6  2001/04/11 15:41:19  mihajlov
> // Modifications for usage as the instance repository for static objeczs.
> //
> // Revision 1.5  2001/04/04 09:24:01  mihajlov
> // Intermediate version (based on SNIA CIMOM 0.6) for repository rewrite.
> //
> // Revision 1.4  2001/03/14 13:18:08  mihajlov
> // Added subclass instance enumeration.
> // Fixed assocName=null case.
> //
> // Revision 1.3  2001/02/12 14:05:29  mihajlov
> // Add caller's namespace to returned object paths.
> //
> // Revision 1.2  2001/02/06 14:56:26  mihajlov
> // Added Association Provider Functionality.
> //
> // Revision 1.1  2001/01/07 22:21:53  mihajlov
> // Initial revision
> //
> package com.ibm.cimom;
> 
> import java.util.Hashtable;
> import java.util.Vector;
> import java.util.Enumeration;
> import com.ibm.cim.*;
> import com.ibm.provider.*;
> import com.ibm.cimom.util.*;
> import com.ibm.query.*;
> import com.ibm.indication.*;
> import com.ibm.cimom.datastore.*;
> 
> //
> // This is an internal provider based on the CIMOMHandleInstanceStore
> // classes. It is the new interface for ServerNameSpace to access "static"
> // instances (i.e. without "provider" qualifier).
> //
> 
> public class InternalProvider implements InstanceProvider, AssociatorProvider,
>              EventProvider {
> 
>     public static InternalProvider internalProvider=null;
> 
>     public InternalProvider(CIMOMHandle ch) throws CIMException {
>         initialize(ch);
>     }
> 
>     public void initialize(CIMOMHandle ch) throws CIMException {
>         instanceStores = new Hashtable();
>         cimomHdl=ch;
>         internalProvider=this;
>     }
> 
>     public void cleanup() throws CIMException {
>         instanceStores=null;
>         cimomHdl=null;
>     }
> 
>     // this returns instance names=object pathes
>     // therefore we have to extract the object paths from the instances
>     // NOTE: would be nice if CIMOMInstanceStore could have a method to
>     //       enumerate the instance names
>     public Vector enumInstances(long tid, CIMObjectPath cop,
>                                 boolean deep,
>                                 CIMClass cimClass) throws CIMException  {
> System.out.println("enumInstances 1");
>         Vector v=enumInstances(tid, cop, true, cimClass, false);
>         Vector w=new Vector();
>         for (int i=0; i<v.size();i++) {
>             CIMInstance ci=(CIMInstance)v.elementAt(i);
>             CIMObjectPath copInst = new CIMObjectPath(ci.getClassName(),
>                                                       ci.getKeyValuePairs());
>             copInst.setNameSpace(cop.getNameSpace());
>             w.addElement(copInst);
>         }
> 
>         return w;
>     }
> 
>     // this returns the instances
>     public Vector enumInstances(long tid, CIMObjectPath cop,
>                                 boolean deep,
>                                 CIMClass cimClass,
>                                 boolean localOnly) throws CIMException {
> 
> System.out.println("enumInstances 2");
>         if (IndicationProcessor.irh != null) {
> try {
> 		String ns = cop.getNameSpace();
> 		CIMClass processIndClass = cimomHdl.getClass(new CIMObjectPath("CIM_ProcessIndication", ns), false);
>        	CIMInstance ci=processIndClass.newInstance();
>        	ci.setProperty("IndicationTime",new CIMValue(new CIMDateTime()));
> 		IndicationProcessor.irh.deliverEvent("CHANGEREPORT", ci);
> } catch (Exception e) {
> System.out.println("FFF");
> e.printStackTrace();
> }
>         }
> 
>         // check whether class name is valid in the parent CIMOM namespace
>         CIMClass cc=cimomHdl.getClass(cop,false);
>         if (cc==null)
>             throw new CIMException(CIMException.CIM_ERR_INVALID_CLASS,
>                                    cop.getObjectName());
>         Vector v=new Vector();
>         String[] propertyList=null;
>         if (!deep) {
>             // must not return properties of child class instances
>             Vector vProps=cc.getAllProperties();
>             propertyList=new String[vProps.size()];
>             for (int i=0; i<vProps.size(); i++) {
>                 CIMProperty cp=(CIMProperty)vProps.elementAt(i);
>                 propertyList[i]=cp.getName();
>             }
>         }
>         getInstancesDeep(v, cop, localOnly, true, true, propertyList);
> 
>         return v;
>     }
> 
>     public CIMInstance getInstance(long tid, CIMObjectPath cop,
>                                    CIMClass cimClass,
>                                    boolean localOnly) throws CIMException {
>         CIMOMInstanceStore iStore=getStore(cop.getNameSpace());
>         synchronized(iStore) {
>             if (localOnly) {
> 			CIMInstance ci = iStore.getCIMInstance(cop);
> 			ci = ci.localElements();
> 			return ci;
> //                return iStore.getCIMInstance(cop).localElements();
>             } else {
>                 return iStore.getCIMInstance(cop);
> 		}
>         }
>     }
> 
>     public CIMObjectPath createInstance(long tid, CIMObjectPath cop,
>                                         CIMInstance cimInstance)
>         throws CIMException {
>         CIMOMInstanceStore iStore=getStore(cop.getNameSpace());
>         synchronized(iStore) {
>             iStore.createInstance(cop,cimInstance);
>         }
> 
> 	  if (IndicationProcessor.irh != null) {
>            IndicationProcessor.irh.processCreation(cimInstance);
> 	  }
> 
>         return (cop);
>     }
> 
>     public void setInstance(long tid,CIMObjectPath cop,CIMInstance cimInstance)
>         throws CIMException {
>         CIMInstance origInstance=null;
> 
>         if (IndicationProcessor.irh != null) {
>            origInstance=getInstance(tid,cop,cimomHdl.getClass(cop,false),false);
>         }
> 
>         CIMOMInstanceStore iStore=getStore(cop.getNameSpace());
>         synchronized(iStore) {
>             iStore.setInstance(cop,cimInstance);
>         }
> 
>         if (IndicationProcessor.irh != null) {
>            IndicationProcessor.irh.processModification(cimInstance,origInstance);
>         }
>     }
> 
>     public void deleteInstance(long tid,CIMObjectPath cop) throws CIMException {
> 
>         if (IndicationProcessor.irh != null) {
>            IndicationProcessor.irh.processDeletion(cop);
>         }
> 
>         CIMOMInstanceStore iStore=getStore(cop.getNameSpace());
>         synchronized(iStore) {
>             iStore.deleteInstance(cop);
>         }
>     }
> 
>     public Vector execQuery(long tid, CIMObjectPath op,String queryStatement,
> 			    int ql, CIMClass cimClass) throws CIMException {
> 	throw new CIMException(CIMException.CIM_ERR_METHOD_NOT_AVAILABLE);
>     }
> 
>     public void authorizeFilter(SelectExp filter,
>                                 String eventType,
>                                 CIMObjectPath classPath,
>                                 String owner) throws CIMException {
>        //System.err.println("--- authorizeFilter()");
> 
>        return;
>     }
> 
>     public boolean mustPoll(SelectExp filter,
>                             String eventType,
>                             CIMObjectPath classPath) throws CIMException {
>        //System.err.println("--- mustPoll()");
>        return false;
>     }
> 
>     public void activateFilter(SelectExp filter,
>                                String eventType,
>                                CIMObjectPath classPath,
>                                boolean firstActivation) throws CIMException {
>        //if (firstActivation)
>        //   System.err.println("--- InternalProvider: activateFilter() first: "
>        //                    +firstActivation+" "+eventType);
>        return;
>     }
> 
>     public void deActivateFilter(SelectExp filter,
>                                  String eventType,
>                                  CIMObjectPath classPath,
>                                  boolean lastActivation) throws CIMException {
>        //if (lastActivation)
>        //   System.err.println("--- InternalProvider: deActivateFilter() last: "
>        //                     +lastActivation+" "+eventType);
>        return;
>     }
> 
> 
>     public Vector associators(long tid,
> 				      CIMObjectPath assocName,
>                               CIMObjectPath pathName,
>                               String resultClass,
>                               String role,
>                               String resultRole,
>                               boolean includeQualifiers,
>                               boolean includeClassOrigin,
>                               String[] propertyList) throws CIMException {
>         return enumRefs(tid, assocName, pathName, resultClass,
>                         role, resultRole, includeQualifiers,
>                         includeClassOrigin, propertyList, ASSOC);
>     }
> 
>     public Vector associatorNames(long tid,	
> 					    CIMObjectPath assocName,
>                                   CIMObjectPath pathName,
>                                   String resultClass,
>                                   String role,
>                                   String resultRole) throws CIMException {
>         return enumRefs(tid, assocName, pathName, resultClass,
>                         role, resultRole, true, true, null, ASSOC_NAME);
>     }
> 
>     public Vector references(long tid,
> 				     CIMObjectPath assocName,
>                              CIMObjectPath pathName,
>                              String role,
>                              boolean includeQualifiers,
>                              boolean includeClassOrigin,
>                              String[] propertyList) throws CIMException {
>         return enumRefs(tid, assocName, pathName, null, role, null,
>                         includeQualifiers, includeClassOrigin, propertyList,
>                         REF);
>     }
> 
>     public Vector referenceNames(long tid,
> 					   CIMObjectPath assocName,
>                                  CIMObjectPath pathName,
>                                  String role) throws CIMException {
>         return enumRefs(tid, assocName, pathName, null, role, null,
>                         true, true, null, REF_NAME);
>     }
> 
>     private Vector enumRefs(long tid,
> 				    CIMObjectPath assocName,
>                             CIMObjectPath pathName,
>                             String resultClass,
>                             String role,
>                             String resultRole,
>                             boolean includeQualifiers,
>                             boolean includeClassOrigin,
>                             String[] propertyList,
>                             int associatorFunction) throws CIMException {
>         // first get the association class instances
>         Vector references = new Vector();
>         if (assocName != null) {
>             // the assocName is given - good
>             if (assocForName(assocName,role,resultRole) == null)
>                 throw new CIMException(CIMException.CIM_ERR_INVALID_PARAMETER,
>                                        assocName);
>             getInstancesDeep(references, assocName, false,
>                              includeQualifiers, includeClassOrigin,
>                              propertyList);
>         } else {
>             // no assocName - this will hurt (performance)
>             Enumeration classNames =
>                 cimomHdl.enumClass(new CIMObjectPath(null,
>                                                      pathName.getNameSpace()),
>                                    true);
>             while (classNames.hasMoreElements()) {
>                 CIMObjectPath cop = (CIMObjectPath)(classNames.nextElement());
>                 if (assocForName(cop,role,resultRole) != null)
>                     getInstances(references, cop, cop.getObjectName(), false,
>                                  includeQualifiers, includeClassOrigin,
>                                  propertyList);
>             }
>         }
>         if (role != null) {
>             // filter out the associations not matching the role property
>             for (int i=references.size(); i > 0; i--) {
>                 CIMInstance ci = (CIMInstance) references.elementAt(i-1);
>                 if (!ci.getProperty(role).getValue().contains(pathName)) {
>                     references.remove(i-1);
>                 }
>             }
>         } else {
>             // filter out associations not referencing pathName
>             for (int i=references.size(); i > 0; i--) {
>                 CIMInstance ci = (CIMInstance) references.elementAt(i-1);
>                 Vector properties = ci.getProperties();
>                 boolean matched = false;
>                 for (int j=0; j < properties.size(); j++) {
>                     CIMProperty cp = (CIMProperty) properties.elementAt(j);
>                     if (cp.getType().isReferenceType() &&
>                         cp.getValue().contains(pathName)) {
>                         matched = true;
>                         break;
>                     }
>                 }
>                 if (!matched)
>                     references.remove(i-1);
>             }
>         }
>         if (associatorFunction==REF) {
>             return references;
>         } else if (associatorFunction==REF_NAME) {
>             Vector ref_names = new Vector();
>             for (int i=0; i < references.size(); i++) {
>                 CIMInstance ci=(CIMInstance)references.elementAt(i);
>                 CIMObjectPath copRef =
>                     new CIMObjectPath(ci.getClassName(),
>                                       ci.getKeyValuePairs());
>                 copRef.setNameSpace(pathName.getNameSpace());
>                 ref_names.addElement(copRef);
>             }
>             return ref_names;
>         } else {
>             // Use hashtable to avoid dup'd associators
>             Hashtable associators = new Hashtable();
>             for (int i=0; i < references.size(); i++) {
>                 CIMInstance ci=(CIMInstance)references.elementAt(i);
>                 // Q: for ASSOC_NAME we should not require the
>                 // object exist if we go by the book, should we?
>                 // The current approach retrieves the instances
>                 // via the CIMOM handle
>                 if (resultRole != null) {
>                     CIMObjectPath cop = (CIMObjectPath)
>                         ci.getProperty(resultRole).getValue().getValue();
>                     if (!cop.equals(pathName) // don't want self-ref
>                         && isInstanceOf(cop,resultClass))
>                         associators.put(cop, cimomHdl.getInstance(cop,false));
>                 } else {
>                     // must loop over the properties to find ref instances
>                     Vector properties = ci.getProperties();
>                     for (int j=0; j < properties.size(); j++) {
>                         CIMProperty cp = (CIMProperty) properties.elementAt(j);
>                         if (cp.getType().isReferenceType()) {
>                             CIMObjectPath cop =
>                                 (CIMObjectPath) cp.getValue().getValue();
>                             if (!cop.equals(pathName) // don't want self-ref
>                                 && isInstanceOf(cop,resultClass))
>                                 associators.put(cop,
>                                                 cimomHdl.
>                                                 getInstance(cop,false));
>                         }
>                     }
>                 }
>             }
>             Enumeration ae;
>             if (associatorFunction == ASSOC) {
>                 // get instances
>                 ae = associators.elements();
>             } else {
>                 // get object pathes
>                 ae = associators.keys();
>             }
>             Vector assocs = new Vector();
>             while (ae.hasMoreElements()) {
>                 assocs.addElement(ae.nextElement());
>             }
>             return assocs;
>         }
>     }
> 
>     private CIMClass assocForName(CIMObjectPath assocName, String role,
>                                   String resultRole) throws CIMException {
>         // check whether class is valid association in parent CIMOM namespace
>         CIMClass cc = cimomHdl.getClass(assocName,false);
>         if (cc!=null && cc.getQualifier("association") != null &&
>             (role==null || cc.getProperty(role) != null) &&
>             (resultRole==null || cc.getProperty(resultRole) != null))
>             return cc;
>         else
>             return null;
>     }
> 
>     private void getInstancesDeep(Vector v,
>                                   CIMObjectPath cop,
>                                   boolean localOnly,
>                                   boolean includeQualifiers,
>                                   boolean includeClassOrigin,
>                                   String[] propertyList) throws CIMException {
>         Enumeration classNames = cimomHdl.enumClass(cop,true);
>         while (classNames.hasMoreElements())
>             getInstances(v, cop,
>                         ((CIMObjectPath)(classNames.nextElement())).getObjectName(),
>                          localOnly, includeQualifiers, includeClassOrigin,
>                          propertyList);
>     }
> 
>     private void getInstances(Vector v,
>                               CIMObjectPath cop,
>                               String cls,
>                               boolean localOnly,
>                               boolean includeQualifiers,
>                               boolean includeClassOrigin,
>                               String[] propertyList) throws CIMException {
>         // Note: cop is needed to select the correct instance store
>         //       and is usually a parent class of cls
>         CIMOMInstanceStore iStore=getStore(cop.getNameSpace());
>         Enumeration e;
>         synchronized (iStore) {
>             e=getStore(cop.getNameSpace()).getCIMInstances(cls);
>         }
>         while (e.hasMoreElements()) {
>             CIMInstance ci=(CIMInstance)e.nextElement();
>             if (localOnly)
>                 ci=ci.localElements();
>             v.addElement(ci.filterProperties(propertyList,includeQualifiers,
>                                              includeClassOrigin));
>         }
>     }
> 
>     private boolean isInstanceOf(CIMObjectPath instancePath,String className)
>         throws CIMException {
>         if (className == null)
>             return true;
>         if (instancePath.getObjectName().equals(className))
>             return true;
>         CIMObjectPath classPath
>             = new CIMObjectPath(className, instancePath.getNameSpace());
>         Enumeration classes = cimomHdl.enumClass(classPath,true);
>         while (classes.hasMoreElements()) {
>             className = ((CIMObjectPath)(classes.nextElement())).getObjectName();
>             if (instancePath.getObjectName().equals(className))
>                 return true;
>         }
>         return false;
>     }
> 
>     public void addNameSpace(CIMNameSpace cns) {
> 				String namespace = Utils.parseFileSeparator(cns.getNameSpace());
>         CIMOMInstanceStore instanceStore = getStore(namespace);
>         if (instanceStore == null) {
> 	        instanceStore=new CIMOMInstanceStore(namespace);
>           instanceStores.put(namespace, instanceStore);
>         }
>     }
> 
>     private CIMOMInstanceStore getStore(String ns) {
> 	  String namespace = Utils.parseFileSeparator(ns); 
>         return (CIMOMInstanceStore) instanceStores.get(namespace);
>     }
> 
>     // Different associator function types
>     private static int ASSOC      = 0;
>     private static int ASSOC_NAME = 1;
>     private static int REF        = 2;
>     private static int REF_NAME   = 3;
>     // Use hash table for the name space specific instance stores
>     private Hashtable   instanceStores;
>     private CIMOMHandle cimomHdl;
> 
> 	public String beginTransaction(long tid, String langTags, String userName, String userPW, int pwEncoding) {
> 		return null;
> 	}
> 
> 	public void endTransaction(long tid) {}
> }
