1,1753c1,2021
< //
< // ServerNameSpace.java
< //
< // The contents of this file are subject to the SNIA Public License Version 1.0
< // (the "License"); you may not use this file except in compliance with the
< // License. You may obtain a copy of the License at
< //
< //      http://www.snia.org/resources/openSource.html
< //
< // Software distributed under the License is distributed on an "AS IS" basis,
< // WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
< // License for the specific language governing rights and limitations
< // under the License.
< //
< // The Original Code is ServerNameSpace.java
< //
< // The Initial Developer of the Original Code is David Simons.
< //
< // Contributor(s):
< // tiags.thiyagarajah@intel.com, Mike LaRocca, Roger Kumpf
< // Adrian Schuur, Viktor Mihajlovski
< //
< //
< // This source uses the Solaris WBEM APIs version 2.2.
< // Synchronization will be maintained with the Solaris WBEM APIS,
< // which are being standardized through the Java Community Process.
< //
< //
< // $Id$
< //
< // $Log$
< // Revision 1.41  2001/08/05 16:03:34  davidsi
< // - Further fixes to handle using object paths rather than class names.
< //
< // Revision 1.40  2001/08/05 11:26:49  davidsi
< // - From Quan Wang. When there is no property provider, the instance that
< //   we set the value on (prior to calling back to the instance provider)
< //   must be a clone otherwise we are (potenially) modifying the real
< //   instance before the provider has had a chance to validate it (etc).
< //
< // Revision 1.39  2001/07/23 21:22:17  davidsi
< // - From Adrian Schuur. Added experimental support for indications.
< //
< // Revision 1.38  2001/07/22 13:40:14  davidsi
< // - Modify the handling of classnames when extracted from the object path.
< //   There is a chance a NOT_SET value will be found, therefore detect
< //   this and treat it as null.
< //
< // Revision 1.37  2001/07/08 16:36:49  davidsi
< // - Fixed method provider invocation - a null ptr check was being performed
< //   against the hash table rather than the provider itself.
< //
< // Revision 1.36  2001/07/07 16:41:24  davidsi
< // - When handling class enumerations (class instances or names) be prepared
< //   to handle a null object path.
< //
< // Revision 1.35  2001/07/07 15:43:41  davidsi
< // - Modify the enumeration of class names to return object path instances
< //   rather than strings.
< //
< // Revision 1.34  2001/07/07 14:27:35  davidsi
< // - Switch to using CIMProperty for method invocations (but allow
< //   CIMValue based vectors if appropriately configured).
< //
< // Revision 1.33  2001/07/07 13:56:01  davidsi
< // - Fixed construction of paths when handling references/assocations.
< //
< //
< // Revision 1.32  2001/06/23 15:03:51  davidsi
< // - reworkChildClasses() clearly is insufficient but for the moment put
< //   in a workaround suggested by Bob St. Jacques.
< //
< // Revision 1.31  2001/06/23 13:49:02  davidsi
< // - Refactored ServerNameSpace and separated out repository handling
< //   code (from Viktor Mihajlovski). Please see separate design.txt
< //   for a complete description.
< //
< // Revision 1.2  2001/04/11 15:39:20  mihajlov
< // Rewrite for the instance methods: now completely based on providers.
< // The repository is the internal provider now.
< //
< // Revision 1.1  2001/03/14 16:22:07  mihajlov
< // Initial revision
< //
< // Revision 1.29  2001/01/07 16:50:52  davidsi
< // - Fixed value check in get property value.
< // - Added checks for result class when handling associations.
< //
< // Revision 1.28  2001/01/02 19:40:41  davidsi
< // - Fixes to ensure the same reference/association is not reported
< //   multiple times.
< //
< // Revision 1.27  2000/11/21 19:00:16  davidsi
< // - Throw an exception when no method provider exists and an invoke
< //   has been issued.
< //
< // Revision 1.26  2000/11/18 14:27:29  davidsi
< // - Fixed typos from merging.
< //
< // Revision 1.25  2000/11/18 14:08:46  davidsi
< // - Futher null pointer checks.
< //
< // Revision 1.24  2000/11/12 14:33:54  davidsi
< // - Fixed typo from last merge.
< //
< // Revision 1.23  2000/11/11 16:42:00  davidsi
< // - From Viktor Mihajlovski, modified invoke code to check existence
< //   of an instance rather than attempting to get the instance.
< //
< // Revision 1.22  2000/11/11 15:26:31  davidsi
< // - From hfcheng, added various protection for null refs.
< // - Fixed further issues with not correctly detected an instance
< //   provider.
< //
< // Revision 1.21  2000/11/09 17:46:22  davidsi
< // - For Mike LaRocca: Fixed ordering for calling out to instance
< //   providers for getInstance.
< //
< // Revision 1.20  2000/11/06 18:40:44  davidsi
< // - Reconciled Error IDs.
< //
< // Revision 1.19  2000/09/15 17:12:09  davidsi
< // - createInstance() now returns CIMObjectPath.
< //
< // Revision 1.18  2000/09/15 16:14:08  davidsi
< // - No longer call standard enumeration when locating association instances
< //   go direct to find static ones - they shouldn't be served up using
< //   the normal dynamic mechanisms for classes.
< //
< // Revision 1.17  2000/09/15 16:06:50  davidsi
< // - BUG FIX. When finding associations don't call into the providers unless
< //   the class is explictly qualified as an association.
< //
< // Revision 1.16  2000/08/16 05:59:56  shawn
< // merge P_TOMCAT31 branch
< //
< // Revision 1.15.2.1  2000/08/10 00:03:32  shawn
< // head merge
< //
< // Revision 1.14  2000/07/19 16:52:16  davidsi
< // - Modified qualifier propagation so that even if a property is overridden
< //   previous qualifiers that are not re-specified will be applied to it.
< //
< // Revision 1.13  2000/07/11 17:58:28  davidsi
< // - Fixed association traversals.
< // - Added support for plug in storage providers.
< //
< // Revision 1.10  2000/05/18 14:22:24  davidsi
< // - Initial support for associator providers.
< //
< // Revision 1.9  2000/05/11 10:12:03  davidsi
< // - Fixed tabs.
< //
< // Revision 1.8  2000/05/09 20:19:05  davidsi
< // - Now passes CIMOMHandle to providers.
< // - Added support to use instance providers as method providers.
< //
< // Revision 1.7  2000/05/07 14:22:05  davidsi
< // - Added checks to ensure a class is found when locating instances.
< //
< // Revision 1.6  2000/05/07 13:26:59  davidsi
< // - Added additional compression code to ensure qualifiers are compressed
< //   for properties and methods.
< //
< // Revision 1.5  2000/05/06 17:15:33  davidsi
< // - Brought in line with SUN WBEM APIs 2.2
< //
< // Revision 1.4  2000/04/09 13:49:08  davidsi
< // - Removed debug prinln() calls
< //
< // Revision 1.3  2000/04/09 13:47:32  davidsi
< // - Added synchronized statements to support multithreaded operation.
< // - Added call to normalize class path in enumeration of classes.
< //
< // Revision 1.2  2000/02/13 15:09:47  davidsi
< // - Added text to cover re-use of Solaris WBEM APIs.
< //
< // Revision 1.1.1.1  2000/02/06 16:41:03  davidsi
< // Initial version.
< //
< //
< //
< package org.snia.wbemcmd.cimom;
< 
< import java.io.Serializable;
< import java.io.*;
< import java.util.Enumeration;
< import java.util.Vector;
< import java.util.HashSet;
< import java.util.Hashtable;
< import java.util.Collection;
< import org.snia.wbem.client.CIMClient;
< import org.snia.wbem.cim.*;
< import org.snia.wbem.provider.*;
< import org.snia.wbemcmd.utils.SimpleEnum;
< import org.snia.wbemcmd.utils.Log;
< import org.snia.wbemcmd.indication.IndicationProcessor;
< import org.snia.wbemcfg.GlobalConfig;
< import org.snia.wbem.client.CIMOMHandle;
< import org.snia.wbem.provider20.*;
< 
< import org.snia.wbemcmd.query.*;
< 
< //
< // Ultimately the real object manager!
< //
< class ServerNameSpace extends UtilNormalize {
<     public static ServerNameSpace serverNameSpace=null;
<     //
<     // PROVIDER_NAME identifies the qualifier that is used to specify
<     // a dynamic provider
<     //
<     public static IndicationProcessor irh=null;
<     public static boolean indicationProcessorAvailable=false;
< 
<     private static String PROVIDER_NAME = "provider";
< 
<     private static int STRING_TABLE=4096;  // init size for strings
<     private static int DATATYPE_TABLE=256; // init size for CIMDataTypes
< 
<     private CIMNameSpace cimNameSpace;     // the real name space
<     private ProviderMapper pmap=new ProviderMapper();
<     private Hashtable stringStore=new Hashtable(STRING_TABLE);
<     private Hashtable dataTypeStore=new Hashtable(DATATYPE_TABLE);
<     public  CIMOMHandle cimomHdl=null;
<     private CIMOMClassStore classStore;
<     private CIMOMQualifierStore qualifierStore;
< 
<     // provider caches
<     private Hashtable instanceProvider;  // single instance
<     private Hashtable methodProvider;    // methods
<     private Hashtable propertyProvider;  // properties
<     private Hashtable instanceProviders; // instance enumeration
<     private Hashtable assocProviders;    // association traversal
<     private Hashtable eventProvider;     // events
<     private Hashtable indicationHandler; // Indication handler
<     private Hashtable childClassNames;   // HashSets of children
<     private Hashtable allEventProviders;
< 
<     private InstanceProvider internalProvider;
< 
<     private String addString(String name) {
<         String existingName;
< 
<         if (name==null)
<             return(name);
<         synchronized (stringStore) {
<             existingName=(String)stringStore.get(name);
<             if (existingName==null) {
<                 stringStore.put(name,name);
<                 return(name);
<             }
<         }
<         //
<         // Return the string with the same value
<         //
<         return(existingName);
<     }
< 
<     private void compressCIMElement(CIMElement cimElement) {
<         String existingName;
< 
<         String name=cimElement.getName();
<         synchronized (stringStore) {
<             existingName=(String)stringStore.get(name);
< 
<             if (existingName==null) {
<                 stringStore.put(name,name);
<                 return;
<             }
<         }
<         //
<         // Map string to existing stored value
<         //
<         cimElement.setName(existingName);
<     }
< 
< 
<     private CIMDataType compressCIMDataType(CIMDataType dt) {
<         synchronized (dataTypeStore) {
<             CIMDataType cdt=(CIMDataType)dataTypeStore.get(dt);
<             if (cdt==null) {
<                 dataTypeStore.put(dt,dt);
<                 return(dt);
<             }
<             return(cdt);
<         }
<     }
< 
<     private CIMValue compressCIMValue(CIMValue cv) {
<         boolean buildagain=false;
<         if (cv==null)
<             return(null);
< 
<         CIMDataType dt=cv.getType();
<         if (dt!=null) {
<             dt=compressCIMDataType(dt);
<             //
<             // Most times the datatype is compressed so assume it
<             // was and take the hit of rebuilding the CIMValue to
<             // use it.
<             //
<             buildagain=true;
<         }
<         Object value=cv.getValue();
<         if (value instanceof String) {
<             String name=(String)value;
<             synchronized (stringStore) {
<                 String existingName=(String)stringStore.get(name);
<                 if (existingName==null) {
<                     stringStore.put(name,name);
<                     return(new CIMValue(name,dt));
<                 }
<                 return(new CIMValue(existingName,dt));
<             }
<         }
<         if (buildagain)
<             return(new CIMValue(value,dt));
<         return(cv);
<     }
< 
<     private void compressCIMProperty(CIMProperty cp) {
<         compressCIMElement(cp);
<         CIMValue cv=cp.getValue();
<         cv=compressCIMValue(cv);
<         cp.setValue(cv);
<         String oClass=cp.getOriginClass();
<         if (oClass!=null)
<             cp.setOriginClass(addString(oClass));
<         String oProp=cp.getOverridingProperty();
<         if (oProp!=null)
<             cp.setOverridingProperty(addString(oProp));
<         Vector qualifiers=cp.getQualifiers();
<         if (qualifiers!=null) {
<             int tsize=qualifiers.size();
<             for(int i=0;i<tsize;i++) {
<                 CIMQualifier q=(CIMQualifier)(qualifiers.elementAt(i));
<                 compressCIMQualifier(q);
<             }
<         }
<     }
< 
<     private void compressCIMMethod(CIMMethod m) {
<         compressCIMElement(m);
<         Vector qualifiers=m.getQualifiers();
<         if (qualifiers!=null) {
<             int tsize=qualifiers.size();
<             for(int i=0;i<tsize;i++) {
<                 CIMQualifier q=(CIMQualifier)(qualifiers.elementAt(i));
<                 compressCIMQualifier(q);
<             }
<         }
<         // TODO - more could be compressed
<     }
< 
<     private void compressCIMQualifier(CIMQualifier q) {
<         compressCIMElement(q);
<         CIMValue cv=q.getValue();
<         cv=compressCIMValue(cv);
<         q.setValue(cv);
<     }
< 
<     private void compressCIMClass(CIMClass cimClass) {
<         compressCIMElement(cimClass);
<         String superClass=cimClass.getSuperClass();
<         if (superClass!=null) {
<             cimClass.setSuperClass(addString(superClass));
<         }
<         Vector cimProperties=cimClass.getAllProperties();
<         if (cimProperties!=null) {
<             int tsize=cimProperties.size();
<             for(int i=0;i<tsize;i++) {
<                 CIMProperty cp=(CIMProperty)(cimProperties.elementAt(i));
<                 compressCIMProperty(cp);
<             }
<         }
<         Vector cimMethods=cimClass.getAllMethods();
<         if (cimMethods!=null) {
<             int tsize=cimMethods.size();
<             for(int i=0;i<tsize;i++) {
<                 CIMMethod m=(CIMMethod)(cimMethods.elementAt(i));
<                 compressCIMMethod(m);
<             }
<         }
<         Vector qualifiers=cimClass.getQualifiers();
<         if (qualifiers!=null) {
<             int tsize=qualifiers.size();
<             for(int i=0;i<tsize;i++) {
<                 CIMQualifier q=(CIMQualifier)(qualifiers.elementAt(i));
<                 compressCIMQualifier(q);
<             }
<         }
<     }
< 
< 
<     //
<     ////////////////////////////////////////////////////////
<     //
<     // Instance Functions
<     //
<     ////////////////////////////////////////////////////////
<     //
<     // Get the instance provider information - we locate the provider
<     // by traversing up the class hierarchy to find the provider closest
<     // to the class that is identified by the instance
<     //
<     private InstanceProvider getInstanceProvider(CIMObjectPath cop)
<         throws CIMException {
<         return(getInstanceProvider(cop.getObjectName()));
<     }
< 
<     private synchronized InstanceProvider
<         getInstanceProvider(String className) throws CIMException {
<         if (instanceProvider==null) {
<             instanceProvider=new Hashtable();
<             try {
<                InstanceProvider
<                  instancep=(InstanceProvider) (pmap.getProvider(cimomHdl,
<                  "org.snia.wbemcmd.indication.IndicationHandlerHTTP"));
<                instanceProvider.put("cim_indicationhandlerxmlhttp",instancep);
< 
<                instancep=(InstanceProvider) (pmap.getProvider(cimomHdl,
<                     "org.snia.wbemcmd.indication.IndicationHandlerRMI"));
<                instanceProvider.put("cim_indicationhandlerrmi",instancep);
<             }
<             catch (CIMException x) {
<                System.err.println("--- Default Indication Handlers not found"+
<                   " - terminating");
<                System.exit(13);
<             }
<         }
<         if (className!=null)
<             className=className.toLowerCase();
<         InstanceProvider instancep=
<             (InstanceProvider)instanceProvider.get(className);
<         if (instancep==null) {
<             // not already cached -- find the correct provider
<             String cls=className;
<             while (cls!=null) {
<                 cls=cls.toLowerCase();
<                 CIMClass cc=classStore.getCIMClass(cls);
<                 CIMQualifier cq=cc.getQualifier(PROVIDER_NAME);
<                 if (cq!=null) {
<                     instancep=(InstanceProvider)
<                         (pmap.getProvider(cimomHdl,cq));
<                     break;
<                 }
<                 cls=cc.getSuperClass();
<             }
<             if (instancep==null)
<                 instancep=internalProvider;
<             instanceProvider.put(className,instancep);
<         }
<         return instancep;
<     }
< 
<     private Enumeration getInstanceProviders(CIMObjectPath cop)
<         throws CIMException {
<         return getInstanceProviders(cop.getObjectName());
<     }
< 
<     private synchronized Enumeration getInstanceProviders(String className)
<         throws CIMException {
<         if (instanceProviders==null)
<             instanceProviders=new Hashtable();
<         if (className!=null)
<             className=className.toLowerCase();
<         Vector ips=(Vector)instanceProviders.get(className);
<         if (ips==null) {
<             // providers not cached - go down the inheritance tree to
<             // find all the providers
<             ips=new Vector();
<             addInstanceProviders(className,ips);
<             instanceProviders.put(className,ips);
<         }
<         return ips.elements();
<     }
< 
<     // perform recursive descent down the class tree to find all the
<     // providers delivering instances of className
<     // Note: the order is important: derived (more specialized) first
<     private void addInstanceProviders(String className, Vector providerList)
<         throws CIMException {
<         InstanceProvider ip=getInstanceProvider(className);
<         if (!providerList.contains(ip))
<             providerList.insertElementAt(ip,0);
<         Enumeration classChildren=
<             classStore.getClassChildren(className).elements();
<         while (classChildren.hasMoreElements())
<             addInstanceProviders((String) classChildren.nextElement(),
<                                  providerList);
<     }
< 
<     // find matching indication handler
<     public synchronized IndicationHandler getIndicationHandler(String className)
<         throws CIMException
<     {
<         if (indicationHandler==null)
<             indicationHandler=new Hashtable();
<         if (className!=null)
<             className=className.toLowerCase();
<         IndicationHandler indhp=
<             (IndicationHandler)indicationHandler.get(className);
<         if (indhp==null) {
< 
<            try {
<               indhp=(IndicationHandler)
<                  getInstanceProvider(new CIMObjectPath(className));
<            } catch (java.lang.ClassCastException ce) {
<              //   Not an indication handler
<            }
<            catch (CIMException e) {}
<            if (indhp==null) // indhp=(IndicationHandler)internalProvider;
<               throw new CIMException("Failed to locate handler for "+className);
<            indicationHandler.put(className,indhp);
<         }
<         return indhp;
<     }
< 
<     // find matching event provider
<     public synchronized EventProvider getEventProvider(String className)
<       //  throws CIMException {
<       {
<         getAllEventProviders();
<         if (eventProvider==null)
<             eventProvider=new Hashtable();
<         if (className!=null)
<             className=className.toLowerCase();
<         EventProvider eventp=
<             (EventProvider)eventProvider.get(className);
<         if (eventp==null) {
<            try {
<               eventp=(EventProvider)
<                  getInstanceProvider(new CIMObjectPath(className));
<            } catch (java.lang.ClassCastException ce) {
<              //   Not and event provider
<            }
<            catch (CIMException e) {}
<            if (eventp==null) eventp=(EventProvider)internalProvider;
<            eventProvider.put(className,eventp);
<         }
<         return eventp;
<     }
< 
< 
<     // return all eventProviders
<     public synchronized Hashtable getAllEventProviders()
<         // throws CIMException
<     {
<        if (allEventProviders!=null) return allEventProviders;
<        allEventProviders=new Hashtable();
<        Enumeration allClasses=null;
<        try {
<           allClasses=classStore.getAllClasses();
<        }
<        catch (Exception e) { return null; }
< 
<        while (allClasses.hasMoreElements()) {
<           CIMClass cc = (CIMClass) allClasses.nextElement();
<           CIMQualifier cq=cc.getQualifier(PROVIDER_NAME);
<           ClassLoader clsl=ClassLoader.getSystemClassLoader();
<           if (cq!=null) try {
<              String epn=(String)cq.getValue().getValue();
<              Class cls=Class.forName(epn,false,clsl);
<              Class ifs[]=cls.getInterfaces();
<              for (int i=0,s=ifs.length; i<s; i++) {
<             //    System.err.println("--- ifs: "+ifs[i].getName());
<                 if (ifs[i].getName().endsWith(".EventProvider")) {
<                    EventProvider eventp=(EventProvider)
<                       (pmap.getProvider(cimomHdl,cq));
<                    allEventProviders.put(epn,eventp);
<                    break;
<                 }
<              }
<           }
<           catch (Exception e) {
<              e.printStackTrace();
<           }
<        }
<        return allEventProviders;
<     }
< 
<     // find matching method provider
<     private synchronized MethodProvider getMethodProvider(CIMObjectPath cop,
<                                                           String methodName)
<         throws CIMException {
<         if (methodProvider==null)
<             methodProvider=new Hashtable();
<         String className=cop.getObjectName();
<         if (className!=null)
<             className=className.toLowerCase();
<         MethodProvider methodp=
<             (MethodProvider)methodProvider.get(className);
<         if (methodp==null) {
<             // not already cached -- find the correct provider
<             String cls=className;
<             while (cls!=null) {
<                 cls=cls.toLowerCase();
<                 CIMClass cc=classStore.getCIMClass(cls);
<                 CIMMethod cm=cc.getMethod(methodName);
<                 if (cm==null)
<                     return null; // method not defined for this class+parents
<                 CIMQualifier cq=cm.getQualifier(PROVIDER_NAME);
<                 if (cq!=null) {
<                     methodp=(MethodProvider)
<                         (pmap.getProvider(cimomHdl,cq));
<                     break;
<                 }
<                 cls=cc.getSuperClass();
<             }
<         }
<         return methodp;
<     }
< 
<     // find property provider
<     private synchronized PropertyProvider
<         getPropertyProvider(CIMObjectPath cop,
<                             String propertyName) throws CIMException {
<         if (propertyProvider==null)
<             propertyProvider=new Hashtable();
<         String className=cop.getObjectName();
<         if (className!=null)
<             className=className.toLowerCase();
<         PropertyProvider propertyp=
<             (PropertyProvider)propertyProvider.get(className);
<         if (propertyp==null) {
<             // not already cached -- find the correct provider
<             String cls=className;
<             while (cls!=null) {
<                 cls=cls.toLowerCase();
<                 CIMClass cc=classStore.getCIMClass(cls);
<                 CIMProperty cp=cc.getProperty(propertyName);
<                 if (cp==null)
<                     return null; // property not defined for this class+parents
<                 CIMQualifier cq=cp.getQualifier(PROVIDER_NAME);
<                 if (cq!=null) {
<                     propertyp=(PropertyProvider)
<                         (pmap.getProvider(cimomHdl,cq));
<                     break;
<                 }
<                 cls=cc.getSuperClass();
<             }
<         }
<         return propertyp;
<     }
< 
<     private synchronized void flushProviderCache() {
<       //  System.err.println("--- flushing caches");
<         instanceProvider=null;
<         methodProvider=null;
<         propertyProvider=null;
<         instanceProviders=null;
<         assocProviders=null;
<         eventProvider=null;
<         childClassNames=null;
<         allEventProviders=null;
<     }
< 
<     public synchronized HashSet getChildClassNames(String base)
<           throws CIMException {
<         HashSet chSet=null;
<         String baseClass="";
<         if (childClassNames==null) childClassNames=new Hashtable();
<         if (base!=null) baseClass=base.toLowerCase();
<         chSet=(HashSet)childClassNames.get(baseClass);
<         if (chSet==null) {
<            Vector cv=classStore.getClassChildren(baseClass);
<            int i,s=cv.size();
<            chSet=new HashSet(s+1);
<            for (i=0; i<s; i++)
<               chSet.add(((String)cv.elementAt(i)).toLowerCase());
<            childClassNames.put(baseClass,chSet);
<         }
<         return chSet;
<     }
< 
< 
< 
<     //
<     //Instance related
<     //
<     public CIMObjectPath addCIMInstance(CIMObjectPath path,
<                                         CIMInstance ci) throws CIMException {
<         //
<         // Check that this instance has keys associated with it
<         //
<         Vector keys=path.getKeys();
<         if (keys==null || keys.size()==0) {
<             throw new CIMInstanceException(CIMInstanceException.
<                                            CIM_ERR_NO_KEYS);
<         }
< 
<         CIMObjectPath npath=getInstanceProvider(path).createInstance(path,ci);
<         // irh.processCreation(cimomHdl,npath,ci);
<         return npath;
<     }
< 
<     public void setInstance(CIMObjectPath path,CIMInstance cimInstance)
<         throws CIMException {
<         //
<         // check keys aren't being changed..
<         //
<         CIMInstance origInstance=getInstance(path,false);
<         Vector keys=origInstance.getKeyValuePairs();
< 
<         for(int i=0;i<keys.size();i++) {
<             CIMProperty property=(CIMProperty)(keys.elementAt(i));
<             CIMProperty newProperty=
<                 cimInstance.getProperty(property.getName());
< 
<             if (newProperty==null)
<                 throw new CIMInstanceException(CIMInstanceException.
<                                                CIM_ERR_MISSING_KEY);
<             CIMValue cv1=newProperty.getValue();
<             if (cv1==null)
<                 throw new CIMInstanceException(CIMInstanceException.
<                                                CIM_ERR_MISSING_KEY);
<             CIMValue cv2=property.getValue();
<             if (cv2==null)
<                 throw new CIMInstanceException(CIMInstanceException.
<                                                CIM_ERR_MISSING_KEY);
<             //
<             // Don't let the key's be modified
<             //
<             if (!cv1.equals(cv2))
<                 throw new CIMInstanceException(CIMInstanceException.
<                                                CIM_ERR_KEY_CANNOT_MODIFY);
<         }
<         getInstanceProvider(path).setInstance(path,cimInstance);
<     }
< 
<     public CIMInstance getInstance(CIMObjectPath cop,boolean localOnly)
<         throws CIMException {
<         return getInstanceProvider(cop).
<             getInstance(cop,classStore.getCIMClass(cop.getObjectName()),
<                         localOnly);
<     }
< 
<     public void deleteInstance(CIMObjectPath path)throws CIMException{
<         getInstanceProvider(path).deleteInstance(path);
<     }
< 
<     //
<     // Returns the names of all instances for this class
<     //
<     public Enumeration enumInstances(CIMObjectPath path,
<                                      boolean deep) throws CIMException{
<         SimpleEnum se=new SimpleEnum();
<         Enumeration ips=getInstanceProviders(path);
<         while (ips.hasMoreElements()) {
<             InstanceProvider ip=(InstanceProvider)ips.nextElement();
<             Enumeration e=
<                 ip.enumInstances(path,deep,
<                                  classStore.
<                                  getCIMClass(path.getObjectName())).elements();
<             while (e.hasMoreElements()) {
<                 se.addElement(e.nextElement());
<             }
<         }
<         return(se);
<     }
< 
<     //
<     // Returns all instances (*not just names*) for this class
<     //
<     public Enumeration enumInstances(CIMObjectPath path, boolean deep,
<                                      boolean localOnly) throws CIMException {
<         SimpleEnum se=new SimpleEnum();
<         Enumeration ips=getInstanceProviders(path);
<         while (ips.hasMoreElements()) {
<             InstanceProvider ip=(InstanceProvider)ips.nextElement();
<             Enumeration e=
<                 ip.enumInstances(path,deep,
<                                  classStore.getCIMClass(path.getObjectName()),
<                                  localOnly).elements();
<             while (e.hasMoreElements()) {
<                 se.addElement(e.nextElement());
<             }
<         }
<         return(se);
<     }
< 
< 
<     public Enumeration execQuery(CIMObjectPath op,String queryStatement,
<                             int ql) throws CIMException {
<         WQLParser wp=null;
<         NonJoinExp fc=null;
<         SelectList sl=null;
<         QueryExp qe=null;
<         CIMObjectPath qp=null,cop=null;
<         SimpleEnum se=new SimpleEnum();
<         Enumeration ips=null,e=null;
<         InstanceProvider ip=null;
<         CIMInstance ci=null;
<         int n=0;
< 
<         try {
<            wp=WQLParser.make(queryStatement); // parser tree is cached
< 
<            int type=wp.getStmtType();
<            switch (type) {
<            case WQLParser.WQL_SELECT:
<               fc=(NonJoinExp)wp.getFromClause();
<               sl=wp.getSelectList();
<               qe=wp.getWhereClause();
< 
<               qp=new CIMObjectPath(fc.getTableName());
<               qp.setNameSpace(op.getNameSpace());
<               qp.setKeys(op.getKeys());
< 
<               ips=getInstanceProviders(op);
<               while (ips.hasMoreElements()) {
<                  ip=(InstanceProvider)ips.nextElement();
<                  e=ip.execQuery(qp,queryStatement,ql,
<                       classStore.getCIMClass(qp.getObjectName())).elements();
<                  while (e.hasMoreElements()) {
<                     ci=(CIMInstance)e.nextElement();
<                     se.addElement(ci);
<                  }
<               }
<               break;
< 
<            case WQLParser.WQL_INSERT:
<               ci=wp.getInsertExp().apply(cimomHdl,op);
<               cop=new CIMObjectPath(ci.getClassName(),ci.getKeyValuePairs());
<               cop.setNameSpace(op.getNameSpace());
<               cop=addCIMInstance(cop,ci);
<               se.addElement(ci);
<               break;
< 
<            case WQLParser.WQL_DELETE:
<               fc=(NonJoinExp)wp.getFromClause();
<               qe=wp.getWhereClause();
< 
<               qp=new CIMObjectPath(fc.getTableName());
<               qp.setNameSpace(op.getNameSpace());
<               qp.setKeys(op.getKeys());
< 
<               ips=getInstanceProviders(op);
<               n=0;
<               while (ips.hasMoreElements()) {
<                  ip=(InstanceProvider)ips.nextElement();
<                  e=ip.execQuery(qp,queryStatement,ql,
<                       classStore.getCIMClass(qp.getObjectName())).elements();
<                  while (e.hasMoreElements()) {
<                     ci=(CIMInstance)e.nextElement();
<                     cop=new
<                        CIMObjectPath(ci.getClassName(),ci.getKeyValuePairs());
<                     cop.setNameSpace(op.getNameSpace());
<                     deleteInstance(cop);
<                     n++;
<                  }
<               }
<               if (n==0) throw new CIMException(CIMException.CIM_ERR_NOT_FOUND,
<                   "No objects found for WQL delete");
<               break;
< 
<            case WQLParser.WQL_UPDATE:
<               String cn=wp.getUpdateExp().getClassName();
<               qe=wp.getWhereClause();
< 
<               qp=new CIMObjectPath(cn);
<               qp.setNameSpace(op.getNameSpace());
<               qp.setKeys(op.getKeys());
< 
<               ips=getInstanceProviders(op);
<               n=0;
<               while (ips.hasMoreElements()) {
<                  ip=(InstanceProvider)ips.nextElement();
<                  e=ip.execQuery(qp,queryStatement,ql,
<                       classStore.getCIMClass(qp.getObjectName())).elements();
<                  while (e.hasMoreElements()) {
<                     ci=(CIMInstance)e.nextElement();
<                     ci=wp.getUpdateExp().apply(cimomHdl,ci);
<                     cop=new
<                        CIMObjectPath(ci.getClassName(),ci.getKeyValuePairs());
<                     cop.setNameSpace(op.getNameSpace());
<                     setInstance(cop,ci);
<                     n++;
<                  }
<               }
<               if (n==0) throw new CIMException(CIMException.CIM_ERR_NOT_FOUND,
<                  "No objects found for WQL update");
<               break;
< 
<            default:
<            }
<         }
<         catch (CIMException ex) {
<            if (wp!=null) wp.dispose();
<            throw ex;
<         }
<         wp.dispose();
<         return(se);
<     }
< 
< 
<     //
<     // Invokes a method
<     //
<     // @param path The path of the object on which the method is being invoked
<     //             this can be a class or instance
<     // @param inParams A vector of CIMProperty for inbound values
<     // @param outParams A vector of CIMProperty for outbound values
<     // @throws CIMException
<     //
<     public CIMValue invokeMethod(CIMObjectPath path,String name,
<                                  Vector in, Vector out) throws CIMException {
<         CIMClass cc;
<         CIMMethod method;
<         Vector inParams=null;
< 
< 
<         if (GlobalConfig.callMethodsWithValues) {
<             //
<             // Convert CIMProperty into CIMValue instances for the provider
<             //
<             // TODO -really need to know one way or other but for the
<             // moment support both property/value calls depending
<             // on config setting. Need confirmation if provider API
<             // also changed given changes to invokeMethod on CIMClient.
<             // TODO, what about out bound?
<             //
<             inParams=new Vector();
< 
<             int len=in.size();
<             for(int i=0;i<len;i++) {
<                 CIMProperty cp=(CIMProperty)(in.elementAt(i));
<                 //
<                 // TODO, not good enough may need to reorder these!!
<                 //
<                 inParams.addElement(cp.getValue());
<             }
<         } else {
<             inParams=in;
<         }
< 
<         //
<         // Check for instance existance
<         //
<         getInstance(path,false);
< 
<         cc=classStore.getCIMClass(path.getObjectName());
<         method=cc.getMethod(name);
<         if (method==null) {
<             throw new CIMException(CIMException.CIM_ERR_METHOD_NOT_FOUND+
<                         "("+name+")");
<         }
<         //
<         // If a provider is defined, go call it directly
<         //
<         MethodProvider methodp=getMethodProvider(path,name);
<         if (methodp==null) {
<             //
<             // Look to see if class instance provider can act as
<             // a method provider
<             //
<             try {
<                 methodp=(MethodProvider)getInstanceProvider(path);
<                 return methodp.invokeMethod(path,name,inParams,out);
<             } catch (java.lang.ClassCastException ce) {
<                 // may not be a method provider, so be silent
<             }
<         } else
<             return methodp.invokeMethod(path,name,inParams,out);
< 
<         throw new CIMException(CIMException.CIM_ERR_FAILED,
<                                "No method provider");
<     }
< 
<     public CIMValue getProperty(CIMObjectPath path,
<                                 String propertyName) throws CIMException {
<         if (propertyName==null)
<             throw new CIMException(CIMException.CIM_ERR_NOT_FOUND,
<                                    "Could not find property "+propertyName);
< 
<         PropertyProvider pp=getPropertyProvider(path,propertyName);
<         if (pp==null) {
<             // no property provider use instance provider
<             CIMInstance ci=getInstance(path,false);
<             return ci.getProperty(propertyName).getValue();
<         } else {
<             return pp.getPropertyValue(path,path.getObjectName(),propertyName);
<         }
<     }
< 
<     public void setProperty(CIMObjectPath path,
<                             String propertyName,
<                             CIMValue newValue) throws CIMException {
<         CIMClass cc=classStore.getCIMClass(path.getObjectName());
<         CIMProperty cp=cc.getProperty(propertyName);
<         if (cp==null)
<             throw new CIMException(CIMException.CIM_ERR_NOT_FOUND,
<                                    "Could not find property "+propertyName);
<         if (cp.getQualifier("KEY")!=null) {
<             //
<             // This is a key, see if the value is different
<             //
<             CIMValue cv1=getProperty(path,propertyName);
<             if (!cv1.equals(newValue)) {
<                 // Trying to change value of key
<                 throw new CIMInstanceException(CIMInstanceException.
<                                                CIM_ERR_KEY_CANNOT_MODIFY);
<             }
<         } else {
<             CIMInstance pinstance=null;
<             boolean doFltr=false;
< 
<             PropertyProvider pp=getPropertyProvider(path,propertyName);
< 
<             if (pp==null) {
<                 // no property provider use instance provider
<                 CIMInstance ci=getInstance(path,false);
<                 CIMInstance ciClone = cc.newInstance();
<                 Vector v1 = ci.getProperties();
<                 Vector v2 = (Vector)v1.clone();
<                 ciClone.setProperties(v2);
< 
<                 ciClone.setProperty(propertyName,newValue);
<                 setInstance(path,ciClone);
< 
<             } else {
<                 pp.setPropertyValue(path,path.getObjectName(),
<                                     propertyName,newValue);
<             }
<         }
<     }
< 
<     //
<     // obtain association providers
<     //
< 
<     private AssociatorProvider getAssociatorProvider(String className)
<         throws CIMException {
<         AssociatorProvider associationp=null;
<         String cls=className;
<         while (cls!=null) {
<             cls=cls.toLowerCase();
<             CIMClass cc=classStore.getCIMClass(cls);
<             CIMQualifier cq=cc.getQualifier(PROVIDER_NAME);
<             if (cq!=null) {
<                 try {
<                     associationp=(AssociatorProvider)
<                         (pmap.getProvider(cimomHdl,cq));
<                     break;
<                 } catch(CIMException ce) {
<                     // no association provider, must continue
<                 }
<             }
<             cls=cc.getSuperClass();
<         }
<         if (associationp==null)
<             associationp=(AssociatorProvider)internalProvider;
<         return associationp;
<     }
< 
<     private Enumeration getAssociatorProviders(CIMObjectPath cop)
<         throws CIMException {
<         String className;
<         if (cop==null)
<             className="$$$ALL-ASSOC$$$"; // if no assoc class given
<         else
<             className=cop.getObjectName();
<         if (assocProviders==null)
<             assocProviders=new Hashtable();
<         Vector aps=(Vector)assocProviders.get(className);
<         if (aps==null) {
<             // providers not cached - go down the inheritance tree to
<             // find all the providers
<             aps=new Vector();
<             addAssociatorProviders(className,aps);
<             assocProviders.put(className,aps);
<         }
<         return aps.elements();
<     }
< 
<     // perform recursive decent down the class tree to find all the
<     // providers delivering associations
<     // Note: the order is important: derived (more specialized) first
<     private void addAssociatorProviders(String className,
<                                         Vector providerList)
<         throws CIMException {
<         Enumeration classChildren;
<         if (!className.equals("$$$ALL-ASSOC$$$")) {
<             AssociatorProvider ap=getAssociatorProvider(className);
<             if (!providerList.contains(ap))
<                 providerList.insertElementAt(ap,0);
<             classChildren=classStore.getClassChildren(className).elements();
<         } else
<             classChildren=associationClassNames();
<         while (classChildren.hasMoreElements())
<             addAssociatorProviders((String) classChildren.nextElement(),
<                                     providerList);
<     }
< 
<     // return only top-level association classes
<     private Enumeration associationClassNames()
<         throws CIMException {
<         Enumeration allClasses=classStore.getAllClasses();
<         SimpleEnum assocNames=new SimpleEnum();
<         while (allClasses.hasMoreElements()) {
<             CIMClass cc = (CIMClass) allClasses.nextElement();
<             if (cc.getSuperClass()==null &&
<                 cc.getQualifier("association")!=null)
<                 assocNames.addElement(cc.getName());
<         }
<         return assocNames;
<     }
< 
<     //
<     //  associations/references
<     //
< 
<     public Enumeration associatorNames(CIMObjectPath path,
<                                        String assocClass,
<                                        String resultClass,
<                                        String role,
<                                        String resultRole) throws CIMException {
<         SimpleEnum se=new SimpleEnum();
<         CIMObjectPath assocPath=null;
<         if (assocClass!=null)
<             assocPath=new CIMObjectPath(assocClass,path.getNameSpace());
<         Enumeration aps=getAssociatorProviders(assocPath);
<         while (aps.hasMoreElements()) {
<             AssociatorProvider ap=(AssociatorProvider)aps.nextElement();
<             Enumeration e=ap.associatorNames(assocPath,path,resultClass,role,
<                                              resultRole).elements();
<             while (e.hasMoreElements()) {
<                 se.addElement(e.nextElement());
<             }
<         }
<         return(se);
<     }
< 
<     public Enumeration associators(CIMObjectPath path,String assocClass,
<                                    String resultClass,String role,
<                                    String resultRole,boolean includeQualifiers,
<                                    boolean includeClassOrigin,
<                                    String propertyList[]) throws CIMException {
<         SimpleEnum se=new SimpleEnum();
<         CIMObjectPath assocPath=null;
<         if (assocClass!=null)
<             assocPath=new CIMObjectPath(assocClass,path.getNameSpace());
<         Enumeration aps=getAssociatorProviders(assocPath);
<         while (aps.hasMoreElements()) {
<             AssociatorProvider ap=(AssociatorProvider)aps.nextElement();
<             Enumeration e=ap.associators(assocPath,path,resultClass,role,
<                                          resultRole,includeQualifiers,
<                                          includeClassOrigin,
<                                          propertyList).elements();
<             while (e.hasMoreElements()) {
<                 se.addElement(e.nextElement());
<             }
<         }
<         return(se);
<     }
< 
<    /**
<     * Locates referencing  names of objects
<     *
<     * @param path the name of the object to which the references should
<     * refer
<     * @param resultClass the classname of the association or null
<     * @param role the name of a property to act as a filter or null
<     */
<     public Enumeration referenceNames(CIMObjectPath path,
<                                       String resultClass,
<                                       String role) throws CIMException {
<         SimpleEnum se=new SimpleEnum();
<         CIMObjectPath assocPath=null;
<         if (resultClass!=null)
<             assocPath=new CIMObjectPath(resultClass,path.getNameSpace());
<         Enumeration aps=getAssociatorProviders(assocPath);
<         while (aps.hasMoreElements()) {
<             AssociatorProvider ap=(AssociatorProvider)aps.nextElement();
<             Enumeration e=ap.referenceNames(assocPath,path,role).elements();
<             while (e.hasMoreElements()) {
<                 se.addElement(e.nextElement());
<             }
<         }
<         return(se);
<     }
< 
<    /**
<     * Locates referencing objects
<     *
<     * @param path the name of the object to which the references should
<     * refer
<     * @param resultClass the classname of the association or null
<     * @param role the name of a property to act as a filter or null
<     * @param includeQualifiers true if qualifiers to be included
<     * @param includeClassOrigin true if class origins to be included
<     * includeClassOrigin is not currently used.
<     * @param propertyList the list of properties to return
<     */
<     public Enumeration references(CIMObjectPath path,
<                                   String resultClass,
<                                   String role,
<                                   boolean includeQualifiers,
<                                   boolean includeClassOrigin,
<                                   String propertyList[]
<                                   ) throws CIMException {
<         SimpleEnum se=new SimpleEnum();
<         CIMObjectPath assocPath=null;
<         if (resultClass!=null)
<             assocPath=new CIMObjectPath(resultClass,path.getNameSpace());
<         Enumeration aps=getAssociatorProviders(assocPath);
<         while (aps.hasMoreElements()) {
<             AssociatorProvider ap=(AssociatorProvider)aps.nextElement();
<             Enumeration e=ap.references(assocPath,path,role,
<                                         includeQualifiers,includeClassOrigin,
<                                         propertyList).elements();
<             while (e.hasMoreElements()) {
<                 se.addElement(e.nextElement());
<             }
<         }
<         return(se);
<     }
< 
<     //
<     ////////////////////////////////////////////////////////
<     //
<     // Qualifier Type Functions
<     //
<     ////////////////////////////////////////////////////////
<     //
<     public synchronized Enumeration enumQualifierTypes() throws CIMException {
<                 return(qualifierStore.getQualifiers());
<     }
< 
<     public synchronized void createQualifierType(CIMObjectPath path,
<                       CIMQualifierType qt) throws CIMException{
<                 qualifierStore.createQualifier(qt);
<     }
< 
<     public synchronized void setQualifierType(CIMObjectPath path,
<                       CIMQualifierType qt) throws CIMException{
<                 qualifierStore.setQualifier(qt);
<     }
< 
< 
<     public synchronized CIMQualifierType getQualifierType(CIMObjectPath path) throws CIMException{
<                 return(qualifierStore.getQualifier(path));
<     }
< 
<     public synchronized void deleteQualifierType(CIMObjectPath path) throws CIMException{
<                 qualifierStore.deleteQualifier(path);
<     }
< 
< 
< 
< 
<     //
<     ////////////////////////////////////////////////////////
<     //
<     // Class Functions
<     //
<     ////////////////////////////////////////////////////////
<     //
< 
<     //
<     // Sets isKeyed if there is a KEY qualifier - just in case
<     // the client didn't do this for us!
<     //
<     private void fixKeys(CIMClass cc) {
<         Vector properties=cc.getAllProperties();
<         int psize=properties.size();
<         for(int i=0;i<psize;i++) {
<             CIMProperty p=((CIMProperty)(properties.elementAt(i)));
<             if (p.isKey()) {
<                 cc.setIsKeyed(true);
<                 break;
<             }
<         }
<     }
< 
< 
<     private boolean methodDefined(CIMClass cc,CIMMethod cimMethod) {
< 
<         Vector methods=cc.getAllMethods();
<         if (methods!=null) {
<             int msize=methods.size();
<             for(int i=0;i<msize;i++) {
<                 CIMMethod mp=(CIMMethod)(methods.elementAt(i));
<                 if (cimMethod.equals(mp))
<                     return(true);
<             }
<         }
<         return(false);
<     }
< 
< 
<     //
<     // Add in parent properties and methods
<     //
<     private void prepareFullClass(CIMClass parentClass,
<                                   CIMClass child) throws CIMException {
< 
<         //
<         // Note, regardless of whether there is an override
<         // this will perform override like behavior - probably
<         // need to add validation code...
<         //
<         Vector properties=parentClass.getAllProperties();
<         if (properties!=null) {
<             int psize=properties.size();
<             for(int i=0;i<psize;i++) {
<                 CIMProperty cp=(CIMProperty) (properties.elementAt(i));
<                 CIMProperty childProp=child.getProperty(cp.getName());
<                 if (childProp==null) {
<                     child.addProperty(cp);
<                 } else {
<                     //
<                     // Propagate any qualifiers that have not been
<                     // re-defined
<                     //
<                     Vector qualifiers=cp.getQualifiers();
<                     if (qualifiers!=null) {
<                         int qualSize=qualifiers.size();
<                         for(int qi=0;qi<qualSize;qi++) {
<                             CIMQualifier qual=(CIMQualifier)(qualifiers.elementAt(qi));
<                             CIMQualifier childQual=childProp.getQualifier(qual.getName());
<                             if (childQual==null) {
<                                 //
<                                 // Qualifier not defined on child property
<                                 // so propagate it
<                                 //
<                                 childProp.addQualifier(qual);
<                             }
<                         }
<                     }
<                 }
<             }
<         }
<         Vector methods=parentClass.getAllMethods();
<         if (methods!=null) {
<             int msize=methods.size();
<             for(int i=0;i<msize;i++) {
<                 CIMMethod cm=(CIMMethod) (methods.elementAt(i)) ;
<                 CIMMethod childMethod=child.getMethod(cm.getName());
<                 if (childMethod==null) {
<                     child.addMethod(cm);
<                 } else {
<                     //
<                     // Propagate any qualifiers that have not been
<                     // re-defined by the method declaration
<                     //
<                     Vector qualifiers=cm.getQualifiers();
<                     Vector newQualifiers=(Vector)(childMethod.getQualifiers().clone());
<                     if (qualifiers!=null) {
<                         int qualSize=qualifiers.size();
<                         for(int qi=0;qi<qualSize;qi++) {
<                             CIMQualifier qual=(CIMQualifier)(qualifiers.elementAt(qi));
<                             CIMQualifier childQual=childMethod.getQualifier(qual.getName());
<                             if (childQual==null) {
<                                 //
<                                 // Qualifier not defined on child
<                                 // method so propagate it
<                                 //
<                                 newQualifiers.addElement(qual);
<                             }
<                         }
<                     childMethod.setQualifiers(newQualifiers);
<                     }
<                 }
<            }
<         }
<     }
< 
< 
<     //
<     // If an origin class is not set, assume it belongs with this class
<     // definition
<     //
<     private void setOriginClass(CIMClass cimClass) throws CIMException {
< 
<         String name=cimClass.getName();
< 
<         Vector properties=cimClass.getAllProperties();
<         if (properties!=null) {
<             int psize=properties.size();
<             for(int i=0;i<psize;i++) {
<                 CIMProperty cp=(CIMProperty) (properties.elementAt(i));
<                 if (cp.getOriginClass()==null) {
<                     cp.setOriginClass(name);
<                 }
<             }
<         }
<         Vector methods=cimClass.getAllMethods();
<         if (methods!=null) {
<             int msize=methods.size();
<             for(int i=0;i<msize;i++) {
<                 CIMMethod cm=(CIMMethod) (methods.elementAt(i)) ;
<                 if (cm.getOriginClass()==null)
<                     cm.setOriginClass(name);
<             }
<         }
<     }
< 
< 
<     public synchronized void addCIMClass(CIMObjectPath name,CIMClass cc) throws CIMException {
<         flushProviderCache(); // schema change - reset provider cache
<         String parentName=normalizeClassPath(cc.getSuperClass());
<         CIMClass parentClass=null;              // assume no parent
< 
<         setOriginClass(cc);
< 
<         if (parentName!=null) {
<             parentClass=classStore.getCIMClass(parentName);
<             //
<             // If parent is keyed, then so is this class otherwise
<             // scan the class for key qualifiers
<             //
<             if (parentClass.isKeyed())
<                 cc.setIsKeyed(true);
<             else
<                 fixKeys(cc);
<             prepareFullClass(parentClass,cc);
<         } else {
<             //
<             // No parent class, need to see if class is keyed
<             //
<             fixKeys(cc);
<         }
< 
< 
<         if (GlobalConfig.compressStrings)
<             compressCIMClass(cc);
<         classStore.createClass(cc);
< 
<         if (indicationProcessorAvailable) {
<            irh.processClsCreation(cc);
<         }
<     }
< 
< 
<     public synchronized void setCIMClass(CIMObjectPath name,CIMClass cc)
<                  throws CIMException {
<         flushProviderCache(); // schema change - reset provider cache
<         String parentName=normalizeClassPath(cc.getSuperClass());
<         CIMClass parentClass=null;              // assume no parent
<         CIMClass origClass=null;
<         boolean subscriber=false;
< 
<         if (indicationProcessorAvailable &&
<            irh.eventSubscriberExists(cc,
<               IndicationProcessor.clsModificationCand)) {
<            subscriber=true;
<            origClass=classStore.getCIMClass(name.getObjectName());
<         }
< 
<         setOriginClass(cc);
< 
<         if (parentName!=null) {
<             parentClass=classStore.getCIMClass(parentName);
<             //
<             // If parent is keyed, then so is this class otherwise
<             // scan the class for key qualifiers
<             //
<             if (parentClass.isKeyed())
<                 cc.setIsKeyed(true);
<             else
<                 fixKeys(cc);
<             prepareFullClass(parentClass,cc);
<         } else {
<             //
<             // No parent class, need to see if class is keyed
<             //
<             fixKeys(cc);
<         }
< 
< 
<         if (GlobalConfig.compressStrings)
<             compressCIMClass(cc);
< 
<         //
<         // Get original children - they'll need revising!
<         //
<         Vector origChildren=classStore.getClassChildren(cc.getName());
< 
<         classStore.setClass(cc);
< 
<         //
<         // Now move the children from the original class definition, updating
<         // inheirited methods, properties etc...
<         //
<         if (origChildren!=null) {
<             int childSize=origChildren.size();
<             for(int i=0;i<childSize;i++) {
<                         String childName=(String)origChildren.elementAt(i);
<                         CIMClass childClass=classStore.getCIMClass(childName);
<                 reworkChildClass(cc,childClass);
<             }
<         }
< 
<         if (subscriber) {
<            irh.processClsModification(origClass,cc);
<         }
<     }
< 
<     private void reworkChildClass(CIMClass parentClass,CIMClass childClass)
<             throws CIMException {
<         //
<         // Remove all properties and methods which aren't defined by this
<         // child
<         //
<         String className=childClass.getName();
<         Vector properties=childClass.getAllProperties();
<         Vector newProperties=new Vector();
<         if (properties!=null) {
<             int psize=properties.size();
<             for(int i=0;i<psize;i++) {
<                 CIMProperty cp=(CIMProperty) (properties.elementAt(i));
<                 String oClass=cp.getOriginClass();
<                 if ( oClass==null ||
<                         oClass.trim().length()==0 ||
<                         oClass.equalsIgnoreCase(className)) {
<                     newProperties.addElement(cp);
<                 }
<             }
<         }
<         Vector methods=childClass.getAllMethods();
<         Vector newMethods=new Vector();
<         if (methods!=null) {
<             int msize=methods.size();
<             for(int i=0;i<msize;i++) {
<                 CIMMethod cm=(CIMMethod) (methods.elementAt(i)) ;
<                 String oClass=cm.getOriginClass();
<                 if ( oClass==null ||
<                         oClass.trim().length()==0 ||
<                         oClass.equalsIgnoreCase(className))
<                     newMethods.addElement(cm);
<             }
<         }
<         childClass.setProperties(newProperties);
<         childClass.setMethods(newMethods);
< 
<         //
<         // Add properties/methods from parent
<         //
<         if (parentClass!=null)
<             prepareFullClass(parentClass,childClass);
< 
<         //
<         // Store the class back
<         //
<         classStore.setClass(childClass);
< 
<         //
<         // Process children of this class
<         //
<         Vector childList=classStore.getClassChildren(childClass.getName());
<         int childSize=childList.size();
<         for(int i=0;i<childSize;i++) {
<             String childName=(String)childList.elementAt(i);
<             CIMClass newChildClass=classStore.getCIMClass(childName);
<             reworkChildClass(childClass,newChildClass);
<         }
<     }
< 
<     //
<     //
<     public synchronized void deleteClass(CIMObjectPath path)
<                 throws CIMException{
< 
<         boolean subscriber=false;
<         CIMClass origClass=null;
<         String name=path.getObjectName();
< 
<         if (indicationProcessorAvailable &&
<            irh.eventSubscriberExists(name,
<               IndicationProcessor.clsDeletionCand)) {
<            subscriber=true;
<            origClass=classStore.getCIMClass(name);
<         }
< 
<         flushProviderCache(); // schema change - reset provider cache
<         classStore.deleteClass(name);
< 
<         if (subscriber) {
<            irh.processClsDeletion(origClass,name);
<         }
<     }
< 
< 
<     public synchronized CIMClass getCIMClass(String className)
<          throws CIMException {
<         return(classStore.getCIMClass(className));
<     }
< 
<     public synchronized CIMClass getCIMClass(CIMObjectPath path)
<          throws CIMException{
<         return(classStore.getCIMClass(path.getObjectName()));
<     }
< 
<     private void addClassAndChildren(SimpleEnum se,CIMClass cc,
<                                      boolean deep,
<                                      boolean localOnly) {
<         if (localOnly==true)
<             cc=cc.localElements();
<         se.addElement(cc);
<         if (deep==CIMClient.DEEP) {
<             try {
<                 Vector children=classStore.getClassChildren(cc.getName());
<                 for(int i=0;i<children.size();i++) {
<                     String childName=(String)(children.elementAt(i));
<                     CIMClass childClass=classStore.getCIMClass(childName);
<                     addClassAndChildren(se,childClass,deep,localOnly);
<                 }
<             }catch (CIMException ce) {}
<         }
<     }
< 
<     //
<     // returns an enumeration of class instances
<     //
<     public synchronized Enumeration enumClass(CIMObjectPath path,
<                        boolean deep, boolean localOnly) throws CIMException{
<         SimpleEnum se=new SimpleEnum();
< 
<         String wantedClassName=null;
<         if (path != null) {
<             wantedClassName=path.getObjectName();
< 
<             if (wantedClassName!=null &&
<                  (wantedClassName.equals("")
<                         || wantedClassName.equals(CIMObjectPath.NOT_SET))) {
<                 //
<                 // Also, any class
<                 //
<                 wantedClassName=null;
<             }
<         }
< 
<         if (wantedClassName==null) {
<             //
<             // If no name is specified, then its a special case to
<             // get all classes
<             //
<             Enumeration enum=classStore.getAllClasses();
<             return(enum);
<         }
<         try {
<             CIMClass cc=classStore.getCIMClass(wantedClassName);
<             addClassAndChildren(se,cc,deep,localOnly);
<         } catch (CIMException ce) {}
<         return(se);
<     }
< 
< 
< 
<     private void addClassAndChildren(SimpleEnum se,CIMClass cc,
<                                      String nameSpace,
<                                      boolean deep) {
<         String className=cc.getName();
<         CIMObjectPath cop = new CIMObjectPath(className,nameSpace);
<         se.addElement(cop);
<         if (deep==CIMClient.DEEP) {
<             try {
<                 Vector children=classStore.getClassChildren(className);
<                 for(int i=0;i<children.size();i++) {
<                     String childName=(String)(children.elementAt(i));
<                     CIMClass childClass=classStore.getCIMClass(childName);
<                     addClassAndChildren(se,childClass,nameSpace,deep);
<                 }
<             } catch (CIMException ce) {}
<         }
<     }
< 
<     //
<     // returns an enumeration of class paths
<     //
<     public synchronized Enumeration enumClass(CIMObjectPath path,
<                        boolean deep) throws CIMException{
<         SimpleEnum se=new SimpleEnum();
<         String wantedClassName=null;
< 
<         if (path!=null) {
<             wantedClassName=path.getObjectName();
< 
<             if (wantedClassName!=null &&
<                  (wantedClassName.equals("")
<                         || wantedClassName.equals(CIMObjectPath.NOT_SET))) {
<                 //
<                 // Also, any class
<                 //
<                 wantedClassName=null;
<             }
<         }
< 
<         if (wantedClassName==null) {
<             //
<             // If no name is specified, then its a special case to
<             // get all classes
<             //
<             Enumeration enum=classStore.getAllClasses();
<             while(enum.hasMoreElements()) {
<                 String className = ((CIMClass)(enum.nextElement())).getName();
<                 CIMObjectPath cop = new CIMObjectPath(className,
<                                                       path.getNameSpace());
<                 se.addElement(cop);
<             }
<             return(se);
<         }
<         try {
<             CIMClass cc=classStore.getCIMClass(wantedClassName);
<             addClassAndChildren(se,cc,path.getNameSpace(),deep);
<         } catch (CIMException ce) {}
< 
<         return(se);
<     }
< 
< 
<     String getNameSpace() {
<         return(cimNameSpace.getNameSpace());
<     }
< 
<     //
<     // This is available because when we  bootstrap the handle is to the
<     // server but the server is calling the ServerNameSpace ctr and therefore
<     // the namespace can't be found because the server isn't up and running
<     // fully.
<     public void makeHandle() {
<         try {
<             cimomHdl=new CIMOMHandleLocal(cimNameSpace);
<             internalProvider=
<                 ((CIMOMHandleLocal)cimomHdl).getInternalProvider();
<         } catch (CIMException ce) {
<             cimomHdl=null;
<         }
<         if (cimomHdl==null)
<             System.err.println("Could not find:"+cimNameSpace);
<     }
< 
<     ServerNameSpace(CIMNameSpace cns) {
<         cimNameSpace=cns;
<         if (GlobalConfig.usePersistence) {
<                 classStore=new CIMOMClassStoreDisk(cns.getNameSpace());
<                 qualifierStore=new CIMOMQualifierStoreDisk(cns.getNameSpace());
<         } else {
<                 classStore=new CIMOMClassStoreMemory();
<                 qualifierStore=new CIMOMQualifierStoreMemory();
<         }
<         try {
<             cimomHdl=new CIMOMHandleLocal(cns);
<             internalProvider=
<                 ((CIMOMHandleLocal)cimomHdl).getInternalProvider();
<         } catch (CIMException ce) {
<             cimomHdl=null;
<         }
<         serverNameSpace=this;
<     }
< }
---
> /*
> static char sccsid[] = "@(#)18  1.2.1.1  src/com/ibm/cimom/ServerNameSpace.java, cimom, ICAT1, ICAT1_1.1.0.0 12/5/02 18:16:12";
> */
> 
> //
> // ServerNameSpace.java
> //
> // The contents of this file are subject to the SNIA Public License Version 1.0
> // (the "License"); you may not use this file except in compliance with the
> // License. You may obtain a copy of the License at
> //
> //      http://www.snia.org/resources/openSource.html
> //
> // Software distributed under the License is distributed on an "AS IS" basis,
> // WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
> // License for the specific language governing rights and limitations
> // under the License.
> //
> // The Original Code is ServerNameSpace.java
> //
> // The Initial Developer of the Original Code is David Simons.
> //
> // Contributor(s):
> // tiags.thiyagarajah@intel.com, Mike LaRocca, Roger Kumpf
> // Adrian Schuur, Viktor Mihajlovski
> //
> //
> // This source uses the Solaris WBEM APIs version 2.2.
> // Synchronization will be maintained with the Solaris WBEM APIS,
> // which are being standardized through the Java Community Process.
> //
> //
> // 11/05/02 Martinew added translated strings for message log 
> //
> //
> // $Id: ServerNameSpace.java,v 1.4 2002/11/05 18:23:30 martinew Exp $
> //
> // $Log: ServerNameSpace.java,v $
> // Revision 1.4  2002/11/05 18:23:30  martinew
> // Merge with CMVC
> //
> // Revision 1.41  2001/08/05 16:03:34  davidsi
> // - Further fixes to handle using object paths rather than class names.
> //
> // Revision 1.40  2001/08/05 11:26:49  davidsi
> // - From Quan Wang. When there is no property provider, the instance that
> //   we set the value on (prior to calling back to the instance provider)
> //   must be a clone otherwise we are (potenially) modifying the real
> //   instance before the provider has had a chance to validate it (etc).
> //
> // Revision 1.39  2001/07/23 21:22:17  davidsi
> // - From Adrian Schuur. Added experimental support for indications.
> //
> // Revision 1.38  2001/07/22 13:40:14  davidsi
> // - Modify the handling of classnames when extracted from the object path.
> //   There is a chance a NOT_SET value will be found, therefore detect
> //   this and treat it as null.
> //
> // Revision 1.37  2001/07/08 16:36:49  davidsi
> // - Fixed method provider invocation - a null ptr check was being performed
> //   against the hash table rather than the provider itself.
> //
> // Revision 1.36  2001/07/07 16:41:24  davidsi
> // - When handling class enumerations (class instances or names) be prepared
> //   to handle a null object path.
> //
> // Revision 1.35  2001/07/07 15:43:41  davidsi
> // - Modify the enumeration of class names to return object path instances
> //   rather than strings.
> //
> // Revision 1.34  2001/07/07 14:27:35  davidsi
> // - Switch to using CIMProperty for method invocations (but allow
> //   CIMValue based vectors if appropriately configured).
> //
> // Revision 1.33  2001/07/07 13:56:01  davidsi
> // - Fixed construction of paths when handling references/assocations.
> //
> //
> // Revision 1.32  2001/06/23 15:03:51  davidsi
> // - reworkChildClasses() clearly is insufficient but for the moment put
> //   in a workaround suggested by Bob St. Jacques.
> //
> // Revision 1.31  2001/06/23 13:49:02  davidsi
> // - Refactored ServerNameSpace and separated out repository handling
> //   code (from Viktor Mihajlovski). Please see separate design.txt
> //   for a complete description.
> //
> // Revision 1.2  2001/04/11 15:39:20  mihajlov
> // Rewrite for the instance methods: now completely based on providers.
> // The repository is the internal provider now.
> //
> // Revision 1.1  2001/03/14 16:22:07  mihajlov
> // Initial revision
> //
> // Revision 1.29  2001/01/07 16:50:52  davidsi
> // - Fixed value check in get property value.
> // - Added checks for result class when handling associations.
> //
> // Revision 1.28  2001/01/02 19:40:41  davidsi
> // - Fixes to ensure the same reference/association is not reported
> //   multiple times.
> //
> // Revision 1.27  2000/11/21 19:00:16  davidsi
> // - Throw an exception when no method provider exists and an invoke
> //   has been issued.
> //
> // Revision 1.26  2000/11/18 14:27:29  davidsi
> // - Fixed typos from merging.
> //
> // Revision 1.25  2000/11/18 14:08:46  davidsi
> // - Futher null pointer checks.
> //
> // Revision 1.24  2000/11/12 14:33:54  davidsi
> // - Fixed typo from last merge.
> //
> // Revision 1.23  2000/11/11 16:42:00  davidsi
> // - From Viktor Mihajlovski, modified invoke code to check existence
> //   of an instance rather than attempting to get the instance.
> //
> // Revision 1.22  2000/11/11 15:26:31  davidsi
> // - From hfcheng, added various protection for null refs.
> // - Fixed further issues with not correctly detected an instance
> //   provider.
> //
> // Revision 1.21  2000/11/09 17:46:22  davidsi
> // - For Mike LaRocca: Fixed ordering for calling out to instance
> //   providers for getInstance.
> //
> // Revision 1.20  2000/11/06 18:40:44  davidsi
> // - Reconciled Error IDs.
> //
> // Revision 1.19  2000/09/15 17:12:09  davidsi
> // - createInstance() now returns CIMObjectPath.
> //
> // Revision 1.18  2000/09/15 16:14:08  davidsi
> // - No longer call standard enumeration when locating association instances
> //   go direct to find static ones - they shouldn't be served up using
> //   the normal dynamic mechanisms for classes.
> //
> // Revision 1.17  2000/09/15 16:06:50  davidsi
> // - BUG FIX. When finding associations don't call into the providers unless
> //   the class is explictly qualified as an association.
> //
> // Revision 1.16  2000/08/16 05:59:56  shawn
> // merge P_TOMCAT31 branch
> //
> // Revision 1.15.2.1  2000/08/10 00:03:32  shawn
> // head merge
> //
> // Revision 1.14  2000/07/19 16:52:16  davidsi
> // - Modified qualifier propagation so that even if a property is overridden
> //   previous qualifiers that are not re-specified will be applied to it.
> //
> // Revision 1.13  2000/07/11 17:58:28  davidsi
> // - Fixed association traversals.
> // - Added support for plug in storage providers.
> //
> // Revision 1.10  2000/05/18 14:22:24  davidsi
> // - Initial support for associator providers.
> //
> // Revision 1.9  2000/05/11 10:12:03  davidsi
> // - Fixed tabs.
> //
> // Revision 1.8  2000/05/09 20:19:05  davidsi
> // - Now passes CIMOMHandle to providers.
> // - Added support to use instance providers as method providers.
> //
> // Revision 1.7  2000/05/07 14:22:05  davidsi
> // - Added checks to ensure a class is found when locating instances.
> //
> // Revision 1.6  2000/05/07 13:26:59  davidsi
> // - Added additional compression code to ensure qualifiers are compressed
> //   for properties and methods.
> //
> // Revision 1.5  2000/05/06 17:15:33  davidsi
> // - Brought in line with SUN WBEM APIs 2.2
> //
> // Revision 1.4  2000/04/09 13:49:08  davidsi
> // - Removed debug prinln() calls
> //
> // Revision 1.3  2000/04/09 13:47:32  davidsi
> // - Added synchronized statements to support multithreaded operation.
> // - Added call to normalize class path in enumeration of classes.
> //
> // Revision 1.2  2000/02/13 15:09:47  davidsi
> // - Added text to cover re-use of Solaris WBEM APIs.
> //
> // Revision 1.1.1.1  2000/02/06 16:41:03  davidsi
> // Initial version.
> //
> //
> //
> package com.ibm.cimom;
> 
> import java.io.Serializable;
> import java.io.*;
> import java.util.Enumeration;
> import java.util.Vector;
> import java.util.HashSet;
> import java.util.Hashtable;
> import java.util.Collection;
> import com.ibm.cim.*;
> import com.ibm.cimom.*;
> import com.ibm.cimom.util.*;
> import com.ibm.provider.*;
> import com.ibm.query.*;
> import com.ibm.indication.*;
> import com.ibm.http.*;
> import com.ibm.cimom.datastore.*;
> 
> 
> //
> // Ultimately the real object manager!
> //
> public class ServerNameSpace extends UtilNormalize {
>     public static ServerNameSpace serverNameSpace=null;
> 
>     public static IndicationProcessor irh=null;
> 
>     private static String PROVIDER_NAME = "provider";
> 
>     private static int STRING_TABLE=4096;  // init size for strings
>     private static int DATATYPE_TABLE=256; // init size for CIMDataTypes
> 
>     private CIMNameSpace cimNameSpace;     // the real name space
>     private ProviderMapper pmap=new ProviderMapper();
>     private Hashtable stringStore=new Hashtable(STRING_TABLE);
>     private Hashtable dataTypeStore=new Hashtable(DATATYPE_TABLE);
>     public  CIMOMHandle cimomHdl=null;
>     private CIMOMClassStore classStore;
>     private CIMOMQualifierStore qualifierStore;
> 
>     // provider caches
>     private Hashtable instanceProvider;  // single instance
>     private Hashtable methodProvider;    // methods
>     private Hashtable propertyProvider;  // properties
>     private Hashtable instanceProviders; // instance enumeration
>     private Hashtable assocProviders;    // association traversal
>     private Hashtable eventProvider;     // events
>     private Hashtable indicationHandler; // Indication handler
>     private Hashtable childClassNames;   // HashSets of children
>     private Hashtable allEventProviders;
> 
>     public InstanceProvider internalProvider;
> 
>     private static Hashtable defaultRoles = new Hashtable();
> 
>     private String addString(String name) {
>         String existingName;
> 
>         if (name==null)
>             return(name);
>         synchronized (stringStore) {
>             existingName=(String)stringStore.get(name);
>             if (existingName==null) {
>                 stringStore.put(name,name);
>                 return(name);
>             }
>         }
>         //
>         // Return the string with the same value
>         //
>         return(existingName);
>     }
> 
>     private void compressCIMElement(CIMElement cimElement) {
>         String existingName;
> 
>         String name=cimElement.getName();
>         synchronized (stringStore) {
>             existingName=(String)stringStore.get(name);
> 
>             if (existingName==null) {
>                 stringStore.put(name,name);
>                 return;
>             }
>         }
>         //
>         // Map string to existing stored value
>         //
>         cimElement.setName(existingName);
>     }
> 
> 
>     private CIMDataType compressCIMDataType(CIMDataType dt) {
>         synchronized (dataTypeStore) {
>             CIMDataType cdt=(CIMDataType)dataTypeStore.get(dt);
>             if (cdt==null) {
>                 dataTypeStore.put(dt,dt);
>                 return(dt);
>             }
>             return(cdt);
>         }
>     }
> 
>     private CIMValue compressCIMValue(CIMValue cv) {
>         boolean buildagain=false;
>         if (cv==null)
>             return(null);
> 
>         CIMDataType dt=cv.getType();
>         if (dt!=null) {
>             dt=compressCIMDataType(dt);
>             //
>             // Most times the datatype is compressed so assume it
>             // was and take the hit of rebuilding the CIMValue to
>             // use it.
>             //
>             buildagain=true;
>         }
>         Object value=cv.getValue();
>         if (value instanceof String) {
>             String name=(String)value;
>             synchronized (stringStore) {
>                 String existingName=(String)stringStore.get(name);
>                 if (existingName==null) {
>                     stringStore.put(name,name);
>                     return(new CIMValue(name,dt));
>                 }
>                 return(new CIMValue(existingName,dt));
>             }
>         }
>         if (buildagain)
>             return(new CIMValue(value,dt));
>         return(cv);
>     }
> 
>     private void compressCIMProperty(CIMProperty cp) {
>         compressCIMElement(cp);
>         CIMValue cv=cp.getValue();
>         cv=compressCIMValue(cv);
>         cp.setValue(cv);
>         String oClass=cp.getOriginClass();
>         if (oClass!=null)
>             cp.setOriginClass(addString(oClass));
>         String oProp=cp.getOverridingProperty();
>         if (oProp!=null)
>             cp.setOverridingProperty(addString(oProp));
>         Vector qualifiers=cp.getQualifiers();
>         if (qualifiers!=null) {
>             int tsize=qualifiers.size();
>             for(int i=0;i<tsize;i++) {
>                 CIMQualifier q=(CIMQualifier)(qualifiers.elementAt(i));
>                 compressCIMQualifier(q);
>             }
>         }
>     }
> 
>     private void compressCIMMethod(CIMMethod m) {
>         compressCIMElement(m);
>         Vector qualifiers=m.getQualifiers();
>         if (qualifiers!=null) {
>             int tsize=qualifiers.size();
>             for(int i=0;i<tsize;i++) {
>                 CIMQualifier q=(CIMQualifier)(qualifiers.elementAt(i));
>                 compressCIMQualifier(q);
>             }
>         }
>         // TODO - more could be compressed
>     }
> 
>     private void compressCIMQualifier(CIMQualifier q) {
>         compressCIMElement(q);
>         CIMValue cv=q.getValue();
>         cv=compressCIMValue(cv);
>         q.setValue(cv);
>     }
> 
>     private void compressCIMClass(CIMClass cimClass) {
>         compressCIMElement(cimClass);
>         String superClass=cimClass.getSuperClass();
>         if (superClass!=null && superClass.length() != 0) {
>             cimClass.setSuperClass(addString(superClass));
>         }
>         Vector cimProperties=cimClass.getAllProperties();
>         if (cimProperties!=null) {
>             int tsize=cimProperties.size();
>             for(int i=0;i<tsize;i++) {
>                 CIMProperty cp=(CIMProperty)(cimProperties.elementAt(i));
>                 compressCIMProperty(cp);
>             }
>         }
>         Vector cimMethods=cimClass.getAllMethods();
>         if (cimMethods!=null) {
>             int tsize=cimMethods.size();
>             for(int i=0;i<tsize;i++) {
>                 CIMMethod m=(CIMMethod)(cimMethods.elementAt(i));
>                 compressCIMMethod(m);
>             }
>         }
>         Vector qualifiers=cimClass.getQualifiers();
>         if (qualifiers!=null) {
>             int tsize=qualifiers.size();
>             for(int i=0;i<tsize;i++) {
>                 CIMQualifier q=(CIMQualifier)(qualifiers.elementAt(i));
>                 compressCIMQualifier(q);
>             }
>         }
>     }
> 
> 
>     //
>     ////////////////////////////////////////////////////////
>     //
>     // Instance Functions
>     //
>     ////////////////////////////////////////////////////////
>     //
>     // Get the instance provider information - we locate the provider
>     // by traversing up the class hierarchy to find the provider closest
>     // to the class that is identified by the instance
>     //
>     private InstanceProvider getInstanceProvider(CIMObjectPath cop)
>         throws CIMException {
>         return(getInstanceProvider(cop.getObjectName()));
>     }
> 
>     private synchronized InstanceProvider
>         getInstanceProvider(String className) throws CIMException {
>         if (instanceProvider==null) {
>             instanceProvider=new Hashtable();
>         }
>         InstanceProvider instancep=
>             (InstanceProvider)instanceProvider.get(className);
>         if (instancep==null) {
>             // not already cached -- find the correct provider
>             String cls=className;
>             while (cls!=null && (cls.length()!= 0)) {
>                 CIMClass cc=classStore.getCIMClass(cls);
>                 CIMQualifier cq=cc.getQualifier(PROVIDER_NAME);
>                 if (cq!=null) {
>                     instancep=(InstanceProvider)
>                         (pmap.getProvider(cimomHdl,cq));
>                     break;
>                 }
>                 cls=cc.getSuperClass();
>             }
>             if (instancep==null)
>                 instancep=internalProvider;
>             instanceProvider.put(className,instancep);
>         }
>         return instancep;
>     }
> 
>     private Enumeration getInstanceProviders(CIMObjectPath cop)
>         throws CIMException {
>         return getInstanceProviders(cop.getObjectName());
>     }
> 
>     private synchronized Enumeration getInstanceProviders(String className)
>         throws CIMException {
>         if (instanceProviders==null)
>             instanceProviders=new Hashtable();
>         Vector ips=(Vector)instanceProviders.get(className);
>         if (ips==null) {
>             // providers not cached - go down the inheritance tree to
>             // find all the providers
>             ips=new Vector();
>             addInstanceProviders(className,ips);
>             instanceProviders.put(className,ips);
>         }
>         return ips.elements();
>     }
> 
>     // perform recursive descent down the class tree to find all the
>     // providers delivering instances of className
>     // Note: the order is important: derived (more specialized) first
>     private void addInstanceProviders(String className, Vector providerList)
>         throws CIMException {
>         InstanceProvider ip=getInstanceProvider(className);
>         if (!providerList.contains(ip))
>             providerList.insertElementAt(ip,0);
>         Enumeration classChildren=
>             classStore.getClassChildren(className).elements();
>         while (classChildren.hasMoreElements())
>             addInstanceProviders((String) classChildren.nextElement(),
>                                  providerList);
>     }
> 
>     // find matching indication handler
>     public synchronized IndicationHandler getIndicationHandler(String className)
>         throws CIMException
>     {
>         if (indicationHandler==null)
>             indicationHandler=new Hashtable();
>         IndicationHandler indhp=
>             (IndicationHandler)indicationHandler.get(className);
>         if (indhp==null) {
> 
>            try {
>               indhp=(IndicationHandler)
>                  getInstanceProvider(new CIMObjectPath(className));
>            } catch (java.lang.ClassCastException ce) {
>              //   Not an indication handler
>            }
>            catch (CIMException e) {}
>            if (indhp==null) // indhp=(IndicationHandler)internalProvider;
>               throw new CIMException(CIMException.CIM_ERR_FAILED_TO_LOCATE_INDICATION_HANDLER);
>            indicationHandler.put(className,indhp);
>         }
>         return indhp;
>     }
> 
>     // find matching event provider
>     public synchronized EventProvider getEventProvider(String className)
>       //  throws CIMException {
>       {
>         getAllEventProviders();
>         if (eventProvider==null)
>             eventProvider=new Hashtable();
>         EventProvider eventp=
>             (EventProvider)eventProvider.get(className);
>         if (eventp==null) {
>            try {
>               eventp=(EventProvider)
>                  getInstanceProvider(new CIMObjectPath(className));
>            } catch (java.lang.ClassCastException ce) {
>              //   Not and event provider
>            }
>            catch (CIMException e) {}
>            if (eventp==null) eventp=(EventProvider)internalProvider;
>            eventProvider.put(className,eventp);
>         }
>         return eventp;
>     }
> 
> 
>     // return all eventProviders
>     public synchronized Hashtable getAllEventProviders()
>         // throws CIMException
>     {
>        if (allEventProviders!=null) return allEventProviders;
>        allEventProviders=new Hashtable();
>        Enumeration allClasses=null;
>        try {
>           allClasses=classStore.getAllClasses();
>        }
>        catch (Exception e) { return null; }
> 
>        while (allClasses.hasMoreElements()) {
>           CIMClass cc = (CIMClass) allClasses.nextElement();
>           CIMQualifier cq=cc.getQualifier(PROVIDER_NAME);
>           ClassLoader clsl=ClassLoader.getSystemClassLoader();
>           if (cq!=null) try {
>              String epn=(String)cq.getValue().getValue();
>              Class cls=Class.forName(epn,false,clsl);
>              Class ifs[]=cls.getInterfaces();
>              for (int i=0,s=ifs.length; i<s; i++) {
>             //    System.err.println("--- ifs: "+ifs[i].getName());
>                 if (ifs[i].getName().endsWith(".EventProvider")) {
>                    EventProvider eventp=(EventProvider)
>                       (pmap.getProvider(cimomHdl,cq));
>                    allEventProviders.put(epn,eventp);
>                    break;
>                 }
>              }
>           }
>           catch (Exception e) {
>              e.printStackTrace();
>           }
>        }
>        return allEventProviders;
>     }
> 
>     // find matching method provider
>     private synchronized MethodProvider getMethodProvider(CIMObjectPath cop,
>                                                           String methodName)
>         throws CIMException {
>         if (methodProvider==null)
>             methodProvider=new Hashtable();
>         String className=cop.getObjectName();
>         MethodProvider methodp=
>             (MethodProvider)methodProvider.get(className);
>         if (methodp==null) {
>             // not already cached -- find the correct provider
>             String cls=className;
>             while (cls!=null && (cls.length() != 0)) {
>                 CIMClass cc=classStore.getCIMClass(cls);
>                 CIMMethod cm=cc.getMethod(methodName);
>                 if (cm==null)
>                     return null; // method not defined for this class+parents
>                 CIMQualifier cq=cm.getQualifier(PROVIDER_NAME);
>                 if (cq!=null) {
>                     methodp=(MethodProvider)
>                         (pmap.getProvider(cimomHdl,cq));
>                     break;
>                 }
>                 cls=cc.getSuperClass();
>             }
>         }
>         return methodp;
>     }
> 
>     // find property provider
>     private synchronized PropertyProvider
>         getPropertyProvider(CIMObjectPath cop,
>                             String propertyName) throws CIMException {
>         if (propertyProvider==null)
>             propertyProvider=new Hashtable();
>         String className=cop.getObjectName();
>         PropertyProvider propertyp=
>             (PropertyProvider)propertyProvider.get(className);
>         if (propertyp==null) {
>             // not already cached -- find the correct provider
>             String cls=className;
>             while (cls!=null && (cls.length() != 0)) {
>                 CIMClass cc=classStore.getCIMClass(cls);
>                 CIMProperty cp=cc.getProperty(propertyName);
>                 if (cp==null)
>                     return null; // property not defined for this class+parents
>                 CIMQualifier cq=cp.getQualifier(PROVIDER_NAME);
>                 if (cq!=null) {
>                     propertyp=(PropertyProvider)
>                         (pmap.getProvider(cimomHdl,cq));
>                     break;
>                 }
>                 cls=cc.getSuperClass();
>             }
>         }
>         return propertyp;
>     }
> 
>     private synchronized void flushProviderCache() {
>       //  System.err.println("--- flushing caches");
>         instanceProvider=null;
>         methodProvider=null;
>         propertyProvider=null;
>         instanceProviders=null;
>         assocProviders=null;
>         eventProvider=null;
>         childClassNames=null;
>         allEventProviders=null;
>     }
> 
>     public synchronized HashSet getChildClassNames(String base)
>           throws CIMException {
>         HashSet chSet=null;
>         String baseClass="";
>         if (childClassNames==null) childClassNames=new Hashtable();
>         if (base!=null) baseClass=base;
>         chSet=(HashSet)childClassNames.get(baseClass);
>         if (chSet==null) {
>            Vector cv=classStore.getClassChildren(baseClass);
>            int i,s=cv.size();
>            chSet=new HashSet(s+1);
>            for (i=0; i<s; i++)
>               chSet.add(((String)cv.elementAt(i)));
>            childClassNames.put(baseClass,chSet);
>         }
>         return chSet;
>     }
> 
> 
> 
>     /**
>      * Create a new CIM Object.
>      *
>      * NB: This is not protected by the role-based authorization scheme.
>      *
>      * @param path COP to new object
>      * @param ci contents of the new object
>      * @return The CIM object path of the new instance
>      */
>     public CIMObjectPath addCIMInstance(CIMObjectPath path,
>                                         CIMInstance ci) throws CIMException {
>         //
>         // Check that this instance has keys associated with it
>         //
>         Vector keys=path.getKeys();
>         if (keys==null || keys.size()==0) {
>             throw new CIMInstanceException(CIMInstanceException.
>                                            CIM_ERR_NO_KEYS);
>         }
> 
> 	  // Check whether transaction started
> 	  InstanceProvider ip = getInstanceProvider(path);
> 	  long tid = beginTransaction((CIMProvider)ip);
>         CIMObjectPath npath=ip.createInstance(tid,path,ci);
> 
>         return npath;
>     }
> 
>     /**
>      * Set the contents of an existing instance.
>      *
>      * NB: This will filter all properties that the user
>      *     does not have access to.  All other properties
>      *     will be set as expected.
>      *
>      * @param path The CIM Object Path of the instance.
>      * @param cimInstance The new contents of the instance
>      */
> 
>     public void setInstance(CIMObjectPath path,CIMInstance cimInstance)
>         throws CIMException {
> 
>         cimInstance = filterUnauthorizedProperties(cimInstance,
>                                                    RoleLevel.WRITE_QUAL);
> 
>         //
>         // check keys aren't being changed..
>         //
>         CIMInstance origInstance=getInstance(path,false);
>         Vector keys=origInstance.getKeyValuePairs();
> 
>         for(int i=0;i<keys.size();i++) {
>             CIMProperty property=(CIMProperty)(keys.elementAt(i));
>             CIMProperty newProperty=
>                 cimInstance.getProperty(property.getName());
> 
>             if (newProperty==null)
>                 throw new CIMInstanceException(CIMInstanceException.
>                                                CIM_ERR_MISSING_KEY);
>             CIMValue cv1=newProperty.getValue();
>             if (cv1==null)
>                 throw new CIMInstanceException(CIMInstanceException.
>                                                CIM_ERR_MISSING_KEY);
>             CIMValue cv2=property.getValue();
>             if (cv2==null)
>                 throw new CIMInstanceException(CIMInstanceException.
>                                                CIM_ERR_MISSING_KEY);
>             //
>             // Don't let the key's be modified
>             //
>             if (!cv1.equals(cv2))
>                 throw new CIMInstanceException(CIMInstanceException.
>                                                CIM_ERR_KEY_CANNOT_MODIFY);
>         }
> 
> 	  // Check whether transaction started
> 	  InstanceProvider ip = getInstanceProvider(path);
> 	  long tid = beginTransaction((CIMProvider)ip);
> 	  ip.setInstance(tid, path, cimInstance);	  
>     }
> 
>     /**
>      * Get the contents of an existing instance.
>      *
>      * NB: This will filter all properties that the user
>      *     does not have access to.  All other properties
>      *     will be obtained as expected.
>      *
>      * @param cop The CIM Object Path of the instance.
>      * @param localOnly Whether to return all properties, or only
>      *        those in the localInstance (and inherited).
>      */
>     public CIMInstance getInstance(CIMObjectPath cop,boolean localOnly)
>         throws CIMException {
> 
> 	  // Check whether transaction started
> 	  InstanceProvider ip = getInstanceProvider(cop);
> 	  long tid = beginTransaction((CIMProvider)ip);
>       CIMInstance ci = ip.getInstance(tid,cop,classStore.getCIMClass(cop.getObjectName()),
>                         localOnly);
> 
>       return filterUnauthorizedProperties(ci, RoleLevel.READ_QUAL);
>     }
> 
>     /**
>      * Remove an existing instance.
>      *
>      * NB: This functiion does not use the role-based authorization
>      *     mechanism.
>      *
>      * @param path The CIM Object Path of the instance to remove.
>      */
> 
>     public void deleteInstance(CIMObjectPath path)throws CIMException{
> 
> 	  // Check whether transaction started
> 	  InstanceProvider ip = getInstanceProvider(path);
> 	  long tid = beginTransaction((CIMProvider)ip);
>         ip.deleteInstance(tid,path);
> 
>     }
> 
> 
>     /**
>      * Returns the names of all instances for this class.
>      *
>      * NB: This function does not use the role-based authorization
>      *     mechanism.
>      *
>      * @param path The CIM Object Path of the class of instances to
>      *        look for.
>      * @param deep Include sub-classed instances.
>      * @return An enumeration of the (filtered) CIM Instances.
>      */
> 
>     public Enumeration enumInstances(CIMObjectPath path,
>                                      boolean deep) throws CIMException{
>         SimpleEnum se=new SimpleEnum();
>         Enumeration ips=getInstanceProviders(path);
> 	  long tid;
> 
>         while (ips.hasMoreElements()) {
>             InstanceProvider ip=(InstanceProvider)ips.nextElement();
> 	      tid = beginTransaction((CIMProvider)ip);
>             Enumeration e=
>                 ip.enumInstances(tid,path,deep,
>                                  classStore.
>                                  getCIMClass(path.getObjectName())).elements();
> 
>   	      Vector g = new Vector();
> 	      Vector q;
> 	      CIMQualifier cimQual;
> 		CIMValue cv;
> 
> 		// Filter out instances that belong to providers
>             while (e.hasMoreElements()) {
>             	CIMInstance ci=(CIMInstance)e.nextElement();
> 
> 			if (ip == internalProvider) {
> 
> 			q = ci.getQualifiers();
> 			cimQual = null;
> 			if (q != null && q.size() > 0) {
> 				for (int i = 0; i < q.size(); i++) {
> 					cimQual = (CIMQualifier)q.elementAt(i);
> 					if((cimQual.getName()).equalsIgnoreCase("provider") == true) {
> 						cv = cimQual.getValue();
> 						if (cv != null) {
> 							break;
> 						}
> 					}
> 					cimQual = null;
> 				}
> 			}
> 			if (cimQual != null) {
> 				continue;
> 			}
> 
> 			} /*if*/
> 			g.addElement(ci);
> 		}
> 
> 		for (int i = 0; i < g.size(); i++) {
> 			CIMInstance ci = (CIMInstance)g.elementAt(i);
>                   se.addElement(ci);
> 		}
> 
> /*
>             while (e.hasMoreElements()) {
>                 se.addElement(e.nextElement());
>             }
> */
>         }
>         return(se);
>     }
> 
> 
>     /**
>      * Returns all instances (*not just names*) for this class
>      *
>      * NB: This function filters out unauthorized properties.
>      *
>      * @param path The CIM Object Path of the class of instances to
>      *        look for.
>      * @param deep Include sub-classed instances/properties.
>      * @param locaOnly Include inherited properties.
>      * @return An enumeration of the (filtered) CIM Instances.
>      */
> 
>     public Enumeration enumInstances(CIMObjectPath path, boolean deep,
>                                      boolean localOnly) throws CIMException {
>         SimpleEnum se=new SimpleEnum();
>         Enumeration ips=getInstanceProviders(path);
>         long tid;
> 
>         CIMClass cc = classStore.getCIMClass(path.getObjectName());
> 
>         while (ips.hasMoreElements()) {
>             InstanceProvider ip=(InstanceProvider)ips.nextElement();
> 	      tid = beginTransaction((CIMProvider)ip);
>             Enumeration e=
>                 ip.enumInstances(tid,path,deep,
>                                  cc,
>                                  localOnly).elements();
> 
>   	      Vector g = new Vector();
> 	      Vector q;
> 	      CIMQualifier cimQual;
> 		CIMValue cv;
> 
> 		// Filter out instances that belong to providers
>             while (e.hasMoreElements()) {
>             	CIMInstance ci=(CIMInstance)e.nextElement();
> 
> 			if (ip == internalProvider) {
> 
> 			q = ci.getQualifiers();
> 			cimQual = null;
> 			if (q != null && q.size() > 0) {
> 				for (int i = 0; i < q.size(); i++) {
> 					cimQual = (CIMQualifier)q.elementAt(i);
> 					if((cimQual.getName()).equalsIgnoreCase("provider") == true) {
> 						cv = cimQual.getValue();
> 						if (cv != null) {
> 							break;
> 						}
> 					}
> 					cimQual = null;
> 				}
> 			}
> 			if (cimQual != null) {
> 				continue;
> 			}
> 
> 			} /*if*/
> 			g.addElement(ci);
> 		}
> 
> 		for (int i = 0; i < g.size(); i++) {
> 			CIMInstance ci = (CIMInstance)g.elementAt(i);
> 	            ci = filterUnauthorizedProperties(ci, RoleLevel.READ_QUAL);
>                   se.addElement(ci);
> 		}
> /*
>             while (e.hasMoreElements()) {
>                 CIMInstance ci = (CIMInstance)e.nextElement();
>                 ci = filterUnauthorizedProperties(ci, RoleLevel.READ_QUAL);
>                 se.addElement(ci);
>             }
> */
>         }
>         return(se);
>     }
> 
> 
>     /**
>      * Invokes a method.
>      *
>      * NB: This will check the ExecuteRole to verify that the user
>      *     is authorized to execute the method.
>      *
>      * @param path The path of the object on which the method is being invoked
>      *             this can be a class or instance
>      * @param inParams A vector of CIMProperty for inbound values
>      * @param outParams A vector of CIMProperty for outbound values
>      * @throws CIMException
>      */
> 
>     public CIMValue invokeMethod(CIMObjectPath path,String name,
>                                  Vector in, Vector out) throws CIMException {
>         CIMClass cc;
>         CIMMethod method;
>         Vector inParams=null;
> 
>         inParams=in;
> 
>         cc=classStore.getCIMClass(path.getObjectName());
>         method=cc.getMethod(name);
>         if (method==null) {
>             throw new CIMException(CIMException.CIM_ERR_METHOD_NOT_FOUND,
>                         name);
>         }
> 
>         //
>         // Check that the user is authorized to execute the method
>         //
> 
>         CIMQualifier qual = method.getQualifier(RoleLevel.EXECUTE_QUAL);
>         if (!isAuthorized(qual, RoleLevel.EXECUTE_QUAL))
>             {
> 		    TransactionThread tx ;
>             String userName;
> 
> 		    if (Thread.currentThread() instanceof TransactionThread) 
>                 {
> 		        tx = (TransactionThread)Thread.currentThread();
>                 userName = tx.userName;
>                 }
>             else 
>                 userName = "-";
> 
>             String[] args = new String[3];
>             args[0] = userName;
>             args[1] = name;
>             args[2] = path.toString();
> 
>             TransactionThread.securityLog.writeLog("NOTAUTHORIZEDMETHOD", null, Log.SEV_WARNING, args);
>             throw new CIMException(CIMException.CIM_ERR_ACCESS_DENIED);
>             }
> 
>         //
>         // Check that an instance exists if the static qualifier
>         // is false or not set.
>         //
>  
>         qual = method.getQualifier("static");
>         if ((qual == null) || qual.getValue().equals(CIMValue.FALSE)) {
>             getInstance(path,false);
> 	  }
> 
>         //
>         // If a provider is defined, go call it directly
>         //
>         MethodProvider methodp=getMethodProvider(path,name);
> 
>         if (methodp==null) {
>             //
>             // Look to see if class instance provider can act as
>             // a method provider
>             //
>             try {
>                 methodp=(MethodProvider)getInstanceProvider(path);
>         	    long tid = beginTransaction((CIMProvider)methodp);
>                 return methodp.invokeMethod(tid,path,name,inParams,out);
>             } catch (java.lang.ClassCastException ce) {
>                 // may not be a method provider, so be silent
>             }
>         } else {
>             long tid = beginTransaction((CIMProvider)methodp);
>             return methodp.invokeMethod(tid,path,name,inParams,out);
> 	  }
> 
>         throw new CIMException(CIMException.CIM_ERR_FAILED,
>                                "No method provider");
>     }
> 
> 
>     /**
>      * Gets a specific CIM Instance's property.
>      *
>      * NB: This function will use the role-based authorization
>      *     mechanim.
>      *
>      * @param path CIM Object Path to instance
>      * @param propertyName Name of property to fetch.
>      * @return A CIMValue object containing the property's value.
>      */
> 
>     public CIMValue getProperty(CIMObjectPath path,
>                                 String propertyName) throws CIMException {
>         if ((path == null) || propertyName==null)
>             throw new CIMException(CIMException.CIM_ERR_NO_SUCH_PROPERTY,propertyName);
> 
>         CIMClass    cc = classStore.getCIMClass(path.getObjectName());
>         CIMProperty cp = cc.getProperty(propertyName);
>         if (cp == null)
>             throw new CIMException(CIMException.CIM_ERR_NO_SUCH_PROPERTY,propertyName);
>         CIMQualifier qual    = cp.getQualifier(RoleLevel.READ_QUAL);
>         if (!isAuthorized(qual, RoleLevel.READ_QUAL))
>             {
> 		    TransactionThread tx ;
>             String userName;
> 
> 		    if (Thread.currentThread() instanceof TransactionThread) 
>                 {
> 		        tx = (TransactionThread)Thread.currentThread();
>                 userName = tx.userName;
>                 }
>             else 
>                 userName = "<unknown>";
> 
>             String[] args = new String[3];
>             args[0] = userName;
>             args[1] = propertyName;
>             args[2] = path.toString();
> 
>             TransactionThread.securityLog.writeLog("NOTAUTHORIZEDPROP", null, Log.SEV_WARNING, args);
>             throw new CIMException(CIMException.CIM_ERR_ACCESS_DENIED);
>             }
> 
>         PropertyProvider pp=getPropertyProvider(path,propertyName);
>         if (pp==null) {
>             // no property provider use instance provider
>             // The getInstance() method will call beginTransaction() internally,
>             // no need to do that here.
>             CIMInstance ci=getInstance(path,false);
>              return ci.getProperty(propertyName).getValue();
>         } else {
>        	long tid = beginTransaction((CIMProvider)pp);
>             return pp.getPropertyValue(tid,path,path.getObjectName(),propertyName);
>         }
>     }
> 
>     /**
>      * Sets a specific CIM Instance's property.
>      *
>      * NB: This function will use the role-based authorization
>      *     mechanim.
>      *
>      * @param path CIM Object Path to instance
>      * @param propertyName Name of property to set.
>      * @param newValue Value of property to be set.
>      */
> 
>     public void setProperty(CIMObjectPath path,
>                             String propertyName,
>                             CIMValue newValue) throws CIMException {
>         if ((path == null) || (propertyName == null))
>             throw new CIMException(CIMException.CIM_ERR_NO_SUCH_PROPERTY,propertyName);
> 
>         CIMClass cc=classStore.getCIMClass(path.getObjectName());
>         CIMProperty cp=cc.getProperty(propertyName);
>         if (cp==null)
>             throw new CIMException(CIMException.CIM_ERR_NO_SUCH_PROPERTY,propertyName);
> 
>         CIMQualifier qual = cp.getQualifier(RoleLevel.WRITE_QUAL);
>         if (!isAuthorized(qual, RoleLevel.WRITE_QUAL))
>             {
> 		    TransactionThread tx ;
>             String userName;
> 
> 		    if (Thread.currentThread() instanceof TransactionThread) 
>                 {
> 		        tx = (TransactionThread)Thread.currentThread();
>                 userName = tx.userName;
>                 }
>             else 
>                 userName = "<unknown>";
> 
>             String[] args = new String[3];
>             args[0] = userName;
>             args[1] = propertyName;
>             args[2] = path.toString();
> 
>             TransactionThread.securityLog.writeLog("NOTAUTHORIZEDPROP", null, Log.SEV_WARNING, args);
>             throw new CIMException(CIMException.CIM_ERR_ACCESS_DENIED);
>             }
> 
>         if (cp.getQualifier("KEY")!=null) {
>             //
>             // This is a key, see if the value is different
>             //
>             CIMValue cv1=getProperty(path,propertyName);
>             if (!cv1.equals(newValue)) {
>                 // Trying to change value of key
>                 throw new CIMInstanceException(CIMInstanceException.
>                                                CIM_ERR_KEY_CANNOT_MODIFY);
>             }
>         } else {
>             CIMInstance pinstance=null;
>             boolean doFltr=false;
> 
>             PropertyProvider pp=getPropertyProvider(path,propertyName);
> 
>             if (pp==null) {
>                 // no property provider use instance provider
>                 CIMInstance ci=getInstance(path,false);
>                 CIMInstance ciClone = cc.newInstance();
>                 Vector v1 = ci.getProperties();
>                 Vector v2 = (Vector)v1.clone();
>                 ciClone.setProperties(v2);
> 
>                 ciClone.setProperty(propertyName,newValue);
>                 setInstance(path,ciClone);
> 
>             } else {
>         	    long tid = beginTransaction((CIMProvider)pp);
> 
>                 pp.setPropertyValue(tid,path,path.getObjectName(),
>                                     propertyName,newValue);
>             }
>         }
>     }
> 
>     //
>     // obtain association providers
>     //
> 
>     private AssociatorProvider getAssociatorProvider(String className)
>         throws CIMException {
>         AssociatorProvider associationp=null;
>         String cls=className;
>         while (cls!=null && (cls.length() != 0)) {
>             CIMClass cc=classStore.getCIMClass(cls);
>             CIMQualifier cq=cc.getQualifier(PROVIDER_NAME);
>             if (cq!=null) {
>                 try {
>                     associationp=(AssociatorProvider)
>                         (pmap.getProvider(cimomHdl,cq));
>                     break;
>                 } catch(CIMException ce) {
>                     // no association provider, must continue
>                 }
>             }
>             cls=cc.getSuperClass();
>         }
>         if (associationp==null)
>             associationp=(AssociatorProvider)internalProvider;
>         return associationp;
>     }
> 
>     private Enumeration getAssociatorProviders(CIMObjectPath cop)
>         throws CIMException {
>         String className;
>         if (cop==null)
>             className="$$$ALL-ASSOC$$$"; // if no assoc class given
>         else
>             className=cop.getObjectName();
>         if (assocProviders==null)
>             assocProviders=new Hashtable();
>         Vector aps=(Vector)assocProviders.get(className);
>         if (aps==null) {
>             // providers not cached - go down the inheritance tree to
>             // find all the providers
>             aps=new Vector();
>             addAssociatorProviders(className,aps);
>             assocProviders.put(className,aps);
>         }
>         return aps.elements();
>     }
> 
>     // perform recursive decent down the class tree to find all the
>     // providers delivering associations
>     // Note: the order is important: derived (more specialized) first
>     private void addAssociatorProviders(String className,
>                                         Vector providerList)
>         throws CIMException {
>         Enumeration classChildren;
>         if (!className.equals("$$$ALL-ASSOC$$$")) {
>             AssociatorProvider ap=getAssociatorProvider(className);
>             if (!providerList.contains(ap))
>                 providerList.insertElementAt(ap,0);
>             classChildren=classStore.getClassChildren(className).elements();
>         } else
>             classChildren=associationClassNames();
>         while (classChildren.hasMoreElements())
>             addAssociatorProviders((String) classChildren.nextElement(),
>                                     providerList);
>     }
> 
>     // return only top-level association classes
>     private Enumeration associationClassNames()
>         throws CIMException {
>         Enumeration allClasses=classStore.getAllClasses();
>         SimpleEnum assocNames=new SimpleEnum();
>         while (allClasses.hasMoreElements()) {
>             CIMClass cc = (CIMClass) allClasses.nextElement();
>             if ((cc.getSuperClass()==null || (cc.getSuperClass()).length() == 0)  && 
>                 cc.getQualifier("association")!=null)
>                 assocNames.addElement(cc.getName());
>         }
>         return assocNames;
>     }
> 
>     //
>     //  associations/references
>     //
> 
>     public Enumeration associatorNames(CIMObjectPath path,
>                                        String assocClass,
>                                        String resultClass,
>                                        String role,
>                                        String resultRole) throws CIMException {
>         SimpleEnum se=new SimpleEnum();
>         CIMObjectPath assocPath=null;
>         if (assocClass!=null)
>             assocPath=new CIMObjectPath(assocClass,path.getNameSpace());
>         Enumeration aps=getAssociatorProviders(assocPath);
>         while (aps.hasMoreElements()) {
>             AssociatorProvider ap=(AssociatorProvider)aps.nextElement();
>         	long tid = beginTransaction((CIMProvider)ap);
> 
>             Enumeration e=ap.associatorNames(tid,assocPath,path,resultClass,role,
>                                              resultRole).elements();
>             while (e.hasMoreElements()) {
>                 se.addElement(e.nextElement());
>             }
>         }
>         return(se);
>     }
> 
>     public Enumeration associators(CIMObjectPath path,String assocClass,
>                                    String resultClass,String role,
>                                    String resultRole,boolean includeQualifiers,
>                                    boolean includeClassOrigin,
>                                    String propertyList[]) throws CIMException {
>         SimpleEnum se=new SimpleEnum();
>         CIMObjectPath assocPath=null;
>         if (assocClass!=null)
>             assocPath=new CIMObjectPath(assocClass,path.getNameSpace());
>         Enumeration aps=getAssociatorProviders(assocPath);
>         while (aps.hasMoreElements()) {
>             AssociatorProvider ap=(AssociatorProvider)aps.nextElement();
>         	long tid = beginTransaction((CIMProvider)ap);
> 
>             Enumeration e=ap.associators(tid,assocPath,path,resultClass,role,
>                                          resultRole,includeQualifiers,
>                                          includeClassOrigin,
>                                          propertyList).elements();
>             while (e.hasMoreElements()) {
>                 se.addElement(e.nextElement());
>             }
>         }
>         return(se);
>     }
> 
>    /**
>     * Locates referencing  names of objects
>     *
>     * @param path the name of the object to which the references should
>     * refer
>     * @param resultClass the classname of the association or null
>     * @param role the name of a property to act as a filter or null
>     */
>     public Enumeration referenceNames(CIMObjectPath path,
>                                       String resultClass,
>                                       String role) throws CIMException {
>         SimpleEnum se=new SimpleEnum();
>         CIMObjectPath assocPath=null;
>         if (resultClass!=null)
>             assocPath=new CIMObjectPath(resultClass,path.getNameSpace());
>         Enumeration aps=getAssociatorProviders(assocPath);
>         while (aps.hasMoreElements()) {
>             AssociatorProvider ap=(AssociatorProvider)aps.nextElement();
>             long tid = beginTransaction((CIMProvider)ap);
> 
>             Enumeration e=ap.referenceNames(tid,assocPath,path,role).elements();
>             while (e.hasMoreElements()) {
>                 se.addElement(e.nextElement());
>             }
>         }
>         return(se);
>     }
> 
>    /**
>     * Locates referencing objects
>     *
>     * @param path the name of the object to which the references should
>     * refer
>     * @param resultClass the classname of the association or null
>     * @param role the name of a property to act as a filter or null
>     * @param includeQualifiers true if qualifiers to be included
>     * @param includeClassOrigin true if class origins to be included
>     * includeClassOrigin is not currently used.
>     * @param propertyList the list of properties to return
>     */
>     public Enumeration references(CIMObjectPath path,
>                                   String resultClass,
>                                   String role,
>                                   boolean includeQualifiers,
>                                   boolean includeClassOrigin,
>                                   String propertyList[]
>                                   ) throws CIMException {
>         SimpleEnum se=new SimpleEnum();
>         CIMObjectPath assocPath=null;
>         if (resultClass!=null)
>             assocPath=new CIMObjectPath(resultClass,path.getNameSpace());
>         Enumeration aps=getAssociatorProviders(assocPath);
>         while (aps.hasMoreElements()) {
>             AssociatorProvider ap=(AssociatorProvider)aps.nextElement();
>             long tid = beginTransaction((CIMProvider)ap);
> 
>             Enumeration e=ap.references(tid,assocPath,path,role,
>                                         includeQualifiers,includeClassOrigin,
>                                         propertyList).elements();
>             while (e.hasMoreElements()) {
>                 se.addElement(e.nextElement());
>             }
>         }
>         return(se);
>     }
> 
>     //
>     ////////////////////////////////////////////////////////
>     //
>     // Qualifier Type Functions
>     //
>     ////////////////////////////////////////////////////////
>     //
>     public synchronized Enumeration enumQualifierTypes() throws CIMException {
>                 return(qualifierStore.getQualifiers());
>     }
> 
>     public synchronized void createQualifierType(CIMObjectPath path,
>                       CIMQualifierType qt) throws CIMException{
>                 qualifierStore.createQualifier(qt);
>     }
> 
>     public synchronized void setQualifierType(CIMObjectPath path,
>                       CIMQualifierType qt) throws CIMException{
>                 qualifierStore.setQualifier(qt);
>     }
> 
> 
>     public synchronized CIMQualifierType getQualifierType(CIMObjectPath path) throws CIMException{
>                 return(qualifierStore.getQualifier(path));
>     }
> 
>     public synchronized void deleteQualifierType(CIMObjectPath path) throws CIMException{
>                 qualifierStore.deleteQualifier(path);
>     }
> 
> 
> 
> 
>     //
>     ////////////////////////////////////////////////////////
>     //
>     // Class Functions
>     //
>     ////////////////////////////////////////////////////////
>     //
> 
>     //
>     // Sets isKeyed if there is a KEY qualifier - just in case
>     // the client didn't do this for us!
>     //
>     private void fixKeys(CIMClass cc) {
>         Vector properties=cc.getAllProperties();
>         int psize=properties.size();
>         for(int i=0;i<psize;i++) {
>             CIMProperty p=((CIMProperty)(properties.elementAt(i)));
>             if (p.isKey()) {
>                 cc.setIsKeyed(true);
>                 break;
>             }
>         }
>     }
> 
> 
>     private boolean methodDefined(CIMClass cc,CIMMethod cimMethod) {
> 
>         Vector methods=cc.getAllMethods();
>         if (methods!=null) {
>             int msize=methods.size();
>             for(int i=0;i<msize;i++) {
>                 CIMMethod mp=(CIMMethod)(methods.elementAt(i));
>                 if (cimMethod.equals(mp))
>                     return(true);
>             }
>         }
>         return(false);
>     }
> 
> 
>     //
>     // Add in parent properties and methods
>     //
>     private void prepareFullClass(CIMClass parentClass,
>                                   CIMClass child) throws CIMException {
> 
>         //
>         // Note, regardless of whether there is an override
>         // this will perform override like behavior - probably
>         // need to add validation code...
>         //
>         Vector properties=parentClass.getAllProperties();
>         if (properties!=null) {
>             int psize=properties.size();
>             for(int i=0;i<psize;i++) {
>                 CIMProperty cp=(CIMProperty) (properties.elementAt(i));
>                 CIMProperty childProp=child.getProperty(cp.getName());
>                 if (childProp==null) {
>                     child.addProperty(cp);
>                 } else {
>                     //
>                     // Propagate any qualifiers that have not been
>                     // re-defined
>                     //
>                     Vector qualifiers=cp.getQualifiers();
>                     if (qualifiers!=null) {
>                         int qualSize=qualifiers.size();
>                         for(int qi=0;qi<qualSize;qi++) {
>                             CIMQualifier qual=(CIMQualifier)(qualifiers.elementAt(qi));
>                             CIMQualifier childQual=childProp.getQualifier(qual.getName());
>                             if (childQual==null) {
>                                 //
>                                 // Qualifier not defined on child property
>                                 // so propagate it
>                                 //
>                                 childProp.addQualifier(qual);
>                             }
>                         }
>                     }
>                 }
>             }
>         }
>         Vector methods=parentClass.getAllMethods();
>         if (methods!=null) {
>             int msize=methods.size();
>             for(int i=0;i<msize;i++) {
>                 CIMMethod cm=(CIMMethod) (methods.elementAt(i)) ;
>                 CIMMethod childMethod=child.getMethod(cm.getName());
>                 if (childMethod==null) {
>                     child.addMethod(cm);
>                 } else {
>                     //
>                     // Propagate any qualifiers that have not been
>                     // re-defined by the method declaration
>                     //
>                     Vector qualifiers=cm.getQualifiers();
>                     Vector newQualifiers=(Vector)(childMethod.getQualifiers().clone());
>                     if (qualifiers!=null) {
>                         int qualSize=qualifiers.size();
>                         for(int qi=0;qi<qualSize;qi++) {
>                             CIMQualifier qual=(CIMQualifier)(qualifiers.elementAt(qi));
>                             CIMQualifier childQual=childMethod.getQualifier(qual.getName());
>                             if (childQual==null) {
>                                 //
>                                 // Qualifier not defined on child
>                                 // method so propagate it
>                                 //
>                                 newQualifiers.addElement(qual);
>                             }
>                         }
>                     childMethod.setQualifiers(newQualifiers);
>                     }
>                 }
>            }
>         }
>     }
> 
> 
>     //
>     // If an origin class is not set, assume it belongs with this class
>     // definition
>     //
>     private void setOriginClass(CIMClass cimClass) throws CIMException {
> 
>         String name=cimClass.getName();
> 
>         Vector properties=cimClass.getAllProperties();
>         if (properties!=null) {
>             int psize=properties.size();
>             for(int i=0;i<psize;i++) {
>                 CIMProperty cp=(CIMProperty) (properties.elementAt(i));
>                 if (cp.getOriginClass()==null) {
>                     cp.setOriginClass(name);
>                 }
>             }
>         }
>         Vector methods=cimClass.getAllMethods();
>         if (methods!=null) {
>             int msize=methods.size();
>             for(int i=0;i<msize;i++) {
>                 CIMMethod cm=(CIMMethod) (methods.elementAt(i)) ;
>                 if (cm.getOriginClass()==null)
>                     cm.setOriginClass(name);
>             }
>         }
>     }
> 
> 
>     public synchronized void addCIMClass(CIMObjectPath name,CIMClass cc) throws CIMException {
>         flushProviderCache(); // schema change - reset provider cache
>         String parentName=normalizeClassPath(cc.getSuperClass());
>         CIMClass parentClass=null;              // assume no parent
> 
>         setOriginClass(cc);
> 
>         if (parentName != null && parentName.length() > 0) {
>             parentClass=classStore.getCIMClass(parentName);
>             //
>             // If parent is keyed, then so is this class otherwise
>             // scan the class for key qualifiers
>             //
>             if (parentClass.isKeyed())
>                 cc.setIsKeyed(true);
>             else
>                 fixKeys(cc);
>             prepareFullClass(parentClass,cc);
>         } else {
>             //
>             // No parent class, need to see if class is keyed
>             //
>             fixKeys(cc);
>         }
> 
>         compressCIMClass(cc);
>         classStore.createClass(cc);
>     }
> 
> 
>     public synchronized void setCIMClass(CIMObjectPath name,CIMClass cc)
>                  throws CIMException {
>         flushProviderCache(); // schema change - reset provider cache
>         String parentName=normalizeClassPath(cc.getSuperClass());
>         CIMClass parentClass=null;              // assume no parent
>         CIMClass origClass=null;
>         boolean subscriber=false;
> 
>         setOriginClass(cc);
> 
>         if (parentName!=null && (parentName.length() != 0)) {
>             parentClass=classStore.getCIMClass(parentName);
>             //
>             // If parent is keyed, then so is this class otherwise
>             // scan the class for key qualifiers
>             //
>             if (parentClass.isKeyed())
>                 cc.setIsKeyed(true);
>             else
>                 fixKeys(cc);
>             prepareFullClass(parentClass,cc);
>         } else {
>             //
>             // No parent class, need to see if class is keyed
>             //
>             fixKeys(cc);
>         }
> 
>         compressCIMClass(cc);
> 
>         //
>         // Get original children - they'll need revising!
>         //
>         Vector origChildren=classStore.getClassChildren(cc.getName());
> 
>         classStore.setClass(cc);
> 
>         //
>         // Now move the children from the original class definition, updating
>         // inheirited methods, properties etc...
>         //
>         if (origChildren!=null) {
>             int childSize=origChildren.size();
>             for(int i=0;i<childSize;i++) {
>                         String childName=(String)origChildren.elementAt(i);
>                         CIMClass childClass=classStore.getCIMClass(childName);
>                 reworkChildClass(cc,childClass);
>             }
>         }
>     }
> 
>     private void reworkChildClass(CIMClass parentClass,CIMClass childClass)
>             throws CIMException {
>         //
>         // Remove all properties and methods which aren't defined by this
>         // child
>         //
>         String className=childClass.getName();
>         Vector properties=childClass.getAllProperties();
>         Vector newProperties=new Vector();
>         if (properties!=null) {
>             int psize=properties.size();
>             for(int i=0;i<psize;i++) {
>                 CIMProperty cp=(CIMProperty) (properties.elementAt(i));
>                 String oClass=cp.getOriginClass();
>                 if ( oClass==null ||
>                         oClass.trim().length()==0 ||
>                         oClass.equalsIgnoreCase(className)) {
>                     newProperties.addElement(cp);
>                 }
>             }
>         }
>         Vector methods=childClass.getAllMethods();
>         Vector newMethods=new Vector();
>         if (methods!=null) {
>             int msize=methods.size();
>             for(int i=0;i<msize;i++) {
>                 CIMMethod cm=(CIMMethod) (methods.elementAt(i)) ;
>                 String oClass=cm.getOriginClass();
>                 if ( oClass==null ||
>                         oClass.trim().length()==0 ||
>                         oClass.equalsIgnoreCase(className))
>                     newMethods.addElement(cm);
>             }
>         }
>         childClass.setProperties(newProperties);
>         childClass.setMethods(newMethods);
> 
>         //
>         // Add properties/methods from parent
>         //
>         if (parentClass!=null)
>             prepareFullClass(parentClass,childClass);
> 
>         //
>         // Store the class back
>         //
>         classStore.setClass(childClass);
> 
>         //
>         // Process children of this class
>         //
>         Vector childList=classStore.getClassChildren(childClass.getName());
>         int childSize=childList.size();
>         for(int i=0;i<childSize;i++) {
>             String childName=(String)childList.elementAt(i);
>             CIMClass newChildClass=classStore.getCIMClass(childName);
>             reworkChildClass(childClass,newChildClass);
>         }
>     }
> 
>     //
>     //
>     public synchronized void deleteClass(CIMObjectPath path)
>                 throws CIMException{
> 
>         boolean subscriber=false;
>         CIMClass origClass=null;
>         String name=path.getObjectName();
> 
>         flushProviderCache(); // schema change - reset provider cache
>         classStore.deleteClass(name);
>     }
> 
> 
>     public synchronized CIMClass getCIMClass(String className)
>          throws CIMException {
>         return(classStore.getCIMClass(className));
>     }
> 
>     public synchronized CIMClass getCIMClass(CIMObjectPath path)
>          throws CIMException{
>         return(classStore.getCIMClass(path.getObjectName()));
>     }
> 
>     private void addClassAndChildren(SimpleEnum se,CIMClass cc,
>                                      boolean deep,
>                                      boolean localOnly) {
>         if (localOnly==true)
>             cc=cc.localElements();
>         se.addElement(cc);
>         if (deep==CIMClient.DEEP) {
>             try {
>                 Vector children=classStore.getClassChildren(cc.getName());
>                 for(int i=0;i<children.size();i++) {
>                     String childName=(String)(children.elementAt(i));
>                     CIMClass childClass=classStore.getCIMClass(childName);
>                     addClassAndChildren(se,childClass,deep,localOnly);
>                 }
>             }catch (CIMException ce) {}
>         }
>     }
> 
>     //
>     // returns an enumeration of class instances
>     //
>     public synchronized Enumeration enumClass(CIMObjectPath path,
>                        boolean deep, boolean localOnly) throws CIMException{
>         SimpleEnum se=new SimpleEnum();
> 
>         String wantedClassName=null;
>         if (path != null) {
>             wantedClassName=path.getObjectName();
> 
>             if (wantedClassName!=null &&
>                  (wantedClassName.equals("")
>                         || wantedClassName.equals(CIMObjectPath.NOT_SET))) {
>                 //
>                 // Also, any class
>                 //
>                 wantedClassName=null;
>             }
>         }
> 
>         if (wantedClassName==null) {
>             //
>             // If no name is specified, then its a special case to
>             // get all classes
>             //
>             Enumeration enum=classStore.getAllClasses();
>             return(enum);
>         }
>         try {
>             CIMClass cc=classStore.getCIMClass(wantedClassName);
>             addClassAndChildren(se,cc,deep,localOnly);
>         } catch (CIMException ce) {}
>         return(se);
>     }
> 
> 
> 
>     private void addClassAndChildren(SimpleEnum se,CIMClass cc,
>                                      String nameSpace,
>                                      boolean deep) {
>         String className=cc.getName();
>         CIMObjectPath cop = new CIMObjectPath(className,nameSpace);
>         se.addElement(cop);
>         if (deep==CIMClient.DEEP) {
>             try {
>                 Vector children=classStore.getClassChildren(className);
>                 for(int i=0;i<children.size();i++) {
>                     String childName=(String)(children.elementAt(i));
>                     CIMClass childClass=classStore.getCIMClass(childName);
>                     addClassAndChildren(se,childClass,nameSpace,deep);
>                 }
>             } catch (CIMException ce) {}
>         }
>     }
> 
>     //
>     // returns an enumeration of class paths
>     //
>     public synchronized Enumeration enumClass(CIMObjectPath path,
>                        boolean deep) throws CIMException{
>         SimpleEnum se=new SimpleEnum();
>         String wantedClassName=null;
> 
>         if (path!=null) {
>             wantedClassName=path.getObjectName();
> 
>             if (wantedClassName!=null &&
>                  (wantedClassName.equals("")
>                         || wantedClassName.equals(CIMObjectPath.NOT_SET))) {
>                 //
>                 // Also, any class
>                 //
>                 wantedClassName=null;
>             }
>         }
> 
>         if (wantedClassName==null) {
>             //
>             // If no name is specified, then its a special case to
>             // get all classes
>             //
>             Enumeration enum=classStore.getAllClasses();
>             while(enum.hasMoreElements()) {
>                 String className = ((CIMClass)(enum.nextElement())).getName();
>                 CIMObjectPath cop = new CIMObjectPath(className,
>                                                       path.getNameSpace());
>                 se.addElement(cop);
>             }
>             return(se);
>         }
>         try {
>             CIMClass cc=classStore.getCIMClass(wantedClassName);
>             addClassAndChildren(se,cc,path.getNameSpace(),deep);
>         } catch (CIMException ce) {}
> 
>         return(se);
>     }
> 
> 
>     String getNameSpace() {
>         return(cimNameSpace.getNameSpace());
>     }
> 
> 	private long beginTransaction(CIMProvider cp) throws CIMException {
> 		TransactionThread tx ;
> 		if (Thread.currentThread() instanceof TransactionThread) {
> 			tx = (TransactionThread)Thread.currentThread();
> 		} else {
> 			return -1;
> 		}
> 
> 		if (tx.provider == null) {
> 			tx.outLangTags = cp.beginTransaction(tx.tid, tx.langTags, tx.userName, tx.userPW, tx.pwEncoding);
> 			tx.provider = cp;
> 		}
> 
> 		return tx.tid;
> 	}
> 			
>     //
>     // This is available because when we  bootstrap the handle is to the
>     // server but the server is calling the ServerNameSpace ctr and therefore
>     // the namespace can't be found because the server isn't up and running
>     // fully.
>       public void makeHandle() {
>         try {
>             cimomHdl=new CIMOMHandleLocal(cimNameSpace);
>             internalProvider=
>                 ((CIMOMHandleLocal)cimomHdl).getInternalProvider();
>         } catch (CIMException ce) {
>             cimomHdl=null;
>         }
>         //if (cimomHdl==null)
>             //System.err.println("Could not find:"+cimNameSpace);
> 	}
> 
>     public ServerNameSpace(CIMNameSpace cns) {
>         cimNameSpace=cns;
>         classStore=new CIMOMClassStore(cns.getNameSpace(), false);
>         qualifierStore=new CIMOMQualifierStore(cns.getNameSpace());
> /*
>                 classStore=new CIMOMClassStoreMemory();
>                 qualifierStore=new CIMOMQualifierStoreMemory();
> */
>         try {
>             cimomHdl=new CIMOMHandleLocal(cns);
>             internalProvider=((CIMOMHandleLocal)cimomHdl).getInternalProvider();
>         } catch (CIMException ce) {
>             cimomHdl=null;
>         }
>         serverNameSpace=this;
>     }
> 
> 
>     /**
>      * Check if current user has authority of roleRequired
>      * or better.  This will look at the current thread to
>      * obtain the username and password, which is passed
>      * into the AuthModule to authorize the user. 
>      *
>      * If <code>roleRequired</code> is <code>null</code>
>      * it will return <code>true</code>.
>      *
>      * @param roleRequired A string containing the name of 
>      *        the role that the user must have (or better)
>      *        in order to be authorized.  The RoleLevel.java
>      *        has a list of constants that may be used.
>      * @return <code>true if authorized, <code>false</code>
>      *        otherwise.
>      */
>     private boolean
>     isAuthorized(CIMQualifier qual,
>                  String       qualType)
>     throws CIMException
>     {
>         // We authorize everyone when authorization disabled
>         if (GlobalParms.authorization == false)
>             return true;
> 
>         String roleRequired = null;
> 
>         if (qual != null)
>             {
>             CIMValue cv = (CIMValue)qual.getValue();
>             if (cv != null)
>                 roleRequired = (String)qual.getValue().getValue();
>             }
>         else
>             {
>             roleRequired = (String)defaultRoles.get(qualType);
>             if (roleRequired == null)
>                 {
>                 // Read MOF for default and set it in cache.
>                 try
>                     {
>                     CIMObjectPath    cop = new CIMObjectPath(qualType);
>                     CIMQualifierType ct = getQualifierType(cop);
>                     CIMValue         cv = ct.getDefaultValue();
>                     roleRequired = (String)cv.getValue();
>                     defaultRoles.put(qualType, roleRequired);
>                     } 
>                 catch (CIMException cx)
>                     {
>                     // do nothing; the qualifier was not
>                     // defined in the mof
>                     }
>                 }
>             }
> 
>         // If role is not in MOF (either directly
>         // or as default) then return true to allow
>         // the provider to athenticate/authorize.
> 
>         if (roleRequired == null)
>             return true;
> 
> 		TransactionThread tx ;
> 		if (Thread.currentThread() instanceof TransactionThread) 
> 		    tx = (TransactionThread)Thread.currentThread();
>         else 
>             throw new CIMException(CIMException.CIM_ERR_FAILED);
> 
>         String userRole = tx.userRole;
>         return RoleLevel.isAuthorized(userRole, roleRequired);
>     }
> 
> 
>     /**
>      * Generate list of all properties that the user has access to.  
>      * It is used as input to a filterProperties() call to eliminate
>      * the unauthorized properties.
>      *
>      * @param cimClass the CIM Class to pull properties from.
>      * @param qualType Either <code>RoleLevel.READ_QUAL</code>
>      *        or <code>RoleLevel.WRITE_QUAL</code>
>      * @return A string array of authorized property names.
>      *
>      */
>     private String[]
>     getAuthorizedProperties(CIMClass cimClass,
>                             String   qualType)
>     throws CIMException
>     {
>         Vector properties = cimClass.getAllProperties();
>         Vector propListV = new Vector();
>         for (int i=0; i < properties.size(); i++)
>             {
>             CIMProperty property=(CIMProperty)(properties.elementAt(i));
>             CIMQualifier qual = property.getQualifier(qualType);
>             if (isAuthorized(qual, qualType))
>                 propListV.add(property.getName());
>             }
>         String[] propertyList = new String[propListV.size()];
>         propListV.toArray(propertyList);
> 
>         return propertyList;
>     }
> 
> 
> 
>     /**
>      * Filter all properties that the user doesn't have access to.  
>      * In general, we don't want to simply throw a 
>      * CIMException.CIM_ERR_ACCESS_DENIED because there may be 
>      * some other properties that are perfectly valid for the 
>      * user to set or get (especially in the case where
>      * this call is used as part of the property provider).
>      *
>      * @param cimInstance The raw cim instance with potentially
>      *        unauthorized properties
>      * @param qualType Either <code>RoleLevel.READ_QUAL</code>
>      *        or <code>RoleLevel.WRITE_QUAL</code>
>      * @return A CIM instance with <b>only</b> authorized properties.
>      *
>      */
> 
>     private CIMInstance
>     filterUnauthorizedProperties(CIMInstance cimInstance,
>                                  String      qualType)
>     throws CIMException
>     {
>         // If authorization turned off, we'll return everything
> 
>         if ((GlobalParms.authorization == false) || (cimInstance == null))
>             return cimInstance;
> 
>         // NOTE: This would be more efficient if it was integrated 
>         // into the CIMInstance class.  It would be possible to 
>         // do the filter in a single probe (rather than build 
>         // a propertyList in one pass and filter in another).
>         // It can't be done here, because of private fields.
> 
>         CIMClass cc = classStore.getCIMClass(cimInstance.getClassName());
>         String[] propertyList = getAuthorizedProperties(cc, qualType);
> 
>         return cimInstance.filterProperties(propertyList, 
>                                             true /*includeQualifiers*/,
>                                             true /*includeClassOrigin*/);
>     }
> }
