#!/bin/sh

OS=`/bin/uname -s`
if test $OS = Linux
then
   bad=
   krnl=`/bin/uname -r`
   if /usr/bin/expr $krnl : '2\.2\.' >/dev/null
   then
      test -z "`/bin/ls /lib/libc-2.1.* 2>/dev/null`" && bad=1
   fi
   if test -n "$bad"
   then
      echo IBM Tivoli SRM cannot be installed on this version of Linux.
      exit 2
   fi
fi

if test $OS = HP-UX
then
   release=`/usr/bin/uname -r`
   major=`/usr/bin/expr "$release" : '[^.]*\.\([^.]*\)'`
   if test "$major" = 10
   then
      echo IBM Tivoli SRM cannot be installed on this version of HP-UX.
      exit 2
   fi
fi

# If no parms have been supplied, print usage and exit.
PROG=`/bin/basename $0`
if test $# -lt 1
then
   echo 'To run the GUI install:'
   echo "    $PROG -g"
   echo
   echo 'To run a quick (agent-only) install:'
   echo '    '$PROG' -s <server-host> [-d <dest-dir>] [-p <server-port>]'
   echo '            [-q <agent-port>] [-x] [-n] [-l [o][r]]'
   echo '<dest-dir>    defaults to /usr/Tivoli/TSRM or /opt/Tivoli/TSRM'
   echo '<server-port> defaults to 2078'
   echo '<agent-port>  defaults to 2077'
   echo '-x prevents agent from running scripts sent from server'
   echo '-n prevents agent from running initial scan'
   echo '-l identifies the products that will be licensed on this agent'
   echo '   Valid product codes are: o - Tivoli SRM'
   echo '                            r - Tivoli SRM for Databases'
   echo '-t activates tracing for the install process'
   exit 1
fi

# Do some more initialization.

INSTALL_HOME=`/usr/bin/dirname $0`

cd $INSTALL_HOME
INSTALL_HOME=`pwd`

CLASSPATH=install/TSRMins.jar:install/TSRMRequest.jar:install/ibmjcefw.jar:install/ibmjceprovider.jar:install/ibmpkcs.jar:install/local_policy.jar:install/US_export_policy.jar:install/autotrace.jar

UDBFOUND=0
if udblib=`./install/findudb`
then
   UDBPROP="-DTivoliTSRM.udb.path=$udblib"
   UDBFOUND=1
else
   UDBPROP=
fi

case $OS in
   HP-UX)
      SHLIB_PATH=$INSTALL_HOME/install
      if test $UDBFOUND -eq 1
      then
         SHLIB_PATH=$SHLIB_PATH:$udblib/lib
      fi
      export SHLIB_PATH
      ;;
   AIX)
      LIBPATH=$INSTALL_HOME/install:$INSTALL_HOME/java/bin/classic:\
$INSTALL_HOME/java/bin
      if test $UDBFOUND -eq 1
      then
         LIBPATH=$LIBPATH:$udblib/lib
      fi
      export LIBPATH
      ;;
   *)
      LD_LIBRARY_PATH=$INSTALL_HOME/install
      if test $UDBFOUND -eq 1
      then
         LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$udblib/lib
      fi
      export LD_LIBRARY_PATH
      ;;
esac

# If parm 1 doesn't start with '-', assume it's path to JRE

JRE=java
if /usr/bin/expr X$1 : X- >/dev/null
then
   true
else
   JRE=$1
   shift
fi
JAVAEXE=$JRE/bin/java

# Determine whether to run the regular (GUI-based) install or a command-line
# install.

MAIN_CLASS=com.tivoli.itsrm.install.Install
GUI_INSTALL=1
OPT=`/usr/bin/expr X$1 : '..\(.*\)'`
case "$OPT" in
   s|d|p|q|n|x)
      MAIN_CLASS=com.tivoli.itsrm.install.CommandLineInstall
      GUI_INSTALL=0
      ;;
   uq)
      MAIN_CLASS=com.tivoli.itsrm.install.QuickUninstall
      GUI_INSTALL=0
      shift
      ;;
   g)
      shift
      ;;
esac

if test $GUI_INSTALL -eq 0
then
   REDIRECT=
else
   if test -z "$DISPLAY"
   then
      echo DISPLAY not set.
      exit 1
   fi
   tmpdir=${TMPDIR:-${TMP:-/tmp}}
   mon=`date +%m%d%H`
   min=`date +%M`
   sec=`date +%S`
   inslog=$tmpdir/install_$mon$min$sec
   REDIRECT="-DTivoliTSRM.log.file=$inslog 2"\>$inslog
   echo Further output directed to $inslog
fi

# If no eval on following line, 2>$inslog would be passed as parm.

eval exec $JAVAEXE -cp $CLASSPATH $UDBPROP $REDIRECT $MAIN_CLASS "$@"
