sqlcount = 22

##########################################################################
# Non dbms specific repository creation script.
# Processed by TrelliSoft.install.RepositoryInstall
#
# This file must have the following format:
# 1) The first line of the file must contain a count of the sql stmts
#    included in the line. This info is used to update a progress bar
#    as the sql file is being processed. This line should be in the
#    format:
#        sqlcount = <n>
#    where <n> represents the # of statements contained in the file
#
# 2) Each sql statement must be terminated by a ';' as the first char
#    of a new line. Everything else on the terminator line is ignored.
#
# 3) Comments are denoted by a '#' as the first char of a line. Everything
#    on a comment line is ignored
#
# Parameter Values (see TrelliSoft.install.RepositoryInstall for all DBs):
#
# Parm    Oracle        MSSQL                      DB2
# ----    ------        -----                      ---
# {0}     decimal(10)   integer                    integer
# {1}     decimal(5)    smallint                   smallint
# {2}     raw(20)       varbinary(20)              varchar(20) for bit data
# {3}     date          datetime                   timestamp
# {4}     long          text                       long varchar
# {5}     long raw      image                      long varchar for bit data
# {6}     <empty>       case_mask varbinary(8)     <empty>
#                         default 0x00 not null,
# {7}     <empty>       case_mask varbinary(64)    <empty>
#                         default 0x00 not null,
# {8}     sysdate       current_timestamp          current timestamp
# {9}     '20'          ' '                        ' '
# {10}    <empty>       , case_mask                <empty>
# {11}    <empty>       0x00,                      <empty>
# {12}    float         float                      float
# {13}    raw(60)       varbinary(60)              varchar(60) for bit data
# {14}    <empty>       t_res_stat_dir.            <empty>
# {15}    modify        alter column               alter
# {16}    <empty>       <empty>                    set data type
# {17}    <empty>       not null                   <empty>
# {18}    column        column                     <empty>
# {19}    '4e'          0x4e                       x'4e'
# {20}    partition     partition                  "partition"
# {21}    default null  null                       <empty>
# {22}    <empty>       <empty>                    IN <tablspace>
###########################################################################

create table t_res_cimom
  (cimom_id          {0}           not null,
   host              varchar(128)  not null,
   port              {0}           not null,
   protocol          char(1)       not null,
   user_name         varchar(128)  {21},
   password          varchar(128)  {21},
   certificate       varchar(256)  {21}) {22}
;

create table t_extension_policy
  (policy_id         {0}          not null,
   provision_lun     char(1)      not null,
   provision_scope   char(1)      not null,
   model_fs_lun      char(1)      not null,
   extend_amt        {0}          not null,
   extend_amt_unit   char(1)      not null,
   extend_amt_type   char(1)      not null,
   max_capacity      {0}          not null,
   max_capacity_unit char(1)      not null,
   assign_lun        char(1)      not null,
   assign_lun_limit  {0}          not null,
   create_lun        char(1)      not null,
   create_lun_size   {0}          not null,
   create_lun_unit   char(1)      not null,
   auto_extend       char(1)      not null,
   trigger_amt       {0}          not null,
   trigger_amt_unit  char(1)      not null,
   log_only          char(1)      not null) {22}
;

create table t_res_volgroup
  (volgroup_id     {0}          not null,
   name            varchar(128) {21},
   computer_id     {0}          not null,
   freespace       decimal(13)  not null,
   capacity        decimal(13)  not null,
   volgroup_type   {1}	      not null,
   lss             char(2)      not null,
   config          {1}          not null,
   nbrvolumes      {0}          not null,
   disk_group      varchar(10)  not null,
   status          {1}          not null,
   format          {1}          not null,
   surf_lun_cap    decimal(13)  not null,
   unsurf_lun_cap  decimal(13)  not null) {22}
;

create unique index t_res_cimom_ix1
  on t_res_cimom (host, port)
;

create unique index t_res_cimom_ix2
  on t_res_cimom (cimom_id)
;

create unique index t_ext_policy_ix
  on t_extension_policy (policy_id)
;

create unique index t_res_volgroup_ix
  on t_res_volgroup (volgroup_id)
;

create index t_res_volgroup_ix2
  on t_res_volgroup (computer_id)
;

alter table t_config_settings
  add tec_server varchar(128) default 'host.com' not null
;

alter table t_config_settings
  add tec_port {0} default 5529 not null
;

alter table t_alert_definition
  add tec_event char(1) default '0' not null
;

alter table t_res_computer
  add guid char(47) {21}
;

alter table t_res_disk_array
  add nvs_mb {0} default 0 not null
;

alter table t_res_disk_array
  add vs_capacity decimal(13) default -1 not null
;

alter table t_res_disk_array
  add vs_freespace decimal(13) default -1 not null
;

alter table t_res_disk_array
  add dg_freecap decimal(13) default -1 not null
;

alter table t_res_logical_disk
  add volgroup_id {0} default -1 not null
;

alter table t_res_device
  add storage_system_type {1} default 0 not null
;

alter table t_alert_log
  add job_log_file varchar(256) default ' ' not null
;

alter table t_stat_totals
  add total_ss {0} default -1 not null
;

alter table t_stat_totals
  add total_luns {0} default -1 not null
;

alter table t_stat_totals
  add lun_capacity decimal(13) default -1 not null
;

