sqlcount = 25

##########################################################################
# Non dbms specific repository creation script. 
# Processed by install.RepositoryInstall
# 
# This file must have the following format:
# 1) The first line of the file must contain a count of the sql stmts
#    included in the line. This info is used to update a progress bar
#    as the sql file is being processed. This line should be in the
#    format:
#        sqlcount = <n>
#    where <n> represents the # of statements contained in the file
#
# 2) Each sql statement must be terminated by a ';' as the first char
#    of a new line. Everything else on the terminator line is ignored.
#
# 3) Comments are denoted by a '#' as the first char of a line. Everything
#    on a comment line is ignored
#
# Parameter Values (see install.RepositoryInstall for all DBs):
#
# Parm    Oracle Value        MSSQL Value
# ----    ------------        ----------------------------------------------
# {0}     decimal(10)         integer
# {1}     decimal(5)          smallint
# {2}     raw(20)             varbinary(20)
# {3}     date                datetime
# {4}     long                text
# {5}     long raw            image
# {6}     <empty>             case_mask varbinary(8) default 0x00 not null,
# {7}     <empty>             case_mask varbinary(64) default 0x00 not null,
# {8}     sysdate             current_timestamp
# {9}     '20'                ' '
# {10}    <empty>             , case_mask
# {11}    <empty>             0x00,
# {12}    float               float
# {13}    raw(60)             varbinary(60)
# {14}    <empty>             t_res_attribute
# {15}    modify              alter column
# {16}    <empty>             <empty>
# {17}    <empty>             not null
# {18}    column              column
# {19}    '4e'                0x4e
# {20}    partition           partition
# {21}    default null        null
# {22}    <empty>             <empty>
# {23}    varchar(256)        varchar(256)
# {24}    varchar(512)        varchar(512)
# {104}   RTRIM (CAST (g.group_id as char(128)))
#
# Parm    Interbase Value                     Sybase Value
# ----    ---------------                     ------------
# {0}     integer                             integer
# {1}     smallint                            smallint
# {2}     varchar(20) character set octets    varbinary(21)
# {3}     timestamp                           datetime
# {4}     varchar(32000)                      text
# {5}     varchar(32000) character set octets image
# {6}     <empty>                             case_mask varbinary(9) default 0x00 not null,
# {7}     <empty>                             case_mask varbinary(65) default 0x00 not null,
# {8}     current_timestamp                   getdate()
# {9}     ' '                                 ' '
# {10}    <empty>                             , case_mask
# {11}    <empty>                             0x00,
# {12}    double precision                    float
# {13}    varchar(60) character set octets    varbinary(61)
# {14}    <empty>                             t_res_stat_dir.
# {15}    alter column                        modify
# {16}    type                                <empty>
# {17}    <empty>                             not null
# {18}    <empty>                             <empty>
# {19}    'N'                                 0x4e
# {20}    partition                           \"partition\"
# {21}    default null                        null
# {22}    <empty>                             <empty>
# {23}    varchar(256)                        varchar(256)
# {24}    varchar(512)                        varchar(512)
# {104} 						              convert(varchar, g.group_id)  
#
# Parm    UDB Values
# ----    ----------       
# {0}     integer        
# {1}     smallint
# {2}     varchar(20) for bit data            
# {3}     timestamp
# {4}     long varchar                
# {5}     long varchar for bit data            
# {6}     <empty>             
# {7}     <empty>             
# {8}     current timestamp             
# {9}     ' '                
# {10}    <empty>             
# {11}    <empty>             
# {12}    float               
# {13}    varchar(60) for bit data
# {14}    <empty>            
# {15}    alter              
# {16}    set data type             
# {17}    <empty>             
# {18}    <empty>              
# {19}    '4e'                
# {20}    \"partition\"
# {21}    <empty>        
# {22}    IN database.tablespace             
# {23}    varchar(256) or varchar(255)        
# {24}    varchar(512) or varchar(255)        
# {104}   RTRIM (CAST (g.group_id as char(128)))
###########################################################################

drop table t_stat_file
;

create table t_stat_file
  (filesystem_id     {0}          not null,
   statistic_id      {0}          not null,
   statistic_type    char(1)      not null,
   path              {24}         not null,
   name              {24}         not null,
   owner_id          {0}          not null,
   group_id          {0}          not null,
   logical_size      decimal(16)  not null,
   physical_size     decimal(13)  not null,
   creation_time     {3}          not null,
   access_time       {3}          not null,
   modification_time {3}          not null,
   attributes        {0}          not null) {22} 
;

create index t_stat_file_ix
  on t_stat_file (filesystem_id, statistic_id,
                             statistic_type)
;

alter table t_res_volgroup
  add nbrdisks {0} default -1 not null
;

alter table t_res_volgroup
  add discovered_time {3} {21} 
;

alter table t_res_volgroup
  add last_probe_time {3} {21}
;

create table t_group2res
   (resource_id        {0}            not null,
    group_id           {0}            not null,
    parent_id          {0}            not null,
    res_type           {1}            not null,
    res_indicator      {1}            not null,
    ancestors          varchar(128)   not null ) {22}
;

create unique index t_group2res_ix1
  on t_group2res (resource_id, group_id, ancestors, res_indicator)
;

insert into t_group2res (resource_id, group_id, parent_id, res_type, res_indicator, ancestors) 
	select f.filesystem_id, f.group_id, f.group_id, 3, 1, {104} 
	from t_res_filesystem f, t_group g 
	where (f.group_id = g.group_id)
;

insert into t_group2res (resource_id, group_id, parent_id, res_type, res_indicator, ancestors) 
	select c.computer_id, c.group_id, c.group_id, 1, 1, {104} 
	from t_stat_computer c, t_group g 
	where (c.group_id = g.group_id)
;

insert into t_group2res (resource_id, group_id, parent_id, res_type, res_indicator, ancestors) 
	select t.tablespace_id, t.group_id, t.group_id, 49, 1, {104} 
	from t_res_tablespace t, t_group g 
	where (t.group_id = g.group_id)
;

create table t_group2mask
   (group_id           {0}            not null,
    masktype           {1}            not null,
    mask               varchar(128)   not null) {22}
;

create unique index t_group2mask_ix1
  on t_group2mask (group_id, mask)
;

create view t_view_group2res 
   (resource_id, group_id, res_type) as 
   (select distinct resource_id, group_id, res_type 
   from   t_group2res 
   where res_indicator in (5,2,1))
;



####################################################################
# Add new columns to T_STAT_COMPUTER for agent clustering support
####################################################################

alter table t_stat_computer
  add   cluster_id {0} default 0 not null
;

alter table t_stat_computer
  add   node_hosted_id {0} default 0 not null
;

###########################################################################
# Add columns to T_CONFIG_SETTINGS for second SNMP trap destination
###########################################################################

alter table t_config_settings
  add snmp2_server      varchar(128) default ' ' not null
;

alter table t_config_settings
  add snmp2_port        {0}          default 0 not null
;

alter table t_config_settings
  add snmp2_community   varchar(64)  default ' ' not null
;

####################################################################
# Create T_RES_CLUSTER table for agent clustering support
####################################################################

create table t_res_cluster
  (cluster_id        {0}            not null,
   cluster_name      varchar(64)    not null,
   domain_name       varchar(128)   not null,
   ip_address        varchar(15)    not null,
   domain_id         {1}            not null,
   cluster_type      {1}            not null,
   discovered_time   {3}            not null) {22}
;

create unique index t_res_clust_ix1
  on t_res_cluster (cluster_name, domain_name)
;

create unique index t_res_clust_ix2
  on t_res_cluster (cluster_id)
;
#### Newport Value report - temporary files profile
insert into t_stat_definition values
   (31, 'Tivoli', '{103}',
    '{102}',
    0, 'Tivoli', '2', -1,
    0, 0, 0, 0, 0, 0, '1', '0', '0', '0', '0', '0', '0', '0', 7, 4, 2, {9})
;
insert into t_res_attribute values
   (31, 18, 04, 15, '0')
;

####################################################################
# Add new column for usable lun capacity (system wide dashboard)
####################################################################
alter table t_stat_totals
  add usable_lun_cap decimal(13) default -1 not null
; 